@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -74,49 +73,46 @@ def DualClassWindow ():
 	NewClassId = 0
 	
 	# set up our main window
-	DCMainWindow = GemRB.LoadWindow (5)
+	DCMainWindow = GemRB.LoadWindow (5, "GUIREC")
 
 	# done button (off)
 	DCMainDoneButton = DCMainWindow.GetControl (2)
 	DCMainDoneButton.SetText (11973)
-	DCMainDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCMainDonePress)
+	DCMainDoneButton.OnPress (DCMainDonePress)
 	DCMainDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
-	DCMainDoneButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 	# cancel button (on)
 	DCMainCancelButton = DCMainWindow.GetControl (1)
 	DCMainCancelButton.SetText (13727)
-	DCMainCancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCMainCancelPress)
+	DCMainCancelButton.OnPress (DCMainCancelPress)
 	DCMainCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
-	DCMainCancelButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCMainCancelButton.MakeEscape ()
 
 	# class button (on)
 	DCMainClassButton = DCMainWindow.GetControl (3)
 	DCMainClassButton.SetText (11959)
-	DCMainClassButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCMainClassPress)
+	DCMainClassButton.OnPress (DCMainClassPress)
 	DCMainClassButton.SetState (IE_GUI_BUTTON_ENABLED)
-	DCMainClassButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCMainClassButton.MakeDefault()
 
 	# skills button (off)
 	DCMainSkillsButton = DCMainWindow.GetControl (4)
 	DCMainSkillsButton.SetText (17372)
-	DCMainSkillsButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCMainSkillsPress)
+	DCMainSkillsButton.OnPress (DCMainSkillsPress)
 	DCMainSkillsButton.SetState (IE_GUI_BUTTON_DISABLED)
-	DCMainSkillsButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 	# back button (on)
 	DCMainBackButton = DCMainWindow.GetControl (5)
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		DCMainBackButton.SetText (15416)
-	DCMainBackButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCMainBackPress)
+	DCMainBackButton.OnPress (DCMainBackPress)
 	DCMainBackButton.SetState (IE_GUI_BUTTON_ENABLED)
-	DCMainBackButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 	# picture of character
 	DCMainPictureButton = DCMainWindow.GetControl (6)
 	DCMainPictureButton.SetState (IE_GUI_BUTTON_LOCKED)
 	DCMainPictureButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)
-	DCMainPictureButton.SetPicture (GemRB.GetPlayerPortrait (pc, 0), "NOPORTMD")
+	DCMainPictureButton.SetPicture (GemRB.GetPlayerPortrait (pc, 0)["Sprite"], "NOPORTMD")
 
 	# text area warning
 	DCTextArea = DCMainWindow.GetControl (7)
@@ -149,8 +145,8 @@ def DumpUnusableItems (pc):
 	"""Dumps everything in the inventory that is now unusable."""
 
 	SlotTypes = [ SLOT_ARMOUR, SLOT_SHIELD, SLOT_HELM, SLOT_RING, SLOT_CLOAK, SLOT_BOOT, SLOT_AMULET, SLOT_GLOVE, SLOT_BELT, SLOT_ITEM, SLOT_WEAPON, SLOT_QUIVER ]
-	for type in SlotTypes:
-		Slots = GemRB.GetSlots (pc, type)
+	for stype in SlotTypes:
+		Slots = GemRB.GetSlots (pc, stype)
 		if not len(Slots):
 			# nothing there
 			continue
@@ -261,8 +257,7 @@ def DCMainDonePress ():
 
 	# close our window
 	if DCMainWindow:
-		DCMainWindow.Unload ()
-	GUIREC.UpdateRecordsWindow()
+		DCMainWindow.Close ()
 	return
 
 def DCMainCancelPress ():
@@ -275,7 +270,7 @@ def DCMainCancelPress ():
 
 	# close our window
 	if DCMainWindow:
-		DCMainWindow.Unload ()
+		DCMainWindow.Close ()
 
 	return
 
@@ -318,7 +313,7 @@ def DCMainClassPress ():
 	GemRB.SetVar ("DCClass", DCClass)
 
 	# open the window
-	DCClassWindow = GemRB.LoadWindow (6)
+	DCClassWindow = GemRB.LoadWindow (6, "GUIREC")
 
 	# string refs for the given classes
 	DCClassStrings = []
@@ -329,10 +324,9 @@ def DCMainClassPress ():
 	for i in range (6):
 		# get the button and associate it with the correct var
 		DCClassButton = DCClassWindow.GetControl (i+1)
-		DCClassButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCClassSelect)
+		DCClassButton.OnPress (DCClassSelect)
 		DCClassButton.SetVarAssoc ("DCClass", i)
 		DCClassButton.SetText (DCClassStrings[i])
-		DCClassButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
 		# enable only if we can dual into the given class
 		if CanDualInto (i):
@@ -343,16 +337,16 @@ def DCMainClassPress ():
 	# done button (off)
 	DCClassDoneButton = DCClassWindow.GetControl (8)
 	DCClassDoneButton.SetText (11973)
-	DCClassDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCClassDonePress)
+	DCClassDoneButton.OnPress (DCClassDonePress)
 	DCClassDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
-	DCClassDoneButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCClassDoneButton.MakeDefault()
 
 	# back button (on)
 	DCClassBackButton = DCClassWindow.GetControl (7)
 	DCClassBackButton.SetText (15416)
-	DCClassBackButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCClassBackPress)
+	DCClassBackButton.OnPress (DCClassBackPress)
 	DCClassBackButton.SetState (IE_GUI_BUTTON_ENABLED)
-	DCClassBackButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCClassBackButton.MakeEscape ()
 
 	# setup the text area with default text
 	DCClassTextArea = DCClassWindow.GetControl (9)
@@ -405,7 +399,7 @@ def DCClassDonePress ():
 
 	# unload our class selection window
 	if DCClassWindow:
-		DCClassWindow.Unload ()
+		DCClassWindow.Close ()
 	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	# enable the skills button and disable the class selection button
@@ -458,7 +452,7 @@ def DCClassBackPress ():
 	"""Unloads the class selection window without making any changes."""
 	# close the class window
 	if DCClassWindow:
-		DCClassWindow.Unload ()
+		DCClassWindow.Close ()
 	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
@@ -482,9 +476,9 @@ def DCOpenProfsWindow ():
 	global DCProfsWindow, DCProfsDoneButton
 
 	# load up our window and set some basic variables
-	DCProfsWindow = GemRB.LoadWindow (15)
+	DCProfsWindow = GemRB.LoadWindow (15, "GUIREC")
 	NewClassId = CommonTables.Classes.GetValue (ClassName, "ID", GTV_INT)
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		LUProfsSelection.SetupProfsWindow (pc, \
 			LUProfsSelection.LUPROFS_TYPE_DUALCLASS, DCProfsWindow, DCProfsRedraw, classid=NewClassId)
 	else:
@@ -495,15 +489,15 @@ def DCOpenProfsWindow ():
 	# setup the done and cancel
 	DCProfsDoneButton = DCProfsWindow.GetControl (76)
 	DCProfsDoneButton.SetText (11973)
-	DCProfsDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCProfsDonePress)
+	DCProfsDoneButton.OnPress (DCProfsDonePress)
 	DCProfsDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
-	DCProfsDoneButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCProfsDoneButton.MakeDefault()
 
 	DCProfsCancelButton = DCProfsWindow.GetControl (77)
 	DCProfsCancelButton.SetText (13727)
-	DCProfsCancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DCProfsCancelPress)
+	DCProfsCancelButton.OnPress (DCProfsCancelPress)
 	DCProfsCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
-	DCProfsCancelButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DCProfsCancelButton.MakeEscape ()
 
 	# show the window and draw away
 	DCProfsWindow.ShowModal (MODAL_SHADOW_GRAY)
@@ -540,7 +534,6 @@ def DCProfsDonePress ():
 		GemRB.SetMemorizableSpellsCount (pc, SpellTable.GetValue (0, 0), IE_SPELL_TYPE_WIZARD, 0)
 		NewMageSpells = 1
 	if ClericTable != "*":
-		print "Setting PriestMask"
 		if not GemRB.HasResource(ClericTable, RES_2DA, 1):
 			ClericTable = "MXSPLPRS" # iwd1 doesn't have a DRUIDSPELL column in the table
 		# make sure we can cast spells at this level (paladins)
@@ -564,7 +557,7 @@ def DCProfsDonePress ():
 
 	# close out the profs window (don't assign yet!)
 	if DCProfsWindow:
-		DCProfsWindow.Unload ()
+		DCProfsWindow.Close ()
 	return
 
 def DCProfsCancelPress ():
@@ -572,7 +565,7 @@ def DCProfsCancelPress ():
 
 	# close out the profs window and go back a step
 	if DCProfsWindow:
-		DCProfsWindow.Unload ()
+		DCProfsWindow.Close ()
 	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	DCMainBackPress ()
@@ -582,12 +575,12 @@ def OpenSkillsWindow ():
 	"""Opens the skills selection window.
 
 	This will allow the selection of thief skills for appropriate classes, or
-	auto-assign them for classes that recieve skills based on level."""
+	auto-assign them for classes that receive skills based on level."""
 
 	global DCSkillsWindow, DCSkillsDoneButton
 
-	DCSkillsWindow = GemRB.LoadWindow (7)
-	if GameCheck.IsBG2():
+	DCSkillsWindow = GemRB.LoadWindow (7, "GUIREC")
+	if GameCheck.IsBG2OrEE ():
 		LUSkillsSelection.SetupSkillsWindow (pc, \
 			LUSkillsSelection.LUSKILLS_TYPE_DUALCLASS, DCSkillsWindow, DCSkillsRedraw, classid=NewClassId)
 	else:
@@ -596,19 +589,20 @@ def OpenSkillsWindow ():
 			DCSkillsRedraw, [0,0,0], [1,1,1], NewClassId, False)
 
 	#just go back if we can't assign skills
-	if GemRB.GetVar ("SkillPointsLeft") <= 0:
+	if not GemRB.GetVar ("SkillPointsLeft"):
+		DCSkillsWindow.Close ()
 		return
 
 	# setup the back and done buttons
 	BackButton = DCSkillsWindow.GetControl(24)
 	BackButton.SetText(15416)
-	BackButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-	BackButton.SetEvent(IE_GUI_BUTTON_ON_PRESS,DCSkillsBackPress)
+	BackButton.MakeEscape()
+	BackButton.OnPress (DCSkillsBackPress)
 
 	DCSkillsDoneButton = DCSkillsWindow.GetControl(25)
 	DCSkillsDoneButton.SetText(11973)
-	DCSkillsDoneButton.SetFlags(IE_GUI_BUTTON_DEFAULT,OP_OR)
-	DCSkillsDoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, DCSkillsDonePress)
+	DCSkillsDoneButton.MakeDefault()
+	DCSkillsDoneButton.OnPress (DCSkillsDonePress)
 	DCSkillsDoneButton.SetState(IE_GUI_BUTTON_DISABLED)
 
 	# setup the default text area
@@ -617,7 +611,6 @@ def OpenSkillsWindow ():
 
 	DCSkillsWindow.ShowModal (MODAL_SHADOW_GRAY)
 	DCSkillsRedraw ()
-	GemRB.SetRepeatClickFlags(GEM_RK_DISABLE, OP_NAND)
 	return
 
 def DCSkillsRedraw ():
@@ -636,7 +629,7 @@ def DCSkillsBackPress ():
 	"""Reverts all changes to this points."""
 
 	if DCSkillsWindow:
-		DCSkillsWindow.Unload ()
+		DCSkillsWindow.Close ()
 	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
 	LUSkillsSelection.SkillsNullify ()
 	DCMainBackPress ()
@@ -646,7 +639,5 @@ def DCSkillsDonePress ():
 	"""Closes the skills selection window."""
 
 	if DCSkillsWindow:
-		DCSkillsWindow.Unload ()
-	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
-	GemRB.SetRepeatClickFlags (GEM_RK_DISABLE, OP_OR)
+		DCSkillsWindow.Close ()
 	return
