@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -28,20 +27,16 @@ import GUICommon
 import CommonTables
 import GUICommonWindows
 import Spellbook
+from GUICommon import BindControlCallbackParams
 from GUIDefines import *
 from ie_stats import *
 from ie_spells import LS_MEMO
 
 MageWindow = None
-MageSpellInfoWindow = None
 MageSpellLevel = 0
-MageSpellUnmemorizeWindow = None
-PortraitWindow = None
-OptionsWindow = None
-OldPortraitWindow = None
-OldOptionsWindow = None
+
 # bg2 stuff for handling triggers and contingencies
-BookType = None
+Sorcerer = None
 OtherWindow = None
 Exclusions = None
 ContCond = None
@@ -49,128 +44,117 @@ ContTarg = None
 SpellType = None
 Level = 1
 
-def OpenMageWindow ():
-	global MageWindow, OptionsWindow, PortraitWindow, OtherWindow, MageSpellInfoWindow
-	global OldPortraitWindow, OldOptionsWindow
+FlashResRef = "FLASHBR" if GameCheck.IsBG2OrEE () else "FLASH"
 
-	if GUICommon.CloseOtherWindow (OpenMageWindow):
-		CloseMageSpellUnmemorizeWindow ()
-		if MageSpellInfoWindow:
-			MageSpellInfoWindow.Unload ()
-			MageSpellInfoWindow = None
-		if OtherWindow:
-			OtherWindow.Unload ()
-			OtherWindow = None
-		if MageWindow:
-			MageWindow.Unload ()
-		if OptionsWindow:
-			OptionsWindow.Unload ()
-		if PortraitWindow:
-			PortraitWindow.Unload ()
-
-		MageWindow = None
-		GemRB.SetVar ("OtherWindow", -1)
-		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE)
-		GemRB.UnhideGUI ()
-		GUICommonWindows.PortraitWindow = OldPortraitWindow
-		OldPortraitWindow = None
-		GUICommonWindows.OptionsWindow = OldOptionsWindow
-		OldOptionsWindow = None
-		GUICommonWindows.SetSelectionChangeHandler (None)
-		return
-
-	GemRB.HideGUI ()
-	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE)
-
-	GemRB.LoadWindowPack ("GUIMG", 640, 480)
-
-	#saving the original portrait window
-	OldOptionsWindow = GUICommonWindows.OptionsWindow
-	OptionsWindow = GemRB.LoadWindow (0)
-	GUICommonWindows.MarkMenuButton (OptionsWindow)
-	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 0, OpenMageWindow)
-	OptionsWindow.SetFrame ()
-	OldPortraitWindow = GUICommonWindows.PortraitWindow
-	PortraitWindow = GUICommonWindows.OpenPortraitWindow (0)
-
-	GUICommonWindows.SetSelectionChangeHandler (SetupMageWindow)
-	SetupMageWindow()
-	return
-
-def SetupMageWindow ():
-	global MageWindow
-	global BookType
-
-	pc = GemRB.GameGetSelectedPCSingle ()
-	BookType = 0
+def ToggleSpellWindow (btn):
+	global Sorcerer
 	# added game check, since although sorcerers have almost no use for their spellbook, there's no other way to quickly check spell descriptions
-	if GameCheck.IsBG2() and Spellbook.HasSorcererBook (pc):
-		BookType = 1
-
-	if MageWindow:
-		MageWindow.Unload()
-		MageWindow = None
-
-	if BookType:
-		MageWindow = Window = GemRB.LoadWindow (8)
+	pc = GemRB.GameGetSelectedPCSingle ()
+	Sorcerer = GameCheck.IsBG2OrEE () and Spellbook.HasSorcererBook (pc) > 0
+	
+	ToggleSpellWindow.Args = btn
+	
+	if Sorcerer:
+		ToggleSorcererWindow (btn)
 	else:
-		MageWindow = Window = GemRB.LoadWindow (2)
-	GemRB.SetVar ("OtherWindow", MageWindow.ID)
+		ToggleMageWindow (btn)
 
-	Button = Window.GetControl (1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MagePrevLevelPress)
+def InitMageWindow (window):
+	global MageWindow
+	MageWindow = window
 
-	Button = Window.GetControl (2)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MageNextLevelPress)
+	Button = MageWindow.GetControl (1)
+	Button.OnPress (MagePrevLevelPress)
 
-	#unknown usage
-	if Window.HasControl (55):
-		Button = Window.GetControl (55)
-		Button.SetState (IE_GUI_BUTTON_LOCKED)
+	Button = MageWindow.GetControl (2)
+	Button.OnPress (MageNextLevelPress)
+
+	# contingencies
+	Button = MageWindow.GetControl (55)
+	if GameCheck.IsBG2EE ():
+		Button = MageWindow.GetControl (59)
+	if Button:
+		Button.OnPress (OpenContingenciesWindow)
+		pc = GemRB.GameGetSelectedPCSingle ()
+		if GemRB.CountEffects (pc, "CastSpellOnCondition", -1, -1) + GemRB.CountEffects (pc, "Sequencer:Store", -1, -1) == 0:
+			Button.SetState (IE_GUI_BUTTON_DISABLED)
+			Button.SetText (None)
+		else:
+			Button.SetState (IE_GUI_BUTTON_ENABLED)
+			Button.SetText (34586)
 
 	#setup level buttons
-	if GameCheck.IsBG2():
+	spellLevelOffset = None
+	if GameCheck.IsBG2 ():
+		spellLevelOffset = 56
+	elif GameCheck.IsBG2EE ():
+		spellLevelOffset = 61
+	if spellLevelOffset:
 		for i in range (9):
-			Button = Window.GetControl (56 + i)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RefreshMageLevel)
+			Button = MageWindow.GetControl (spellLevelOffset + i)
+			Button.OnPress (RefreshMageLevel)
 			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 			Button.SetVarAssoc ("MageSpellLevel", i)
 
 	# Setup memorized spells buttons
-	if not BookType:
+	if not Sorcerer:
 		for i in range (12):
-			Button = Window.GetControl (3 + i)
-			Button.SetBorder (0,0,0,0,0,0,0,0,64,0,1)
+			Button = MageWindow.GetControl (3 + i)
+			color = {'r' : 0, 'g' : 0, 'b' :0, 'a' : 64}
+			Button.SetBorder (0,color,0,1)
 			Button.SetSprites ("SPELFRAM",0,0,0,0,0)
-			Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_PLAYONCE | IE_GUI_BUTTON_PLAYALWAYS, OP_OR)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
+			Button.SetAnimation (None)
+			Button.SetVarAssoc ("Memorized", i)
+
+	# reposition from a 4x3 grid to the bottom or top
+	# for mages that's the memoed spells, for sorcerers the known
+	if GameCheck.IsBG2EE ():
+		for i in range (12):
+			Button = MageWindow.GetControl (3 + i + 24 * int(Sorcerer))
+			row = i // 4
+			col = i % 4
+			width = 54 + 8 * (1 + int(Sorcerer)) # slot + padding
+			# don't bother transposing 4x3 grid to 3x4 grid
+			if Sorcerer:
+				frame = Button.GetFrame ()
+				x = frame["x"] - (314 - 166)
+				y = frame["y"] - (536 - 234)
+			else:
+				x = 100 + row * width * 4 + col * width + 66 * int(Sorcerer)
+				y = 644
+			Button.SetPos (x, y)
+			# the bam isn't centered, so let's cheat
+			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+
+		# some extra slot button, sorcerer button and a label on a button
+		MageWindow.DeleteControl (51)
+		if Sorcerer:
+			MageWindow.DeleteControl (59)
+		else:
+			MageWindow.DeleteControl (0x10000000 + 51)
 
 	# Setup book spells buttons
 	for i in range (GUICommon.GetGUISpellButtonCount()):
-		Button = Window.GetControl (27 + i)
-		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PLAYONCE | IE_GUI_BUTTON_PLAYALWAYS, OP_OR)
+		Button = MageWindow.GetControl (27 + i)
+		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
+		Button.SetValue (i)
 
-	UpdateMageWindow ()
-	OptionsWindow.SetVisible (WINDOW_VISIBLE)
-	Window.SetVisible (WINDOW_FRONT)
-	PortraitWindow.SetVisible (WINDOW_VISIBLE)
+	UpdateMageWindow (MageWindow)
 	return
 
-def UpdateMageWindow ():
-	global MageMemorizedSpellList, MageKnownSpellList
-
-	MageMemorizedSpellList = []
-	MageKnownSpellList = []
-
-	Window = MageWindow
+def UpdateMageWindow (MageWindow):
 	pc = GemRB.GameGetSelectedPCSingle ()
-	type = IE_SPELL_TYPE_WIZARD
+	spelltype = IE_SPELL_TYPE_WIZARD
 	level = MageSpellLevel
-	max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, type, level, 1)
+	max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, spelltype, level, 1)
+	
+	CantCast = CommonTables.ClassSkills.GetValue (GUICommon.GetClassRowName (pc), "MAGESPELL") == "*"
+	GUICommon.AdjustWindowVisibility (MageWindow, pc, CantCast)
 
-	Label = Window.GetControl (0x10000032)
-	if GameCheck.IsBG2():
+	Label = MageWindow.GetControl (0x10000032)
+	if GameCheck.IsBG2OrEE ():
 		GemRB.SetToken ("SPELLLEVEL", str(level + 1))
 		Label.SetText (10345)
 	else:
@@ -178,83 +162,97 @@ def UpdateMageWindow ():
 		Label.SetText (12137)
 
 	Name = GemRB.GetPlayerName (pc, 0)
-	Label = Window.GetControl (0x10000035)
+	Label = MageWindow.GetControl (0x10000035)
 	Label.SetText (Name)
 
-	known_cnt = GemRB.GetKnownSpellsCount (pc, type, level)
-	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, False)
-	true_mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, True)
-	if not BookType:
+	known_cnt = GemRB.GetKnownSpellsCount (pc, spelltype, level)
+	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
+	true_mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, True)
+	if not Sorcerer:
 		for i in range (12):
-			Button = Window.GetControl (3 + i)
-			Button.SetAnimation ("")
+			Button = MageWindow.GetControl (3 + i)
+
 			if i < mem_cnt:
-				ms = GemRB.GetMemorizedSpell (pc, type, level, i)
+				ms = GemRB.GetMemorizedSpell (pc, spelltype, level, i)
 				Button.SetSpellIcon (ms['SpellResRef'], 0)
-				Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-				Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_PLAYONCE | IE_GUI_BUTTON_PLAYALWAYS, OP_OR)
+				if not GameCheck.IsBG2EE ():
+					Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
 				if ms['Flags']:
-					Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMageSpellUnmemorizeWindow)
+					Button.OnPress (OpenMageSpellUnmemorizeWindow)
 				else:
-					Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageUnmemorizeSpell)
-				Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenMageSpellInfoWindow)
-				MageMemorizedSpellList.append (ms['SpellResRef'])
-				Button.SetVarAssoc ("SpellButton", i)
-				Button.EnableBorder (0, ms['Flags'] == 0)
+					Button.OnPress (OnMageUnmemorizeSpell)
 				spell = GemRB.GetSpell (ms['SpellResRef'])
-				if not spell:
-					print "Missing memorised spell!", ms['SpellResRef']
-					continue
+				Button.OnRightPress(BindControlCallbackParams(OpenMageSpellInfoWindow, spell, Button.VarName))
+				Button.EnableBorder (0, ms['Flags'] == 0)
 				Button.SetTooltip (spell['SpellName'])
 			else:
 				if i < max_mem_cnt:
-					Button.SetFlags (IE_GUI_BUTTON_NORMAL | IE_GUI_BUTTON_PLAYONCE | IE_GUI_BUTTON_PLAYALWAYS, OP_SET)
+					Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
 				else:
 					Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
-				Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
+				Button.OnPress (None)
+				Button.OnRightPress (None)
 				Button.SetTooltip ('')
 				Button.EnableBorder (0, 0)
 	else:
-		label = Window.GetControl (0x10000040)
+		label = MageWindow.GetControl (0x10000040)
 		if known_cnt:
 			# we give sorcerers all charges for all the spells, so some extra math is needed
-			label.SetText (GemRB.GetString(61256) + " " + str(true_mem_cnt/known_cnt) + "/" + str(max_mem_cnt))
+			label.SetText (GemRB.GetString(61256) + " " + str(true_mem_cnt // known_cnt) + "/" + str(max_mem_cnt))
 		else:
 			label.SetText ("")
+	
+	btncount = GUICommon.GetGUISpellButtonCount()
+	i = 0
+	for i in range (known_cnt):
+		Button = MageWindow.GetControl (27 + i)
+		
+		ks = GemRB.GetKnownSpell (pc, spelltype, level, i)
+		Button.SetSpellIcon (ks['SpellResRef'], 0)
+		Button.OnPress (OnMageMemorizeSpell)
+		spell = GemRB.GetSpell (ks['SpellResRef'])
+		spell['CurrentIndex'] = i # save it for later
+		Button.OnRightPress (BindControlCallbackParams(OpenMageSpellInfoWindow, spell, Button.VarName))
+		Button.SetTooltip (spell['SpellName'])
 
-	for i in range (GUICommon.GetGUISpellButtonCount()):
-		Button = Window.GetControl (27 + i)
-		Button.SetAnimation ("")
-		if i < known_cnt:
-			ks = GemRB.GetKnownSpell (pc, type, level, i)
-			Button.SetSpellIcon (ks['SpellResRef'], 0)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageMemorizeSpell)
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenMageSpellInfoWindow)
-			MageKnownSpellList.append (ks['SpellResRef'])
-			Button.SetVarAssoc ("SpellButton", 100 + i)
-			spell = GemRB.GetSpell (ks['SpellResRef'])
-			if not spell:
-				print "Missing known spell!", ms['SpellResRef']
-				continue
-			Button.SetTooltip (spell['SpellName'])
-		else:
-			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
-			Button.SetTooltip ('')
-			Button.EnableBorder (0, 0)
-
-	CantCast = CommonTables.ClassSkills.GetValue (GUICommon.GetClassRowName (pc), "MAGESPELL") == "*"
-	GUICommon.AdjustWindowVisibility (Window, pc, CantCast)
+	if known_cnt == 0: i = -1
+	for i in range (i + 1, btncount):
+		Button = MageWindow.GetControl (27 + i)
+		
+		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
+		Button.OnPress (None)
+		Button.OnRightPress (None)
+		Button.SetTooltip ('')
+		Button.EnableBorder (0, 0)
+	
 	return
 
+def MageSelectionChanged (oldwin):
+	global Sorcerer
+	# added game check, since although sorcerers have almost no use for their spellbook, there's no other way to quickly check spell descriptions
+	
+	UpdateMageWindow(oldwin)
+	
+	pc = GemRB.GameGetSelectedPCSingle ()
+	Sorcerer = GameCheck.IsBG2OrEE () and Spellbook.HasSorcererBook (pc)
+
+	if Sorcerer:
+		OpenSorcererWindow (ToggleSpellWindow.Args)
+	else:
+		OpenMageWindow (ToggleSpellWindow.Args)
+
+ToggleMageWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIMG", GUICommonWindows.ToggleWindow, InitMageWindow, MageSelectionChanged, True)
+OpenMageWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIMG", GUICommonWindows.OpenWindowOnce, InitMageWindow, MageSelectionChanged, True)
+
+ToggleSorcererWindow = GUICommonWindows.CreateTopWinLoader(8, "GUIMG", GUICommonWindows.ToggleWindow, InitMageWindow, MageSelectionChanged, True)
+OpenSorcererWindow = GUICommonWindows.CreateTopWinLoader(8, "GUIMG", GUICommonWindows.OpenWindowOnce, InitMageWindow, MageSelectionChanged, True)
+
 def MagePrevLevelPress ():
 	global MageSpellLevel
 
 	if MageSpellLevel > 0:
 		MageSpellLevel = MageSpellLevel - 1
-		UpdateMageWindow ()
+		UpdateMageWindow (MageWindow)
 	return
 
 def MageNextLevelPress ():
@@ -262,54 +260,39 @@ def MageNextLevelPress ():
 
 	if MageSpellLevel < 8:
 		MageSpellLevel = MageSpellLevel + 1
-		UpdateMageWindow ()
+		UpdateMageWindow (MageWindow)
 	return
 
 def RefreshMageLevel ():
 	global MageSpellLevel
 
 	MageSpellLevel = GemRB.GetVar ("MageSpellLevel")
-	UpdateMageWindow ()
+	UpdateMageWindow (MageWindow)
 	return
 
-def OpenMageSpellInfoWindow ():
-	global MageSpellInfoWindow
-
-	if MageSpellInfoWindow != None:
-		if MageSpellInfoWindow:
-			MageSpellInfoWindow.Unload ()
-		MageSpellInfoWindow = None
-		return
-
-	MageSpellInfoWindow = Window = GemRB.LoadWindow (3)
+def OpenMageSpellInfoWindow (spell, kind):
+	Window = GemRB.LoadWindow (3, "GUIMG")
 
 	#back
 	Button = Window.GetControl (5)
 	Button.SetText (15416)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMageSpellInfoWindow)
+	Button.OnPress (Window.Close)
 
 	#erase
-	index = GemRB.GetVar ("SpellButton")
-	if GameCheck.HasTOB() or Window.HasControl(6):
-		Button = Window.GetControl (6)
-		if index < 100:
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+	Button = Window.GetControl (6)
+	if Button:
+		if kind == "Memorized" or Sorcerer:
+			Button.OnPress (None)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		else:
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMageSpellRemoveWindow)
+			Button.OnPress (lambda: OpenMageSpellRemoveWindow(Window, spell['CurrentIndex']))
 			Button.SetText (63668)
-	if index < 100:
-		ResRef = MageMemorizedSpellList[index]
-	else:
-		ResRef = MageKnownSpellList[index - 100]
-
-	spell = GemRB.GetSpell (ResRef)
 
 	Label = Window.GetControl (0x0fffffff)
 	Label.SetText (spell['SpellName'])
 
 	Button = Window.GetControl (2)
-	Button.SetSpellIcon (ResRef, 1)
+	Button.SetSpellIcon (spell['SpellResRef'], 1)
 
 	Text = Window.GetControl (3)
 	Text.SetText (spell['SpellDesc'])
@@ -317,42 +300,33 @@ def OpenMageSpellInfoWindow ():
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
-def OnMageMemorizeSpell ():
+def OnMageMemorizeSpell (btn):
 	pc = GemRB.GameGetSelectedPCSingle ()
 	level = MageSpellLevel
-	type = IE_SPELL_TYPE_WIZARD
+	spelltype = IE_SPELL_TYPE_WIZARD
+	Window = btn.Window
 
-	index = GemRB.GetVar ("SpellButton") - 100
-	blend = 1
-	if GameCheck.IsBG2():
-		blend = 0
+	def Complete():
+		mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
+		AnimBtn = Window.GetControl(mem_cnt + 2)
+		AnimBtn.SetAnimation(FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
+		AnimBtn.OnAnimEnd(lambda: UpdateMageWindow(Window))
+		UpdateMageWindow(Window)
 
-	if GemRB.MemorizeSpell (pc, type, level, index):
-		UpdateMageWindow ()
+	index = btn.Value
+	if GemRB.MemorizeSpell (pc, spelltype, level, index):
 		GemRB.PlaySound ("GAM_24")
 		Button = MageWindow.GetControl(index + 27)
-		Button.SetAnimation ("FLASH", 0, blend)
-		mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, False)
-		Button = MageWindow.GetControl(mem_cnt + 2)
-		Button.SetAnimation ("FLASH", 0, blend)
+		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
+		Button.OnAnimEnd(Complete)
+
 	return
 
-def CloseMageSpellUnmemorizeWindow ():
-	global MageSpellUnmemorizeWindow
-
-	if MageSpellUnmemorizeWindow:
-		MageSpellUnmemorizeWindow.Unload ()
-	MageSpellUnmemorizeWindow = None
-	return
-
-def OpenMageSpellRemoveWindow ():
-	global MageSpellUnmemorizeWindow
-
-	if GameCheck.IsBG2():
-		MageSpellUnmemorizeWindow = GemRB.LoadWindow (101)
+def OpenMageSpellRemoveWindow (parentWin, spellIndex):
+	if GameCheck.IsBG2OrEE ():
+		Window = GemRB.LoadWindow (101, "GUIMG")
 	else:
-		MageSpellUnmemorizeWindow = GemRB.LoadWindow (5)
-	Window = MageSpellUnmemorizeWindow
+		Window = GemRB.LoadWindow (5, "GUIMG")
 
 	# "Are you sure you want to ....?"
 	TextArea = Window.GetControl (3)
@@ -361,26 +335,29 @@ def OpenMageSpellRemoveWindow ():
 	# Remove
 	Button = Window.GetControl (0)
 	Button.SetText (17507)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageRemoveSpell)
-	Button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	
+	def RemoveSpell (spellIndex):
+		OnMageRemoveSpell (spellIndex)
+		Window.Close()
+		parentWin.Close()
+	
+	Button.OnPress (lambda: RemoveSpell(spellIndex))
+	Button.MakeDefault()
 
 	# Cancel
 	Button = Window.GetControl (1)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseMageSpellUnmemorizeWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	Button.OnPress (Window.Close)
+	Button.MakeEscape()
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
-def OpenMageSpellUnmemorizeWindow ():
-	global MageSpellUnmemorizeWindow
-
-	if GameCheck.IsBG2():
-		MageSpellUnmemorizeWindow = GemRB.LoadWindow (101)
+def OpenMageSpellUnmemorizeWindow (btn):
+	if GameCheck.IsBG2OrEE ():
+		Window = GemRB.LoadWindow (101, "GUIMG")
 	else:
-		MageSpellUnmemorizeWindow = GemRB.LoadWindow (5)
-	Window = MageSpellUnmemorizeWindow
+		Window = GemRB.LoadWindow (5, "GUIMG")
 
 	# "Are you sure you want to ....?"
 	TextArea = Window.GetControl (3)
@@ -389,51 +366,45 @@ def OpenMageSpellUnmemorizeWindow ():
 	# Remove
 	Button = Window.GetControl (0)
 	Button.SetText (17507)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageUnmemorizeSpell)
-	Button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	
+	def Unmemorize (btn):
+		OnMageUnmemorizeSpell (btn)
+		Window.Close()
+
+	Button.SetValue (btn.Value)
+	Button.OnPress (Unmemorize)
+	Button.MakeDefault()
 
 	# Cancel
 	Button = Window.GetControl (1)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseMageSpellUnmemorizeWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	Button.OnPress (Window.Close)
+	Button.MakeEscape()
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
-def OnMageUnmemorizeSpell ():
-	if MageSpellUnmemorizeWindow:
-		CloseMageSpellUnmemorizeWindow()
-
+def OnMageUnmemorizeSpell (btn):
 	pc = GemRB.GameGetSelectedPCSingle ()
 	level = MageSpellLevel
-	type = IE_SPELL_TYPE_WIZARD
+	spelltype = IE_SPELL_TYPE_WIZARD
+	index = btn.Value
 
-	index = GemRB.GetVar ("SpellButton")
-	blend = 1
-	if GameCheck.IsBG2():
-		blend = 0
-
-	if GemRB.UnmemorizeSpell (pc, type, level, index):
-		UpdateMageWindow ()
+	if GemRB.UnmemorizeSpell (pc, spelltype, level, index):
 		GemRB.PlaySound ("GAM_44")
 		Button = MageWindow.GetControl(index + 3)
-		Button.SetAnimation ("FLASH", 0, blend)
+		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
+		Button.OnAnimEnd(lambda: UpdateMageWindow (MageWindow))
 	return
 
-def OnMageRemoveSpell ():
-	CloseMageSpellUnmemorizeWindow()
-	OpenMageSpellInfoWindow()
-
+def OnMageRemoveSpell (spellIndex):
 	pc = GemRB.GameGetSelectedPCSingle ()
 	level = MageSpellLevel
-	type = IE_SPELL_TYPE_WIZARD
-
-	index = GemRB.GetVar ("SpellButton")-100
+	spelltype = IE_SPELL_TYPE_WIZARD
 
 	#remove spell from book
-	GemRB.RemoveSpell (pc, type, level, index)
-	UpdateMageWindow ()
+	GemRB.RemoveSpell (pc, spelltype, level, spellIndex)
+	UpdateMageWindow (MageWindow)
 	return
 
 def LoadCondition ():
@@ -445,8 +416,8 @@ def LoadCondition ():
 
 	for i in range (CondCount):
 		#get the condition's name and description
-		tuple = (Table.GetValue (i, 0),Table.GetValue (i, 1) )
-		ContCond[i] = tuple
+		CondTuple = (Table.GetValue (i, 0), Table.GetValue (i, 1))
+		ContCond[i] = CondTuple
 
 	Table = GemRB.LoadTable ("conttarg")
 	TargCount = Table.GetRowCount ()
@@ -454,8 +425,8 @@ def LoadCondition ():
 
 	for i in range (TargCount):
 		#get the condition's name and description
-		tuple = (Table.GetValue (i, 0),Table.GetValue (i, 1) )
-		ContTarg[i] = tuple
+		CondTuple = (Table.GetValue (i, 0), Table.GetValue (i, 1))
+		ContTarg[i] = CondTuple
 	return
 
 def OpenSequencerWindow ():
@@ -483,16 +454,14 @@ def OpenSequencerWindow ():
 	p2 = GemRB.GetVar("P2")
 	Source = GemRB.GetSpellCastOn(pc)
 
-	print "Source: ", Source
+	print("Source: ", Source)
 	Target = p2>>16
 	Count = p2&255
 	if Count > 3:
 		Count = 3
 
-	GemRB.LoadWindowPack ("GUIMG", 640, 480)
-
 	#saving the original portrait window
-	OtherWindow = Window = GemRB.LoadWindow (6)
+	OtherWindow = Window = GemRB.LoadWindow (6, "GUIMG")
 
 	Title = Window.GetControl (0x0fffffff)
 
@@ -510,45 +479,41 @@ def OpenSequencerWindow ():
 		TypeButton.SetState (IE_GUI_BUTTON_DISABLED)
 
 	if Target == 2:
-		CondSelect.SetPos (-1,-1)
-		CondLabel.SetPos (-1,-1)
-		TargSelect.SetPos (-1,-1)
-		TargLabel.SetPos (-1,-1)
-		sb = Window.GetControl (5)
-		sb.SetPos (-1,-1)
-		sb = Window.GetControl (7)
-		sb.SetPos (-1,-1)
+		CondSelect.SetVisible (False)
+		CondLabel.SetVisible (False)
+		TargSelect.SetVisible (False)
+		TargLabel.SetVisible (False)
 	else:
-		CondSelect.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, ContingencyHelpCondition)
+		CondSelect.OnSelect (ContingencyHelpCondition)
 		CondSelect.SetOptions ([elem[0] for elem in ContCond], "ContCond", 0)
 
-		TargSelect.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, ContingencyHelpTarget)
+		TargSelect.OnSelect (ContingencyHelpTarget)
 		if Target:
 			TargSelect.SetOptions ([ContTarg[0][0]], "ContTarg", 1)
 		else:
 			TargSelect.SetOptions ([elem[0] for elem in ContTarg], "ContTarg", 0)
 
 	GemRB.SetVar ("SpellType", 0)
-	TypeButton.SetVarAssoc ("SpellType", 1)
+	TypeButton.SetVarAssoc ("SpellType", 0)
 	TypeButton.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
-	TypeButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ContTypePressed)
+	TypeButton.OnPress (ContTypePressed)
 
 	Button = Window.GetControl (9)
-	Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, LevelIncrease)
+	Button.OnPress (LevelIncrease)
 	Button = Window.GetControl (10)
-	Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, LevelDecrease)
+	Button.OnPress (LevelDecrease)
 
 	OkButton = Window.GetControl (27)
 	OkButton.SetText (11973)
-	OkButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	OkButton.MakeDefault()
 	OkButton.SetState (IE_GUI_BUTTON_DISABLED)
 
 	CancelButton = Window.GetControl (29)
 	CancelButton.SetText (13727)
-	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	CancelButton.MakeEscape()
 
-	OkButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ContingencyOk)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ContingencyCancel)
+	OkButton.OnPress (ContingencyOk)
+	CancelButton.OnPress (ContingencyCancel)
 	ContTypePressed ()
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 
@@ -583,7 +548,7 @@ def UpdateSpellList ():
 
 	BuildSpellList(pc, Type, Level-1)
 
-	names = SpellList.keys()
+	names = list(SpellList.keys())
 	names.sort()
 
 	cnt = len(names)
@@ -596,7 +561,7 @@ def UpdateSpellList ():
 			Button.SetSpellIcon (names[j], 1)
 			Button.SetText( str(SpellList[names[j]]) )
 			Button.SetVarAssoc("PickedSpell", j)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ContingencyHelpSpell)
+			Button.OnPress (ContingencyHelpSpell)
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		else:
 			Button.SetSpellIcon("")
@@ -606,13 +571,13 @@ def UpdateSpellList ():
 
 	Button = Window.GetControl (22)
 	Button.SetSpellIcon(Spell1, 1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteSpell1)
+	Button.OnPress (DeleteSpell1)
 	Button = Window.GetControl (23)
 	Button.SetSpellIcon(Spell2, 1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteSpell2)
+	Button.OnPress (DeleteSpell2)
 	Button = Window.GetControl (24)
 	Button.SetSpellIcon(Spell3, 1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteSpell3)
+	Button.OnPress (DeleteSpell3)
 
 	if not Spell1:
 		OkButton.SetState (IE_GUI_BUTTON_DISABLED)
@@ -620,7 +585,7 @@ def UpdateSpellList ():
 		OkButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
 
-def BuildSpellList (pc, type, level):
+def BuildSpellList (pc, spelltype, level):
 	global SpellList
 
 	import Spellbook
@@ -629,7 +594,7 @@ def BuildSpellList (pc, type, level):
 		LoadExclusions()
 
 	SpellList = {}
-	memoedSpells = Spellbook.GetMemorizedSpells (pc, type, level)
+	memoedSpells = Spellbook.GetMemorizedSpells (pc, spelltype, level)
 
 	# are we a sorcerer-style?
 	SorcererBook = Spellbook.HasSorcererBook (pc)
@@ -686,26 +651,24 @@ def ContingencyOk ():
 	if Target == 2:
 		GemRB.ApplyEffect (pc, "Sequencer:Store", 0, 0, Spell1, Spell2, Spell3, Source)
 	else:
-		GemRB.ApplyEffect (pc, "CastSpellOnCondition", 0, GemRB.GetVar ("ContCond"), Spell1, Spell2, Spell3, Source)
+		GemRB.ApplyEffect (pc, "CastSpellOnCondition", GemRB.GetVar ("ContTarg"), GemRB.GetVar ("ContCond"), Spell1, Spell2, Spell3, Source)
 	#set the innate
 	if GemRB.LearnSpell (pc, Source+"d", LS_MEMO):
-		print "EEEEK! Failed to learn sequencer/contingency!\n\n"
-	OtherWindow.Unload()
-	GUICommon.GameWindow.SetVisible (WINDOW_VISIBLE) # restores focus
+		GemRB.Log (LOG_ERROR, "ContingencyOk", "Failed to learn sequencer/contingency!")
+	OtherWindow.Close ()
 	return
 
 def ContingencyCancel ():
 	global OtherWindow
 
 	GemRB.SetPlayerStat (pc, IE_IDENTIFYMODE, 0)
-	OtherWindow.Unload()
-	GUICommon.GameWindow.SetVisible (WINDOW_VISIBLE) # restores focus
+	OtherWindow.Close ()
 	return
 
 def ContingencyHelpSpell ():
 	global Spell1, Spell2, Spell3
 
-	names = SpellList.keys()
+	names = list(SpellList.keys())
 	names.sort()
 	i = GemRB.GetVar("PickedSpell")
 	spell = names[i]
@@ -792,5 +755,75 @@ def LevelDecrease():
 	UpdateSpellList()
 	return
 
+def OpenContingenciesWindow():
+	global OtherWindow
+
+	OtherWindow = Window = GemRB.LoadWindow (7, "GUIMG")
+
+	OkButton = Window.GetControl (2)
+	OkButton.SetText (11973)
+	OkButton.OnPress (Window.Close)
+	OkButton.MakeDefault ()
+
+	pc = GemRB.GameGetSelectedPCSingle ()
+	contingencies = GemRB.GetEffects (pc, "CastSpellOnCondition")
+	sequencers = GemRB.GetEffects (pc, "Sequencer:Store")
+	targets = [ 30992, 30990, 30994, "Anyone"]
+	conditions = [ "Was hit", 30973, 30975, 30982, 30984, 30986, 30988, "Attacked", "Near 4'", "Near 10'", "Every round", "Took damage", "Killed", "Time of day", "In personal space", "State", "I died", "Someone died", "Got turned", "HP <", "HP% <", "Spell state" ]
+	for cidx in range(5):
+		rowID1 = Window.GetControl (0x10000000 + 31 + cidx)
+		rowID2 = Window.GetControl (0x10000000 + 37 + cidx)
+		rowID3 = Window.GetControl (0x10000000 + 43 + cidx)
+		spellBtn1 = Window.GetControl (25 + cidx)
+		spellBtn2 = Window.GetControl (31 + cidx)
+		spellBtn3 = Window.GetControl (37 + cidx)
+		handBtn = Window.GetControl (43 + cidx)
+		spellBtn1.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+		spellBtn2.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+		spellBtn3.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+		spellBtn1.SetState (IE_GUI_BUTTON_LOCKED)
+		spellBtn2.SetState (IE_GUI_BUTTON_LOCKED)
+		spellBtn3.SetState (IE_GUI_BUTTON_LOCKED)
+		if cidx < len(contingencies):
+			rowID1.SetText (str(cidx + 1))
+			cont = contingencies[cidx]
+			rowID2.SetText (conditions[cont["Param2"]])
+			rowID3.SetText (targets[cont["Param1"]])
+
+			spellBtn1.SetBAM (cont["Spell1Icon"][:-1] + "b", 0, 0)
+			if cont["Resource2"] != "":
+				spellBtn2.SetBAM (cont["Spell2Icon"][:-1] + "b", 0, 0)
+			if cont["Resource3"] != "":
+				spellBtn3.SetBAM (cont["Spell3Icon"][:-1] + "b", 0, 0)
+
+			# warning: can take out too many
+			handBtn.OnPress (lambda: GemRB.DispelEffect (pc, "CastSpellOnCondition", cont["Param2"]))
+		elif cidx < len(contingencies) + len(sequencers):
+			rowID1.SetText (str(cidx + 1))
+			cont = sequencers[cidx - len(contingencies)]
+			rowID2.SetText (25951)
+			rowID3.SetText (None)
+
+			spellBtn1.SetBAM (cont["Spell1Icon"][:-1] + "b", 0, 0)
+			if cont["Resource2"] != "":
+				spellBtn2.SetBAM (cont["Spell2Icon"][:-1] + "b", 0, 0)
+			if cont["Resource3"] != "":
+				spellBtn3.SetBAM (cont["Spell3Icon"][:-1] + "b", 0, 0)
+
+			# warning: can take out too many
+			handBtn.OnPress (lambda: GemRB.DispelEffect (pc, "CastSpellOnCondition", cont["Param2"]))
+		else:
+			rowID1.SetText (None)
+			rowID2.SetText (None)
+			rowID3.SetText (None)
+			spellBtn1.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
+			spellBtn1.SetState (IE_GUI_BUTTON_DISABLED)
+			spellBtn2.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
+			spellBtn2.SetState (IE_GUI_BUTTON_DISABLED)
+			spellBtn3.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
+			spellBtn3.SetState (IE_GUI_BUTTON_DISABLED)
+			handBtn.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+
+
 ###################################################
 # End of file GUIMG.py
