@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -24,6 +23,7 @@ from ie_stats import *
 import GameCheck
 import GUICommon
 import CommonTables
+from CommonWindow import AddScrollbarProxy
 
 #the different types possible
 LUPROFS_TYPE_LEVELUP = 1
@@ -47,7 +47,6 @@ OddIDs = 0
 #internal variables
 ProfsPointsLeft = 0
 ProfsNumButtons = 0
-ProfsTopIndex = 0
 ProfsTextArea = 0
 ProfsColumn = 0
 ProfsTable = 0
@@ -56,14 +55,13 @@ ProfCount = 0
 # the table offset is used in bg2, since in the end they made it use different
 # profs than in bg1, but left the table entries intact
 ProfsTableOffset = 0
-ProfsScrollBar = None
 
 # iwd1/bg1 table that holds the allowed proficiency <-> class mapping
 ClassWeaponsTable = None
 
 ProfsType = None
 
-def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1], classid=0, scroll=True, profTableOffset=8):
+def SetupProfsWindow (pc, proftype, window, callback, level1=[0,0,0], level2=[1,1,1], classid=0, scroll=True, profTableOffset=8):
 	"""Opens the proficiency selection window.
 
 	type specifies the type of selection we are doing; choices are above.
@@ -72,8 +70,8 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 	classid is sent only during dualclassing to specify the new class."""
 
 	global ProfsOffsetSum, ProfsOffsetButton1, ProfsOffsetLabel, ProfsOffsetStar, OddIDs
-	global ProfsOffsetPress, ProfsPointsLeft, ProfsNumButtons, ProfsTopIndex
-	global ProfsScrollBar, ProfsTableOffset, ProfsType
+	global ProfsOffsetPress, ProfsPointsLeft, ProfsNumButtons
+	global ProfsTableOffset, ProfsType
 	global ProfsWindow, ProfsCallback, ProfsTextArea, ProfsColumn, ProfsTable, ProfCount
 	global Profs2ndOffsetButton1, Profs2ndOffsetStar, Profs2ndOffsetLabel, ClassNameSave, ClassWeaponsTable
 
@@ -86,9 +84,10 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 	ProfsWindow = window
 	ProfsCallback = callback
 	ProfsTableOffset = profTableOffset
-	ProfsType = type
+	ProfsType = proftype
+	ProfsScrollBar = None
 
-	if type == LUPROFS_TYPE_CHARGEN and (GameCheck.IsBG1() or GameCheck.IsBG2()): #chargen
+	if proftype == LUPROFS_TYPE_CHARGEN and (GameCheck.IsBG1() or GameCheck.IsBG2OrEE ()):
 		ProfsOffsetSum = 9
 		ProfsOffsetButton1 = 11
 		ProfsOffsetStar = 27
@@ -99,7 +98,11 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		ProfsTextArea.SetText (9588)
 		if (scroll):
 			ProfsScrollBar = ProfsWindow.GetControl (78)
-	elif type == LUPROFS_TYPE_LEVELUP and GameCheck.IsBG2(): #levelup
+
+		if GameCheck.IsBG2OrEE () or GameCheck.IsIWD2 ():
+			import CharGenCommon
+			CharGenCommon.PositionCharGenWin (ProfsWindow)
+	elif proftype == LUPROFS_TYPE_LEVELUP and GameCheck.IsBG2OrEE ():
 		ProfsOffsetSum = 36
 		ProfsOffsetButton1 = 1
 		ProfsOffsetStar = 48
@@ -108,7 +111,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		ProfsNumButtons = 7
 		ProfsTextArea = ProfsWindow.GetControl (110)
 		ProfsScrollBar = ProfsWindow.GetControl (108)
-	elif type == LUPROFS_TYPE_LEVELUP and GameCheck.IsBG1(): #levelup
+	elif proftype == LUPROFS_TYPE_LEVELUP and GameCheck.IsBG1(): #levelup
 		ProfsOffsetSum = 36
 		ProfsOffsetButton1 = 1
 		ProfsOffsetStar = 48
@@ -118,7 +121,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		ProfsTextArea = ProfsWindow.GetControl (42)
 		if (scroll):
 			ProfsScrollBar = ProfsWindow.GetControl (108)
-	elif type == LUPROFS_TYPE_LEVELUP and GameCheck.IsIWD1(): #levelup
+	elif proftype == LUPROFS_TYPE_LEVELUP and GameCheck.IsIWD1(): #levelup
 		ProfsOffsetSum = 36
 		ProfsOffsetButton1 = 1
 		ProfsOffsetStar = 48
@@ -132,7 +135,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		if (scroll):
 			ProfsScrollBar = ProfsWindow.GetControl (108)
 		OddIDs = 0
-	elif type == LUPROFS_TYPE_DUALCLASS and GameCheck.IsIWD1(): #dualclass
+	elif proftype == LUPROFS_TYPE_DUALCLASS and GameCheck.IsIWD1(): #dualclass
 		ProfsOffsetSum = 40
 		ProfsOffsetButton1 = 50
 		ProfsOffsetStar = 0
@@ -147,7 +150,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		if (scroll):
 			ProfsScrollBar = ProfsWindow.GetControl (None)
 		OddIDs = 1
-	elif type == LUPROFS_TYPE_DUALCLASS and GameCheck.IsBG1(): #dualclass
+	elif proftype == LUPROFS_TYPE_DUALCLASS and GameCheck.IsBG1(): #dualclass
 		ProfsOffsetSum = 40
 		ProfsOffsetButton1 = 50
 		ProfsOffsetStar = 0
@@ -158,7 +161,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		ProfsTextArea.SetText (9588)
 		if (scroll):
 			ProfsScrollBar = ProfsWindow.GetControl (None)
-	elif type == LUPROFS_TYPE_DUALCLASS: #dualclass
+	elif proftype == LUPROFS_TYPE_DUALCLASS: #dualclass
 		ProfsOffsetSum = 40
 		ProfsOffsetButton1 = 50
 		ProfsOffsetStar = 0
@@ -175,7 +178,6 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 	#nullify internal variables
 	GemRB.SetVar ("ProfsTopIndex", 0)
 	ProfsPointsLeft = 0
-	ProfsTopIndex = 0
 
 	ProfsTable = GemRB.LoadTable ("profs")
 	if GameCheck.IsIWD1() or GameCheck.IsBG1():
@@ -188,27 +190,27 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 	if classid: #for dual classes when we can't get the class dualing to
 		Class = classid
 	elif IsDual[0] == 3:
-		Class = CommonTables.KitList.GetValue (IsDual[2], 7)
+		Class = CommonTables.KitList.GetValue (IsDual[2], 7, GTV_INT)
 	elif IsDual[0]:
 		Class = GUICommon.GetClassRowName(IsDual[2], "index")
-		Class = CommonTables.Classes.GetValue (Class, "ID")
+		Class = CommonTables.Classes.GetValue (Class, "ID", GTV_INT)
 	else:
 		Class = GemRB.GetPlayerStat (pc, IE_CLASS)
 	ClassName = GUICommon.GetClassRowName (Class, "class")
 
 	# profs.2da has entries for everyone, so no need to muck around
-	ProfsRate = ProfsTable.GetValue (ClassName, "RATE")
+	ProfsRate = ProfsTable.GetValue (ClassName, "RATE", GTV_INT)
 
 	#figure out how many prof points we have
 	if sum (level1) == 0: #character is being generated (either chargen or dual)
-		ProfsPointsLeft = ProfsTable.GetValue (ClassName, "FIRST_LEVEL")
+		ProfsPointsLeft = ProfsTable.GetValue (ClassName, "FIRST_LEVEL", GTV_INT)
 
 	ProfIndex = 0
 	IsMulti = GUICommon.IsMultiClassed (pc, 1)
 	if IsMulti[0] > 1:
 		if sum (level1) == 0: #character is being generated (either chargen or dual)
 			# don't give too many points; exact formula unknown, but this works with f/m and f/m/t
-			ProfsPointsLeft += sum(level2)/IsMulti[0]/ProfsRate
+			ProfsPointsLeft += sum(level2) // IsMulti[0] // ProfsRate
 		else:
 			# just look at the fastest prof-gaining class in the bunch and consider their level
 			# eg. a m/t should get a point at 3/4 and 7/8 (rate of 4)
@@ -217,18 +219,18 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 				if cls == 0:
 					break
 				ClsName = GUICommon.GetClassRowName (cls, "class")
-				Rate = ProfsTable.GetValue (ClsName, "RATE")
+				Rate = ProfsTable.GetValue (ClsName, "RATE", GTV_INT)
 				if Rate < BestRate:
 					BestRate = Rate
 					ProfIndex = IsMulti[1:].index(cls)
 
-			ProfsPointsLeft += level2[ProfIndex]/ProfsRate - level1[ProfIndex]/ProfsRate
+			ProfsPointsLeft += level2[ProfIndex] // ProfsRate - level1[ProfIndex] // ProfsRate
 	else:
 		if GUICommon.IsDualSwap (pc):
 			ProfIndex = 1
 
 		#we need these 2 number to floor before subtracting
-		ProfsPointsLeft += level2[ProfIndex]/ProfsRate - level1[ProfIndex]/ProfsRate
+		ProfsPointsLeft += level2[ProfIndex] // ProfsRate - level1[ProfIndex] // ProfsRate
 
 	#setup prof vars for passing between functions
 	ProfsTable = GemRB.LoadTable ("weapprof")
@@ -243,7 +245,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		ProfsColumn = ClassWeaponsTable.GetRowIndex (ClassName)
 	else:
 		Kit = GUICommon.GetKitIndex (pc)
-		if Kit and type != LUPROFS_TYPE_DUALCLASS and IsMulti[0]<2 and IsDual[0] in [0, 3]:
+		if Kit and proftype != LUPROFS_TYPE_DUALCLASS and IsMulti[0]<2 and IsDual[0] in [0, 3]:
 			#if we do kit with dualclass, we'll get the old kit (usually)
 			#also don't want to worry about kitted multis
 			ProfsColumn = CommonTables.KitList.GetValue (Kit, 5)
@@ -251,7 +253,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 			ProfsColumn = ProfsTable.GetColumnIndex (ClassName)
 
 	#setup some basic counts
-	RowCount = ProfsTable.GetRowCount () - ProfsTableOffset + 1
+	RowCount = ProfsTable.GetRowCount () - ProfsTableOffset
 	ProfCount = RowCount-ProfsNumButtons #decrease it with the number of controls
 
 	ProfsAssignable = 0
@@ -259,13 +261,13 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 	for i in range(RowCount):
 		ProfName = ProfsTable.GetValue (i+ProfsTableOffset, 1)
 		#decrease it with the number of invalid proficiencies
-		if ProfName > 0x1000000 or ProfName < 0:
+		if ProfName > 0x1000000 or ProfName <= 0:
 			ProfCount -= 1
 
 		#we only need the low 3 bits for proficiencies on levelup; otherwise
 		#we just set them all to 0
 		currentprof = 0
-		if type == LUPROFS_TYPE_LEVELUP:
+		if proftype == LUPROFS_TYPE_LEVELUP:
 			stat = ProfsTable.GetValue (i+ProfsTableOffset, 0)
 			if GameCheck.IsBG1():
 				stat = stat + IE_PROFICIENCYBASTARDSWORD
@@ -283,6 +285,8 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 			# it also has different ordering than ProfsTable
 			col = ClassWeaponsTable.GetColumnIndex (ProfsTable.GetRowName(i))
 			maxprof = ClassWeaponsTable.GetValue (ProfsColumn, col)
+			# the above is underreporting for lvl1 fighters, but it doesn't matter,
+			# since there are enough prof types and the relevant code does use profsmax.2da
 		else:
 			maxprof = ProfsTable.GetValue(i+ProfsTableOffset, ProfsColumn)
 		if maxprof > currentprof:
@@ -298,7 +302,7 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 		if ProfsOffsetPress != -1:
 			Button=ProfsWindow.GetControl(i+ProfsOffsetPress)
 			Button.SetVarAssoc("Prof", i)
-			Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, ProfsJustPress)
+			Button.OnPress (ProfsJustPress)
 
 		cid = i*2+ProfsOffsetButton1
 		if Profs2ndOffsetButton1 != -1 and i > 7:
@@ -306,26 +310,25 @@ def SetupProfsWindow (pc, type, window, callback, level1=[0,0,0], level2=[1,1,1]
 
 		Button=ProfsWindow.GetControl(cid)
 		Button.SetVarAssoc("Prof", i)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, ProfsLeftPress)
+		Button.OnPress (ProfsLeftPress)
+		Button.SetActionInterval (200)
 
 		Button=ProfsWindow.GetControl(cid+1)
 		Button.SetVarAssoc("Prof", i)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, ProfsRightPress)
+		Button.OnPress (ProfsRightPress)
+		Button.SetActionInterval (200)
 
+	value = ProfsRedraw (0)
 	if(ProfsScrollBar):
 		# proficiencies scrollbar
-		ProfsScrollBar.SetEvent(IE_GUI_SCROLLBAR_ON_CHANGE, ProfsScrollBarPress)
-		ProfsScrollBar.SetDefaultScrollBar ()
-		ProfsScrollBar.SetVarAssoc ("ProfsTopIndex", ProfCount)
-	ProfsRedraw (1)
+		ProfsScrollBar.OnChange (lambda sb: ProfsRedraw (sb.Value))
+		Button = ProfsWindow.GetControl(ProfsOffsetPress)
+		AddScrollbarProxy(ProfsWindow, ProfsScrollBar, Button)
+		ProfsScrollBar.SetVarAssoc ("ProfsTopIndex", value, 0, ProfCount)
 	return
 
-def ProfsRedraw (first=0):
-	"""Redraws the proficiencies part of the window.
-
-	If first is true, it skips ahead to the first assignable proficiency."""
-
-	global ProfsTopIndex, ProfsScrollBar
+def ProfsRedraw (ProfsTopIndex):
+	"""Redraws the proficiencies part of the window starting from ProfsTopIndex."""
 
 	ProfSumLabel = ProfsWindow.GetControl(0x10000000+ProfsOffsetSum)
 	ProfSumLabel.SetText(str(ProfsPointsLeft))
@@ -352,7 +355,7 @@ def ProfsRedraw (first=0):
 			Button2.SetState(IE_GUI_BUTTON_DISABLED)
 			Button1.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
 			Button2.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
-			if GameCheck.IsBG2() and (i==0 or ((i-1) in SkipProfs)):
+			if GameCheck.IsBG2OrEE () and (i == 0 or ((i - 1) in SkipProfs)):
 				SkipProfs.append (i)
 		else:
 			Button1.SetState(IE_GUI_BUTTON_ENABLED)
@@ -382,35 +385,20 @@ def ProfsRedraw (first=0):
 			else:
 				Star.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
 
-	if first and len (SkipProfs):
-		ProfsTopIndex += SkipProfs[-1]+1
-		GemRB.SetVar ("ProfsTopIndex", ProfsTopIndex)
-		if (ProfsScrollBar):
-			ProfsScrollBar.SetVarAssoc ("ProfsTopIndex", ProfCount)
-		ProfsRedraw ()
-	return
+	return ProfsTopIndex + SkipProfs[-1] + 1 if len(SkipProfs) else ProfsTopIndex
 
-def ProfsScrollBarPress():
-	"""Scrolls the window by reassigning ProfsTopIndex."""
-
-	global ProfsTopIndex
-
-	ProfsTopIndex = GemRB.GetVar ("ProfsTopIndex")
-	ProfsRedraw ()
-	return
-
-def ProfsJustPress():
+def ProfsJustPress (btn):
 	"""Updates the text area with a description of the proficiency."""
-	Pos = GemRB.GetVar ("Prof")+ProfsTopIndex
+	Pos = btn.Value + GemRB.GetVar ("ProfsTopIndex")
 	ProfsTextArea.SetText (ProfsTable.GetValue(Pos+ProfsTableOffset, 2) )
 	return
 
-def ProfsRightPress():
+def ProfsRightPress (btn):
 	"""Decrease the current proficiency by one."""
 
 	global ProfsPointsLeft
 
-	Pos = GemRB.GetVar("Prof")+ProfsTopIndex
+	Pos = btn.Value + GemRB.GetVar ("ProfsTopIndex")
 	ProfsTextArea.SetText(ProfsTable.GetValue(Pos+ProfsTableOffset, 2) )
 	ActPoint = GemRB.GetVar("Prof "+str(Pos) )
 	MinPoint = GemRB.GetVar ("ProfBase "+str(Pos) )
@@ -419,16 +407,16 @@ def ProfsRightPress():
 	GemRB.SetVar("Prof "+str(Pos),ActPoint-1)
 	ProfsPointsLeft += 1
 	GemRB.SetVar ("ProfsPointsLeft", ProfsPointsLeft)
-	ProfsRedraw ()
+	ProfsRedraw (GemRB.GetVar("ProfsTopIndex"))
 	ProfsCallback ()
 	return
 
-def ProfsLeftPress():
+def ProfsLeftPress (btn):
 	"""Increases the current proficiency by one."""
 
 	global ProfsPointsLeft
 
-	Pos = GemRB.GetVar("Prof")+ProfsTopIndex
+	Pos = btn.Value + GemRB.GetVar("ProfsTopIndex")
 	ProfsTextArea.SetText(ProfsTable.GetValue(Pos+ProfsTableOffset, 2) )
 	if ProfsPointsLeft == 0:
 		return
@@ -449,11 +437,11 @@ def ProfsLeftPress():
 	GemRB.SetVar("Prof "+str(Pos),ActPoint+1)
 	ProfsPointsLeft -= 1
 	GemRB.SetVar ("ProfsPointsLeft", ProfsPointsLeft)
-	ProfsRedraw ()
+	ProfsRedraw (GemRB.GetVar("ProfsTopIndex"))
 	ProfsCallback ()
 	return
 
-def ProfsSave (pc, type=LUPROFS_TYPE_LEVELUP):
+def ProfsSave (pc, proftype = LUPROFS_TYPE_LEVELUP):
 	"""Updates the actor with the new proficiencies."""
 
 	ProfCount = ProfsTable.GetRowCount () - ProfsTableOffset
@@ -463,20 +451,18 @@ def ProfsSave (pc, type=LUPROFS_TYPE_LEVELUP):
 			ProfID = ProfID + IE_PROFICIENCYBASTARDSWORD
 		SaveProf = GemRB.GetVar ("Prof "+str(i))
 
-		if type == LUPROFS_TYPE_CHARGEN and GameCheck.IsBG2():
+		if proftype == LUPROFS_TYPE_CHARGEN and GameCheck.IsBG2OrEE ():
 			GemRB.DispelEffect (pc, "Proficiency", ProfID)
-		else:
-			if type != LUPROFS_TYPE_DUALCLASS:
-				OldProf = GemRB.GetPlayerStat (pc, ProfID) & 0x38
-				SaveProf = OldProf | SaveProf
-			else: # gotta move the old prof to the back for dual class
-				OldProf = GemRB.GetPlayerStat (pc, ProfID) & 0x07
-				SaveProf = (OldProf << 3) | SaveProf
+		elif proftype != LUPROFS_TYPE_DUALCLASS:
+			OldProf = GemRB.GetPlayerStat (pc, ProfID) & 0x38
+			SaveProf = OldProf | SaveProf
+		else: # gotta move the old prof to the back for dual class
+			OldProf = GemRB.GetPlayerStat (pc, ProfID) & 0x07
+			SaveProf = (OldProf << 3) | SaveProf
 
 		GemRB.SetPlayerStat (pc, ProfID, SaveProf)
-		if GameCheck.IsBG2() and (type == LUPROFS_TYPE_LEVELUP or type == LUPROFS_TYPE_CHARGEN):
-			if SaveProf:
-				GemRB.ApplyEffect (pc, "Proficiency", SaveProf, ProfID)
+		if GameCheck.IsBG2OrEE () and proftype != LUPROFS_TYPE_DUALCLASS and SaveProf:
+			GemRB.ApplyEffect (pc, "Proficiency", SaveProf, ProfID)
 	return
 
 def ProfsNullify ():
