@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -49,18 +48,15 @@ EnhanceGUI = 0			# << scrollbars and extra spell slot for sorcs on LU
 BonusPoints = [0]*9		# bonus learning/memo points
 
 # chargen only
-SpellsPickButton = 0		# << button to select random spells
+SpellsPickButton = None	# << button to select random spells
 SpellsCancelButton = 0		# << cancel chargen
 
 IWD2 = False
 SpellBookType = IE_SPELL_TYPE_WIZARD
 if GameCheck.IsIWD2():
-	WIDTH = 800
-	HEIGHT = 600
 	IWD2 = True
-else:
-	WIDTH = 640
-	HEIGHT = 480
+
+NewScrollBarID = 1000
 
 def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, booktype=0):
 	"""Opens the spells selection window.
@@ -88,7 +84,7 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 	if kit == 0:
 		KitMask = 0x4000
 	else: # need to implement this if converted to CharGen
-		KitMask = kit 
+		KitMask = kit
 
 	if IWD2:
 		# save the spellbook type (class) that corresponds to our table
@@ -101,7 +97,7 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 	SpellsToMemoTable = GemRB.LoadTable (table)
 	if not SpellsToMemoTable.GetValue (str(level), str(1), GTV_INT):
 		if chargen:
-			if GameCheck.IsBG2():
+			if GameCheck.IsBG2OrEE ():
 				GemRB.SetNextScript("GUICG6")
 			elif GameCheck.IsBG1():
 				# HACK
@@ -113,33 +109,38 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 
 	# load our window
 	if chargen:
-		GemRB.LoadWindowPack("GUICG", WIDTH, HEIGHT)
-		SpellsWindow = GemRB.LoadWindow (7)
-		if not recommend:
-			GUICommon.CloseOtherWindow (SpellsWindow.Unload)
+		SpellsWindow = GemRB.LoadWindow (7, "GUICG")
+
+		if GameCheck.IsBG2OrEE ():
+			import CharGenCommon
+			CharGenCommon.PositionCharGenWin (SpellsWindow)
+		elif IWD2:
+			import CharOverview
+			CharOverview.PositionCharGenWin (SpellsWindow)
+
 		DoneButton = SpellsWindow.GetControl (0)
 		SpellsTextArea = SpellsWindow.GetControl (27)
 		SpellPointsLeftLabel = SpellsWindow.GetControl (0x1000001b)
 		if (EnhanceGUI):
-			SpellsWindow.CreateScrollBar (1000, 325,42, 16,252)
-			HideUnhideScrollBar(1)
+			sb = SpellsWindow.CreateScrollBar (NewScrollBarID, {'x' : 325, 'y' : 42, 'w' : 16, 'h' : 252})
+			sb.SetVisible(False)
 		SpellStart = 2
 
 		# cancel button only applicable for chargen
 		SpellsCancelButton = SpellsWindow.GetControl(29)
 		SpellsCancelButton.SetState(IE_GUI_BUTTON_ENABLED)
-		SpellsCancelButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, SpellsCancelPress)
+		SpellsCancelButton.OnPress (SpellsCancelPress)
 		SpellsCancelButton.SetText(13727)
-		SpellsCancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+		SpellsCancelButton.MakeEscape()
 
 		if (recommend):
 			# recommended spell picks
 			SpellsPickButton = SpellsWindow.GetControl(30)
 			SpellsPickButton.SetState(IE_GUI_BUTTON_ENABLED)
-			SpellsPickButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, SpellsPickPress)
+			SpellsPickButton.OnPress (SpellsPickPress)
 			SpellsPickButton.SetText(34210)
 	else:
-		SpellsWindow = GemRB.LoadWindow (8)
+		SpellsWindow = GemRB.LoadWindow (8, "GUIREC")
 		if IWD2:
 			DoneButton = SpellsWindow.GetControl (33)
 			SpellsTextArea = SpellsWindow.GetControl(30)
@@ -149,10 +150,11 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 			SpellsTextArea = SpellsWindow.GetControl(26)
 			SpellPointsLeftLabel = SpellsWindow.GetControl (0x10000018)
 		if(EnhanceGUI):
-			SpellsWindow.CreateScrollBar (1000, 290,142, 16,252)
-			HideUnhideScrollBar(1)
+			sb = SpellsWindow.CreateScrollBar (NewScrollBarID, {'x' : 290, 'y' : 142, 'w' : 16, 'h' : 252})
+			sb.SetVisible(False)
 			#25th spell button for sorcerers
 			SpellsWindow.CreateButton (24, 231, 345, 42, 42)
+
 		SpellStart = 0
 
 	# setup our variables
@@ -160,28 +162,30 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 	Memorization = 0
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
 	if IWD2 and not chargen:
-		LUClass = GemRB.GetVar ("LUClass")
+		LUClass = GemRB.GetVar ("LUClass") or 0
 		LUClassName = CommonTables.Classes.GetRowName (LUClass)
 		LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
 		Class = LUClassID
 
 	# the done button also doubles as a next button
-	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
-	DoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, SpellsDonePress)
+	DoneButton.SetDisabled(True)
+	DoneButton.OnPress (SpellsDonePress)
 	DoneButton.SetText(11973)
-	DoneButton.SetFlags(IE_GUI_BUTTON_DEFAULT, OP_OR)
+	DoneButton.MakeDefault()
 
 	# adjust the table for the amount of spells available for learning for free
 	# bg2 had SPLSRCKN, iwd2 also SPLBRDKN, but all the others lacked the tables
-	if SpellLearnTable == "MXSPLSOR":
+	if SpellLearnTable == "MXSPLSOR" or SpellLearnTable == "MXSPLSRC":
 		SpellLearnTable = "SPLSRCKN"
 	elif SpellLearnTable == "MXSPLBRD":
 		SpellLearnTable = "SPLBRDKN"
+	elif SpellLearnTable == "MXSPLSHM":
+		SpellLearnTable = "SPLSHMKN"
 	# ... which is also important for mages during chargen and then never again
 	elif SpellLearnTable == "MXSPLWIZ":
 		SpellLearnTable = "SPLWIZKN"
 	else:
-		print "OpenSpellsWindow: unhandled spell learning type encountered, falling back to memo table:", table
+		print("OpenSpellsWindow: unhandled spell learning type encountered, falling back to memo table:", table)
 	SpellLearnTable = GemRB.LoadTable (SpellLearnTable)
 
 	CastingStatValue = 0
@@ -232,23 +236,8 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 			SpellLevel = i
 			SpellBook = [0]*len(Spells[i])
 
-			if(EnhanceGUI):
-				# setup the scrollbar
-				ScrollBar = SpellsWindow.GetControl (1000)
-#				ScrollBar.SetDefaultScrollBar ()
-
-				# only scroll if we have more than 24 spells or 25 if extra 25th spell slot is available in sorcs LevelUp
-				if len (Spells[i]) > ( ButtonCount + ExtraSpellButtons() ):
-					ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, ShowSpells)
-					HideUnhideScrollBar(0)
-					if chargen:
-						ScrollBar.SetVarAssoc ("SpellTopIndex", GUICommon.ceildiv ( ( len (Spells[i])-ButtonCount ) , 6 ) + 1 )
-					else: #there are five rows of 5 spells in level up of sorcs
-						ScrollBar.SetVarAssoc ("SpellTopIndex", GUICommon.ceildiv ( ( len (Spells[i])-ButtonCount-1 ) , 5 ) + 1 )
-				else:
-					ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, None)
-					ScrollBar.SetVarAssoc ("SpellTopIndex", 0)
-					HideUnhideScrollBar(1)
+			ScrollBar = SpellsWindow.GetControl (NewScrollBarID)
+			UpdateScrollBar (ScrollBar, len (Spells[i]))
 
 			# show our spells
 			ShowSpells ()
@@ -257,14 +246,35 @@ def OpenSpellsWindow (actor, table, level, diff, kit=0, gen=0, recommend=True, b
 	# show the selection window
 	if chargen:
 		if recommend:
-			SpellsWindow.SetVisible (WINDOW_VISIBLE)
+			SpellsWindow.Focus()
 		else:
-			SpellsWindow.ShowModal (MODAL_SHADOW_NONE)
+			SpellsWindow.ShowModal (MODAL_SHADOW_GRAY)
 	else:
 		SpellsWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	return
 
+def UpdateScrollBar (ScrollBar, SpellCount):
+	if not ScrollBar:
+		return
+
+	# only scroll if we have more spells than buttons
+	# that's 25 if the extra 25th spell slot is available in sorcerer level up
+	if SpellCount > ButtonCount + ExtraSpellButtons():
+		ScrollBar.SetVisible (True)
+		ScrollBar.OnChange (ShowSpells)
+
+		extraCount = SpellCount - ButtonCount
+		if chargen:
+			count = GUICommon.ceildiv (extraCount, 6) + 1
+		else: # there are five rows of 5 spells in level up of sorcerers
+			count = GUICommon.ceildiv (extraCount - 1, 5) + 1
+		ScrollBar.SetVarAssoc ("SpellTopIndex", count, 0, count)
+	else:
+		ScrollBar.SetVarAssoc ("SpellTopIndex", 0)
+		ScrollBar.SetVisible (False)
+		ScrollBar.OnChange (None)
+
 def SpellsDonePress ():
 	"""Move to the next assignable level.
 
@@ -283,12 +293,10 @@ def SpellsDonePress ():
 
 	# save all the spells
 	if not Memorization:
+		sbt = SpellBookType if IWD2 else -1
 		for i in range (len (Spells[SpellLevel])):
 			if SpellBook[i]: # we need to learn this spell
-				if IWD2:
-					GemRB.LearnSpell (pc, Spells[SpellLevel][i][0], 0, 1<<SpellBookType)
-				else:
-					GemRB.LearnSpell (pc, Spells[SpellLevel][i][0])
+				GemRB.LearnSpell (pc, Spells[SpellLevel][i][0], 0, sbt)
 
 		# check to see if we need to update again
 		for i in range (SpellLevel+1, 9):
@@ -299,47 +307,43 @@ def SpellsDonePress ():
 				if not (chargen and GameCheck.IsBG1()):
 					SpellBook = [0]*len(Spells[i])
 
-				if (EnhanceGUI):
-					# setup the scrollbar
-					ScrollBar = SpellsWindow.GetControl (1000)
-					if len (Spells[i]) > ( ButtonCount + ExtraSpellButtons() ):
-						HideUnhideScrollBar(0)
-						if chargen:
-							ScrollBar.SetVarAssoc ("SpellTopIndex", GUICommon.ceildiv ( ( len (Spells[i])-ButtonCount ) , 6 ) + 1 )
-						else:
-							ScrollBar.SetVarAssoc ("SpellTopIndex", GUICommon.ceildiv ( ( len (Spells[i])-ButtonCount-1 ) , 5 ) + 1 )
-					else:
-						ScrollBar.SetVarAssoc ("SpellTopIndex", 0)
-						HideUnhideScrollBar(1)
+				ScrollBar = SpellsWindow.GetControl (NewScrollBarID)
+				UpdateScrollBar (ScrollBar, len (Spells[i]))
 
 				# show the spells and set the done button to off
 				ShowSpells ()
-				DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
+				DoneButton.SetDisabled(True)
 				return
 
 		# bg1 lets you memorize spells too (iwd too, but it does it by itself)
+		# tob doesn't, but we could enhance it — need to add multilevel support
+		# enable below if GameCheck.IsTOB () and not Spellbook.HasSorcererBook (pc)
 		if chargen and sum(MemoBook) == 0 and \
 		(GameCheck.IsBG1() or (IWD2 and SpellBookType == IE_IWD2_SPELL_WIZARD)):
 			SpellLevel = 0
-			# bump it for specialists and iwd2 casters with high stats
-			SpellsSelectPointsLeft[SpellLevel] = 1 + BonusPoints[SpellLevel]
+			if GameCheck.IsTOB ():
+				SpellsSelectPointsLeft[SpellLevel] = GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_WIZARD, SpellLevel, 1)
+			else:
+				# bump it for specialists and iwd2 casters with high stats
+				SpellsSelectPointsLeft[SpellLevel] = 1 + BonusPoints[SpellLevel]
 			# FIXME: setting the proper count here breaks original characters, see #680
 			#GemRB.SetMemorizableSpellsCount (pc, SpellsSelectPointsLeft[SpellLevel], SpellBookType, SpellLevel)
-			DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
+			DoneButton.SetDisabled (True)
 			Memorization = 1
 			ShowKnownSpells()
 			return
 
 	# close our window and update our records
-	if SpellsWindow and (not chargen or GameCheck.IsBG2() or IWD2):
-		SpellsWindow.Unload ()
+	if SpellsWindow and (not chargen or GameCheck.IsBG2OrEE () or IWD2):
+		SpellsWindow.Close ()
 		SpellsWindow = None
 
 	# move to the next script if this is chargen
 	if chargen:
-		if GameCheck.IsBG2():
+		if GameCheck.IsBG2OrEE ():
 			GemRB.SetNextScript("GUICG6")
 		elif GameCheck.IsBG1():
+			SpellsWindow.Close ()
 			# HACK
 			from CharGenCommon import next
 			next()
@@ -369,7 +373,7 @@ def ShowKnownSpells ():
 		if i + j >= len (SpellBook) or not SpellBook[i+j]:
 			SpellButton.SetState (IE_GUI_BUTTON_DISABLED)
 			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-			SpellButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, None)
+			SpellButton.OnPress (None)
 			continue
 		else:
 			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
@@ -378,13 +382,13 @@ def ShowKnownSpells ():
 		# fill in the button with the spell data
 		Spell = GemRB.GetSpell (Spells[SpellLevel][i+j][0], 1)
 		SpellButton.SetTooltip(Spell['SpellName'])
-		SpellButton.SetVarAssoc("MemorizePressed", i)
-		SpellButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, MemorizePress)
-		if GameCheck.IsBG2():
+		SpellButton.SetValue (i)
+		SpellButton.OnPress (MemorizePress)
+		if GameCheck.IsBG2OrEE ():
 			SpellButton.SetSprites("GUIBTBUT",0, 0,1,2,3)
 		else:
 			SpellButton.SetSprites("GUIBTBUT",0, 0,1,24,25)
-		SpellButton.SetBorder (0, 0,0, 0,0, 0,0,0,0, 0,0)
+		SpellButton.SetBorder (0, None, 0,0)
 
 		SpellButton.SetSpellIcon(Spells[SpellLevel][i+j][0], 1)
 		SpellButton.SetFlags(IE_GUI_BUTTON_PICTURE, OP_OR)
@@ -395,7 +399,7 @@ def ShowKnownSpells ():
 	GemRB.SetToken("number", str(SpellsSelectPointsLeft[SpellLevel]))
 	SpellsTextArea.SetText(17253)
 
-	if SpellsPickButton == 0:
+	if SpellsPickButton == None:
 		# no recommendations at all
 		return
 
@@ -406,14 +410,14 @@ def ShowKnownSpells ():
 
 	return
 
-def MemorizePress ():
+def MemorizePress (btn):
 	"""Toggles the memorisation of the given spell."""
 
 	global SpellsSelectPointsLeft, Spells, SpellBook, MemoBook
 
 	# get our variables
 	j = RowIndex()
-	i = GemRB.GetVar ("MemorizePressed") + j
+	i = btn.Value + j
 
 	# get the spell that's been pushed
 	Spell = GemRB.GetSpell (Spells[SpellLevel][i][0], 1)
@@ -423,7 +427,7 @@ def MemorizePress ():
 	if MemoBook[i]: # already picked -- unselecting
 		SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] + 1
 		MemoBook[i] = 0
-		DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
+		DoneButton.SetDisabled (True)
 	else: # selecting
 		# we don't have any picks left
 		if SpellsSelectPointsLeft[SpellLevel] == 0:
@@ -434,7 +438,7 @@ def MemorizePress ():
 		SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] - 1
 		MemoBook[i] = Spellbook.HasSpell(pc, SpellBookType, SpellLevel, Spells[SpellLevel][i][0]) + 1 # so all values are above 0
 		if SpellsSelectPointsLeft[SpellLevel] == 0:
-			DoneButton.SetState (IE_GUI_BUTTON_ENABLED)
+			DoneButton.SetDisabled (False)
 
 	# show selected spells
 	ShowSelectedSpells ()
@@ -447,7 +451,8 @@ def ShowSpells ():
 	j = RowIndex()
 
 	# we have a grid of 24 spells
-	for i in range (ButtonCount + ExtraSpellButtons()):
+	extraButtons = ExtraSpellButtons ()
+	for i in range (ButtonCount + extraButtons):
 		# ensure we can learn this many spells
 		SpellButton = SpellsWindow.GetControl (i+SpellStart)
 		if i + j >= len (Spells[SpellLevel]):
@@ -462,8 +467,8 @@ def ShowSpells ():
 		Spell = GemRB.GetSpell (Spells[SpellLevel][i+j][0], 1)
 		SpellButton.SetTooltip(Spell['SpellName'])
 		SpellButton.SetVarAssoc("ButtonPressed", i)
-		SpellButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, SpellsSelectPress)
-		if GameCheck.IsBG2():
+		SpellButton.OnPress (SpellsSelectPress)
+		if GameCheck.IsBG2OrEE ():
 			SpellButton.SetSprites("GUIBTBUT",0, 0,1,2,3)
 		else:
 			SpellButton.SetSprites("GUIBTBUT",0, 0,1,24,25)
@@ -475,15 +480,17 @@ def ShowSpells ():
 		if Spells[SpellLevel][i+j][1] == 0:
 			SpellButton.SetState(IE_GUI_BUTTON_LOCKED)
 			# shade red
-			SpellButton.SetBorder (0, 0,0, 0,0, 200,0,0,100, 1,1)
+			color = {'r' : 200, 'g' : 0, 'b' : 0, 'a' : 100}
+			SpellButton.SetBorder (0, color, 1, 1)
 		elif Spells[SpellLevel][i+j][1] == 1: # learnable
 			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
 			# unset any borders on this button or an un-learnable from last level
 			# will still shade red even though it is clickable
-			SpellButton.SetBorder (0, 0,0, 0,0, 0,0,0,0, 0,0)
+			SpellButton.SetBorder (0, None, 0,0)
 		else: # specialist (for iwd2 which has no green frames)
 			# use the green border state for matching specialist spells
-			SpellButton.SetBorder (0, 0,0, 0,0, 0,200,0,100, 1,0)
+			color = {'r' : 0, 'g' : 200, 'b' : 0, 'a' : 100}
+			SpellButton.SetBorder (0, color, 1,0)
 			SpellButton.SetState (IE_GUI_BUTTON_FAKEDISABLED)
 
 	# show which spells are selected
@@ -491,21 +498,21 @@ def ShowSpells ():
 
 	GemRB.SetToken("number", str(SpellsSelectPointsLeft[SpellLevel]))
 	SpellsTextArea.SetText(17250)
-	if SpellsWindow.HasControl (0x10000000):
-		LevelLabel = SpellsWindow.GetControl (0x10000000)
+	LevelLabel = SpellsWindow.GetControl (0x10000000)
+	if LevelLabel:
 		GemRB.SetToken ("SPELLLEVEL", str(SpellLevel+1))
 		LevelLabel.SetText (10345)
 
 	return
 
-def SpellsSelectPress ():
+def SpellsSelectPress (btn):
 	"""Toggles the selection of the given spell."""
 
 	global SpellsSelectPointsLeft, Spells, SpellBook
 
 	# get our variables
 	j = RowIndex()
-	i = GemRB.GetVar ("ButtonPressed") + j
+	i = btn.Value + j
 
 	# get the spell that's been pushed
 	Spell = GemRB.GetSpell (Spells[SpellLevel][i][0], 1)
@@ -516,7 +523,7 @@ def SpellsSelectPress ():
 		if SpellBook[i]: # already picked -- unselecting
 			SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] + 1
 			SpellBook[i] = 0
-			DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
+			DoneButton.SetDisabled (True)
 		else: # selecting
 			# we don't have any picks left
 			if SpellsSelectPointsLeft[SpellLevel] == 0:
@@ -535,7 +542,7 @@ def SpellsSelectPress ():
 			SpellsSelectPointsLeft[SpellLevel] = SpellsSelectPointsLeft[SpellLevel] - 1
 			SpellBook[i] = 1
 			if SpellsSelectPointsLeft[SpellLevel] == 0:
-				DoneButton.SetState (IE_GUI_BUTTON_ENABLED)
+				DoneButton.SetDisabled (False)
 
 	# show selected spells
 	ShowSelectedSpells ()
@@ -551,19 +558,19 @@ def MarkButton (i, select):
 	j = RowIndex()
 
 	if select:
-		type = IE_GUI_BUTTON_SELECTED
+		state = IE_GUI_BUTTON_SELECTED
 	else:
 		if Spells[SpellLevel][i][1] == 1:
-			type = IE_GUI_BUTTON_ENABLED
+			state = IE_GUI_BUTTON_ENABLED
 		elif Spells[SpellLevel][i][1] == 2:
 			# specialist spell
-			type = IE_GUI_BUTTON_FAKEDISABLED
+			state = IE_GUI_BUTTON_FAKEDISABLED
 		else: # can't learn
-			type = IE_GUI_BUTTON_LOCKED
+			state = IE_GUI_BUTTON_LOCKED
 
 	# we have to use the index on the actual grid
 	SpellButton = SpellsWindow.GetControl(i+SpellStart-j)
-	SpellButton.SetState(type)
+	SpellButton.SetState(state)
 	return
 
 def ShowSelectedSpells ():
@@ -579,7 +586,7 @@ def ShowSelectedSpells ():
 			MarkButton (j+k, 1)
 		else: # not selected
 			MarkButton (j+k, 0)
-	
+
 	# show how many spell picks are left
 	SpellPointsLeftLabel.SetText (str (SpellsSelectPointsLeft[SpellLevel]))
 	return
@@ -592,20 +599,22 @@ def SpellsCancelPress ():
 	# remove all learned spells
 	Spellbook.RemoveKnownSpells (pc, SpellBookType, 1, 9, 1)
 
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		# unload teh window and go back
 		if SpellsWindow:
-			SpellsWindow.Unload()
+			SpellsWindow.Close ()
 		GemRB.SetNextScript("CharGen6") #haterace
 	elif GameCheck.IsBG1():
+		if SpellsWindow:
+			SpellsWindow.Close ()
 		import CharGenCommon
-		CharGenCommon.BackPress()
+		CharGenCommon.back()
 	elif IWD2:
 		if SpellsWindow:
-			SpellsWindow.Unload()
+			SpellsWindow.Close ()
 		GemRB.SetNextScript("Feats")
 	else:
-		print "Uh-oh in SpellsCancelPress in", GemRB.GameType
+		print("Uh-oh in SpellsCancelPress in", GemRB.GameType)
 	return
 
 def SpellsPickPress ():
@@ -617,7 +626,11 @@ def SpellsPickPress ():
 	global SpellBook, SpellsSelectPointsLeft
 
 	# load up our table
-	AutoTable = GemRB.LoadTable ("splautop")
+	if Spellbook.HasSorcererBook (pc) & 3 == 3:
+		# priestly sorcerer like shaman
+		AutoTable = GemRB.LoadTable ("splautos")
+	else:
+		AutoTable = GemRB.LoadTable ("splautop")
 
 	for i in range (AutoTable.GetRowCount ()):
 		if SpellsSelectPointsLeft[SpellLevel] == 0:
@@ -644,8 +657,8 @@ def SpellsPickPress ():
 
 	# if we don't have any points left, we can enable the done button
 	if not SpellsSelectPointsLeft[SpellLevel]:
-		DoneButton.SetState (IE_GUI_BUTTON_ENABLED)
-				
+		DoneButton.SetDisabled (False)
+
 	return
 
 def HasSpecialistSpell ():
@@ -701,19 +714,3 @@ def ExtraSpellButtons ():
 		return 1
 	else:
 		return 0
-
-def HideUnhideScrollBar (hide = 0):
-	ScrollBar = SpellsWindow.GetControl (1000)
-
-	if hide == 1:
-		scrollx = -1
-		scrolly = -1
-	else:
-		if chargen:
-			scrollx = 325
-			scrolly = 42
-		else:
-			scrollx = 290
-			scrolly = 142
-
-	ScrollBar.SetPos (scrollx, scrolly)
