@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -75,21 +74,22 @@ def OpenLevelUpWindow():
 	global OldHPMax, OldSaves, OldLore, OldThaco, DeltaDSpells, DeltaWSpells
 	global NewDSpells, NewWSpells, OldDSpells, OldWSpells, pc, HLACount, ClassName, IsDual
 
-	LevelUpWindow = GemRB.LoadWindow (3)
+	LevelUpWindow = GemRB.LoadWindow (3, "GUIREC")
 
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		InfoButton = LevelUpWindow.GetControl (125)
 		InfoButton.SetText (13707)
-		InfoButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUpInfoPress)
+		InfoButton.OnPress (LevelUpInfoPress)
+		# hide "Character Generation"
+		LevelUpWindow.DeleteControl(0x1000007e)
 
 	DoneButton = LevelUpWindow.GetControl (0)
 	DoneButton.SetText (11962)
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUpDonePress)
+	DoneButton.OnPress (LevelUpDonePress)
 	DoneButton.SetState (IE_GUI_BUTTON_DISABLED)
-	DoneButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
-
-	# hide "Character Generation"
-	LevelUpWindow.DeleteControl(0x1000007e)
+	DoneButton.MakeDefault()
+	# also disable closing by ESC, so we don't stack upgrades
+	DoneButton.MakeEscape ()
 
 	# name
 	pc = GemRB.GameGetSelectedPCSingle ()
@@ -107,17 +107,16 @@ def OpenLevelUpWindow():
 	# class
 	Label = LevelUpWindow.GetControl (0x10000000+106)
 	Label.SetText (GUICommon.GetActorClassTitle (pc))
-	print "Title:",GUICommon.GetActorClassTitle (pc),"\tActor Title:",actor.ClassTitle()
+	print("Title: " + GUICommon.GetActorClassTitle (pc) + "\tActor Title: " + actor.ClassTitle())
 
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	print "Class:",Class,"\tActor Class:",actor.classid
-	SkillTable = GemRB.LoadTable("skills")
+	print("Class: " + str(Class) + "\tActor Class: " + str(actor.classid))
 
 	# kit
 	ClassName = GUICommon.GetClassRowName (Class, "class")
 	Kit = GUICommon.GetKitIndex (pc)
-	print "Kit:", Kit, "\tActor Kit:",actor.KitIndex()
-	print "ClassName:",ClassName,"\tActor ClassNames:",actor.ClassNames()
+	print("Kit: " + str(Kit) + "\tActor Kit: " + str(actor.KitIndex()))
+	print("ClassName: " + ClassName + "\tActor ClassNames: " + str(actor.ClassNames()))
 
 	# need this for checking gnomes
 	RaceName = GemRB.GetPlayerStat (pc, IE_RACE, 1)
@@ -164,30 +163,21 @@ def OpenLevelUpWindow():
 		# store a boolean for IsDual
 		IsDual = IsDual[0] > 0
 
-	print "NumClasses:",NumClasses,"\tActor NumClasses:",actor.NumClasses()
+	print("NumClasses: " + str(NumClasses) + "\tActor NumClasses: " + str(actor.NumClasses()))
 
-	Level = [0]*3
-	LevelDiff = [0]*3
-
-	# reorganize the leves if we're dc so the one we want to use is in Level[0]
-	# and the old one is in Level[1] (used to regain old class abilities)
 	if IsDual:
-		# convert the classes from indicies to class id's
+		# convert the classes from indices to class id's
 		DualSwap = GUICommon.IsDualSwap (pc)
 		ClassName = GUICommon.GetClassRowName (Classes[0], "index")
 		KitName = ClassName # for simplicity throughout the code
 
-		# we need the old level as well
-		if DualSwap:
-			Level[1] = GemRB.GetPlayerStat (pc, IE_LEVEL)
-		else:
-			Level[1] = GemRB.GetPlayerStat (pc, IE_LEVEL2)
+	print("Classes: " + str(Classes) + "\tActor Classes: " + str(actor.Classes()))
+	print("IsDual: " + str(IsDual > 0) + "\tActor IsDual: " + str(actor.isdual))
 
-	print "Classes:",Classes,"\tActor Classes:",actor.Classes()
-	print "IsDual:",IsDual>0,"\tActor IsDual",actor.isdual
+	# get the next target levels and difference between levels
+	Level = LUCommon.GetNextLevels(pc, Classes)
+	LevelDiff = LUCommon.GetLevelDiff(pc, Level)
 
-	hp = 0
-	HaveCleric = 0
 	# clear some globals, since we may get called multiple times with different classes
 	DeltaWSpells = 0
 	DeltaDSpells = 0
@@ -196,79 +186,8 @@ def OpenLevelUpWindow():
 	NewDSpells = [0]*7
 	NewWSpells = [0]*9
 
-	# get a bunch of different things each level
-	for i in range(NumClasses):
-#		print "Class:",Classes[i]
-		# we don't care about the current level, but about the to-be-achieved one
-		# get the next level
-		Level[i] = LUCommon.GetNextLevelFromExp (GemRB.GetPlayerStat (pc, IE_XP)/NumClasses, Classes[i])
-		TmpClassName = GUICommon.GetClassRowName (Classes[i], "class")
-
-		# find the level diff for each classes (3 max, obviously)
-		if i == 0:
-			if DualSwap:
-				LevelDiff[i] = Level[i] - GemRB.GetPlayerStat (pc, IE_LEVEL2)
-			else:
-				LevelDiff[i] = Level[i] - GemRB.GetPlayerStat (pc, IE_LEVEL)
-		elif i == 1:
-			LevelDiff[i] = Level[i] - GemRB.GetPlayerStat (pc, IE_LEVEL2)
-		elif i == 2:
-			LevelDiff[i] = Level[i] - GemRB.GetPlayerStat (pc, IE_LEVEL3)
-
-#		print "Level (",i,"):",Level[i]
-#		print "Level Diff (",i,"):",LevelDiff[i]
-
-		# save our current and next spell amounts
-		StartLevel = Level[i] - LevelDiff[i]
-		DruidTable = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", GTV_STR)
-		ClericTable = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", GTV_STR)
-		MageTable = CommonTables.ClassSkills.GetValue (TmpClassName, "MAGESPELL", GTV_STR)
-
-		# see if we have mage spells
-		if MageTable != "*":
-			# we get 1 extra spell per level if we're a specialist
-			Specialist = 0
-			if CommonTables.KitList.GetValue (Kit, 7) == 1: # see if we're a kitted mage
-				Specialist = 1
-
-			if Spellbook.HasSorcererBook (pc, Classes[i]):
-				MageTable = "SPLSRCKN"
-
-			MageTable = GemRB.LoadTable (MageTable)
-			# loop through each spell level and save the amount possible to cast (current)
-			for j in range (MageTable.GetColumnCount ()):
-				NewWSpells[j] = MageTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
-				OldWSpells[j] = MageTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
-				if NewWSpells[j] > 0: # don't want specialist to get 1 in levels they should have 0
-					NewWSpells[j] += Specialist
-				if OldWSpells[j] > 0:
-					OldWSpells[j] += Specialist
-			DeltaWSpells = sum(NewWSpells)-sum(OldWSpells)
-		elif ClericTable != "*":
-			# check for cleric spells
-			if not GemRB.HasResource(ClericTable, RES_2DA, 1):
-				ClericTable = "MXSPLPRS" # iwd1 doesn't have a DRUIDSPELL column in the table
-			ClericTable = GemRB.LoadTable (ClericTable)
-			HaveCleric = 1
-			# same as above
-			for j in range (ClericTable.GetColumnCount ()):
-				NewDSpells[j] = ClericTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
-				OldDSpells[j] = ClericTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
-			DeltaDSpells = sum(NewDSpells)-sum(OldDSpells)
-		elif DruidTable != "*":
-			# clerics have precedence in multis (ranger/cleric)
-			if HaveCleric == 0:
-				#use MXSPLPRS if we can't find the resource (SoA fix)
-				if not GemRB.HasResource (DruidTable, RES_2DA):
-					DruidTable = "MXSPLPRS"
-
-				# check druid spells
-				DruidTable = GemRB.LoadTable (DruidTable)
-				# same as above
-				for j in range (DruidTable.GetColumnCount ()):
-					NewDSpells[j] = DruidTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
-					OldDSpells[j] = DruidTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
-				DeltaDSpells = sum(NewDSpells)-sum(OldDSpells)
+	# calculate the new spells (results are stored in global variables)
+	GetNewSpells(pc, Classes, Level, LevelDiff, Kit)
 
 # this is handled by core
 #		# setup class bonuses for this class
@@ -281,9 +200,9 @@ def OpenLevelUpWindow():
 #		if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
 #			GUICommon.AddClassAbilities (pc, ABTable, Level[i], LevelDiff[i])
 
-	print "Actor CurrentLevels:",actor.Levels()
-	print "Levels:",Level,"Actor NextLevels:",actor.NextLevels()
-	print "LevelDiffs:",LevelDiff,"Actor LevelDiffs:",actor.LevelDiffs()
+	print("Actor CurrentLevels:" + str(actor.Levels()))
+	print("Levels: " + str(Level) + "\tActor NextLevels: " + str(actor.NextLevels()))
+	print("LevelDiffs: " + str(LevelDiff) + "\tActor LevelDiffs: " + str(actor.LevelDiffs()))
 
 	#update our saves, thaco, hp and lore
 	LUCommon.SetupSavingThrows (pc, Level)
@@ -314,10 +233,10 @@ def OpenLevelUpWindow():
 		HLATable = GemRB.LoadTable("lunumab")
 		# we need to check each level against a multi value (this is kinda screwy)
 		if actor.multiclass:
-			print "Actor HLA Names:",["MULTI"+str(actor.NumClasses())+name \
-				for name in actor.ClassNames()]
+			print("Actor HLA Names: " + str(["MULTI" + str(actor.NumClasses()) + name \
+				for name in actor.ClassNames()]))
 		else:
-			print "Actor HLA Names:",actor.ClassNames()
+			print("Actor HLA Names: " + str(actor.ClassNames()))
 
 		for i in range (NumClasses):
 			if IsMulti:
@@ -340,13 +259,13 @@ def OpenLevelUpWindow():
 				HLACount += LevelDiff[i]
 
 		# set values required by the hla level up code
-		HLACount = HLACount / HLATable.GetValue (ClassName, "RATE", GTV_INT)
+		HLACount = HLACount // HLATable.GetValue (ClassName, "RATE", GTV_INT)
 		GemRB.SetVar ("HLACount", HLACount)
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		HLAButton = LevelUpWindow.GetControl (126)
 		if HLACount:
 			HLAButton.SetText (4954)
-			HLAButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUpHLAPress)
+			HLAButton.OnPress (LevelUpHLAPress)
 		else:
 			HLAButton.SetFlags (IE_GUI_BUTTON_DISABLED, OP_OR)
 
@@ -354,7 +273,7 @@ def OpenLevelUpWindow():
 	Level1 = []
 	for i in range (len (Level)):
 		Level1.append (Level[i]-LevelDiff[i])
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		LUProfsSelection.SetupProfsWindow (pc, LUProfsSelection.LUPROFS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level)
 	else:
 		LUProfsSelection.SetupProfsWindow (pc, LUProfsSelection.LUPROFS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level, 0, False, 0)
@@ -363,13 +282,13 @@ def OpenLevelUpWindow():
 	#we autohide the skills and let SetupSkillsWindow show them if needbe
 	for i in range (4):
 		HideSkills (i)
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		LUSkillsSelection.SetupSkillsWindow (pc, LUSkillsSelection.LUSKILLS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level)
 	else:
 		LUSkillsSelection.SetupSkillsWindow (pc, LUSkillsSelection.LUSKILLS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level, 0, False)
 	NewSkillPoints = GemRB.GetVar ("SkillPointsLeft")
 
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		TextAreaControl = LevelUpWindow.GetControl(110)
 		TextAreaControl.SetText(GetLevelUpNews())
 	else:
@@ -377,7 +296,6 @@ def OpenLevelUpWindow():
 		TextAreaControl.SetText(GUIREC.GetStatOverview(pc, LevelDiff))
 
 	RedrawSkills()
-	GemRB.SetRepeatClickFlags (GEM_RK_DISABLE, OP_NAND)
 	LevelUpWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	# if we have a sorcerer who can learn spells, we need to do spell selection
@@ -409,8 +327,8 @@ def RedrawSkills():
 	global DoneButton, LevelUpWindow, HLACount
 
 	# we need to disable the HLA button if we don't have any HLAs left
-	HLACount = GemRB.GetVar ("HLACount")
-	if GameCheck.IsBG2() and HLACount == 0:
+	HLACount = GemRB.GetVar ("HLACount") or 0
+	if GameCheck.IsBG2OrEE () and HLACount == 0:
 		# turn the HLA button off
 		HLAButton = LevelUpWindow.GetControl (126)
 		HLAButton.SetState(IE_GUI_BUTTON_DISABLED)
@@ -508,7 +426,7 @@ def GetLevelUpNews():
 			# only display classes with new spells
 			if (NewDSpells[i]-OldDSpells[i]) == 0:
 				continue
-			GemRB.SetToken("level", str(i+1))
+			GemRB.SetToken ("LEVEL", str(i + 1))
 			News += GemRB.GetString(61269)+": " + str(NewDSpells[i]-OldDSpells[i]) + '\n'
 		News += '\n'
 	if DeltaWSpells > 0: # new wizard spells
@@ -517,7 +435,7 @@ def GetLevelUpNews():
 			# only display classes with new spells
 			if (NewWSpells[i]-OldWSpells[i]) == 0:
 				continue
-			GemRB.SetToken("level", str(i+1))
+			GemRB.SetToken ("LEVEL", str(i + 1))
 			News += GemRB.GetString(61269)+": " + str(NewWSpells[i]-OldWSpells[i]) + '\n'
 		News += '\n'
 
@@ -557,7 +475,6 @@ def LevelUpInfoPress():
 
 	if InfoCounter % 2:
 		# call GetStatOverview with the new levels, so the future overview is shown
-		# TODO: show only xp, levels, thac0, #att, lore, reputation, backstab, saving throws
 		TextAreaControl.SetText(GUIREC.GetStatOverview(pc, LevelDiff))
 	else:
 		TextAreaControl.SetText(GetLevelUpNews())
@@ -569,9 +486,8 @@ def LevelUpDonePress():
 	"""Updates the PC with the new choices.
 
 	Closes the window when finished."""
-	import GUICommonWindows
+	import PortraitWindow
 	import GUIREC
-	global SkillTable
 
 	# proficiencies
 	LUProfsSelection.ProfsSave (pc)
@@ -588,57 +504,24 @@ def LevelUpDonePress():
 		GemRB.SetPlayerStat (pc, IE_LEVEL2, Level[1])
 	GemRB.SetPlayerStat (pc, IE_LEVEL3, Level[2])
 
-	print "Levels:",Level[0],"/",Level[1],"/",Level[2]
+	print("Levels:", Level[0], "/", Level[1], "/", Level[2])
 
-	# save our number of memorizable spells per level
-	if DeltaWSpells > 0:
-		# loop through each wizard spell level
-		for i in range(len(NewWSpells)):
-			if NewWSpells[i] > 0: # we have new spells, so update
-				GemRB.SetMemorizableSpellsCount(pc, NewWSpells[i], IE_SPELL_TYPE_WIZARD, i)
-
-	# save our number of memorizable priest spells
-	if DeltaDSpells > 0: # druids and clerics count
-		for i in range (len(NewDSpells)):
-			# get each update
-			if NewDSpells[i] > 0:
-				GemRB.SetMemorizableSpellsCount (pc, NewDSpells[i], IE_SPELL_TYPE_PRIEST, i)
-
-			# learn all the spells we're given, but don't have, up to our given casting level
-			# bonus spells don't count in determining if we can use this level
-			if GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_PRIEST, i, 0) > 0: # we can memorize spells of this level
-				for j in range(NumClasses): # loop through each class
-					TmpClassName = GUICommon.GetClassRowName (Classes[j], "class")
-					IsDruid = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", GTV_STR)
-					IsCleric = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", GTV_STR)
-					if IsCleric == "*" and IsDruid == "*": # no divine spells (so mage/cleric multis don't screw up)
-						continue
-					elif IsCleric == "*": # druid spells
-						ClassFlag = 0x8000
-					else: # cleric spells
-						ClassFlag = 0x4000
-
-					Learnable = Spellbook.GetLearnablePriestSpells(ClassFlag, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1)
-					for k in range(len(Learnable)): # loop through all the learnable spells
-						if Spellbook.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
-							GemRB.LearnSpell(pc, Learnable[k])
+	# spells
+	SaveNewSpells()
 
 	# hlas
 	# level, xp and other stuff by the core?
 
 	# 5261 - Regained abilities from inactive class
 	if IsDual: # we're dual classed
-		print "activation?"
+		print("activation?")
 		if (Level[0] - LevelDiff[0]) <= Level[1] and Level[0] > Level[1]: # our new classes now surpasses our old class
-			print "reactivating base class"
+			print("reactivating base class")
 			ReactivateBaseClass ()
 
 	if LevelUpWindow:
-		LevelUpWindow.Unload()
-	GUICommonWindows.UpdatePortraitWindow ()
-	GUIREC.UpdateRecordsWindow()
-
-	GemRB.SetRepeatClickFlags (GEM_RK_DISABLE, OP_OR)
+		LevelUpWindow.Close()
+	PortraitWindow.UpdatePortraitWindow()
 	return
 
 def LevelUpHLAPress ():
@@ -647,7 +530,7 @@ def LevelUpHLAPress ():
 	# we can turn the button off and temporarily set HLACount to 0
 	# because there is no cancel button on the HLA window; therefore,
 	# it's guaranteed to come back as 0
-	TmpCount = GemRB.GetVar ("HLACount")
+	TmpCount = GemRB.GetVar ("HLACount") or 0
 	GemRB.SetVar ("HLACount", 0)
 	RedrawSkills ()
 	GemRB.SetVar ("HLACount", TmpCount)
@@ -659,11 +542,10 @@ def ReactivateBaseClass ():
 	"""Regains all abilities of the base dual-class.
 
 	Proficiencies, THACO, saves, spells, and innate abilites,
-	most noteably."""
+	most notably."""
 
 	# we construct the Classes array, so that the active class is always first and the base is second
 	ClassName = GUICommon.GetClassRowName (Classes[1], "class")
-	KitIndex = GUICommon.GetKitIndex (pc)
 
 	# force reinitialization of the actionbar by forcing the PCF to run
 	ClassID = GemRB.GetPlayerStat (pc, IE_CLASS)
@@ -673,7 +555,7 @@ def ReactivateBaseClass ():
 	# reactivate all our proficiencies
 	TmpTable = GemRB.LoadTable ("weapprof")
 	ProfsTableOffset = 0
-	if GameCheck.IsBG2 ():
+	if GameCheck.IsBG2OrEE ():
 		ProfsTableOffset = 8 # skip bg1 weapprof.2da proficiencies
 	ProfCount = TmpTable.GetRowCount () - ProfsTableOffset
 	for i in range(ProfCount):
@@ -685,8 +567,8 @@ def ReactivateBaseClass ():
 		NewProf = Value & 0x07
 		if OldProf > NewProf:
 			Value = (OldProf << 3) | OldProf
-			print "Value:",Value
-			if GameCheck.IsBG2():
+			print("Value:", Value)
+			if GameCheck.IsBG2OrEE ():
 				GemRB.ApplyEffect (pc, "Proficiency", Value, ProfID )
 			else:
 				GemRB.SetPlayerStat (pc, ProfID, Value)
@@ -746,3 +628,110 @@ def ReactivateBaseClass ():
 				if Spellbook.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
 					GemRB.LearnSpell(pc, Learnable[k])
 
+def GetNewSpells(actor, Classes, Level, LevelDiff, Kit=0):
+	'''Scan each class for new spells. Values are stored in the global variables'''
+
+	global DeltaDSpells, DeltaWSpells, NewDSpells, NewWSpells, OldDSpells, OldWSpells
+
+	# clear the globals, since we may get called multiple times with different classes
+	HaveCleric = 0
+	DeltaWSpells = 0
+	DeltaDSpells = 0
+	OldDSpells = [0]*7
+	OldWSpells = [0]*9
+	NewDSpells = [0]*7
+	NewWSpells = [0]*9
+
+	# run through each class to detect new spells
+	for i in range(len(Classes)):
+		if Classes[i] == 0:
+			break
+		TmpClassName = GUICommon.GetClassRowName (Classes[i], "class")
+
+		# save our current and next spell amounts
+		StartLevel = Level[i] - LevelDiff[i]
+		DruidTable = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", GTV_STR)
+		ClericTable = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", GTV_STR)
+		MageTable = CommonTables.ClassSkills.GetValue (TmpClassName, "MAGESPELL", GTV_STR)
+
+		# see if we have mage spells
+		if MageTable != "*":
+			# we get 1 extra spell per level if we're a specialist
+			Specialist = 0
+			if CommonTables.KitList.GetValue (Kit, 7) == 1: # see if we're a kitted mage
+				Specialist = 1
+
+			if Spellbook.HasSorcererBook (actor, Classes[i]):
+				MageTable = "SPLSRCKN"
+
+			MageTable = GemRB.LoadTable (MageTable)
+			# loop through each spell level and save the amount possible to cast (current)
+			for j in range (MageTable.GetColumnCount ()):
+				NewWSpells[j] = MageTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
+				OldWSpells[j] = MageTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
+				if NewWSpells[j] > 0: # don't want specialist to get 1 in levels they should have 0
+					NewWSpells[j] += Specialist
+				if OldWSpells[j] > 0:
+					OldWSpells[j] += Specialist
+			DeltaWSpells = sum(NewWSpells)-sum(OldWSpells)
+		elif ClericTable != "*":
+			# check for cleric spells
+			if not GemRB.HasResource(ClericTable, RES_2DA, 1):
+				ClericTable = "MXSPLPRS" # iwd1 doesn't have a DRUIDSPELL column in the table
+			ClericTable = GemRB.LoadTable (ClericTable)
+			HaveCleric = 1
+			# same as above
+			for j in range (ClericTable.GetColumnCount ()):
+				NewDSpells[j] = ClericTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
+				OldDSpells[j] = ClericTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
+			DeltaDSpells = sum(NewDSpells)-sum(OldDSpells)
+		elif DruidTable != "*":
+			# clerics have precedence in multis (ranger/cleric)
+			if HaveCleric == 0:
+				#use MXSPLPRS if we can't find the resource (SoA fix)
+				if not GemRB.HasResource (DruidTable, RES_2DA):
+					DruidTable = "MXSPLPRS"
+
+				# check druid spells
+				DruidTable = GemRB.LoadTable (DruidTable)
+				# same as above
+				for j in range (DruidTable.GetColumnCount ()):
+					NewDSpells[j] = DruidTable.GetValue (str(Level[i]), str(j+1), GTV_INT)
+					OldDSpells[j] = DruidTable.GetValue (str(StartLevel), str(j+1), GTV_INT)
+				DeltaDSpells = sum(NewDSpells)-sum(OldDSpells)
+
+def SaveNewSpells():
+	# save our number of memorizable spells per level
+	if DeltaWSpells > 0:
+		# loop through each wizard spell level
+		for i in range(len(NewWSpells)):
+			if NewWSpells[i] > 0: # we have new spells, so update
+				GemRB.SetMemorizableSpellsCount(pc, NewWSpells[i], IE_SPELL_TYPE_WIZARD, i)
+
+	# save our number of memorizable priest spells
+	if DeltaDSpells <= 0: # druids and clerics count
+		return
+
+	for i in range (len(NewDSpells)):
+		# get each update
+		if NewDSpells[i] > 0:
+			GemRB.SetMemorizableSpellsCount (pc, NewDSpells[i], IE_SPELL_TYPE_PRIEST, i)
+
+		# can we memorize spells of this level?
+		# bonus spells don't count in determining if we can use this level
+		if GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_PRIEST, i, 0) <= 0:
+			continue
+
+		# learn all the spells we're given, but don't have, up to our given casting level
+		for j in range(NumClasses):
+			TmpClassName = GUICommon.GetClassRowName (Classes[j], "class")
+			IsDruid = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", GTV_STR)
+			IsCleric = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", GTV_STR)
+			if IsCleric == "*" and IsDruid == "*": # no divine spells (so mage/cleric multis don't screw up)
+				continue
+			elif IsCleric == "*": # druid spells
+				ClassFlag = 0x8000
+			else: # cleric spells
+				ClassFlag = 0x4000
+
+			Spellbook.LearnPriestSpells (pc, i + 1, ClassFlag)
