@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2011 The GemRB Project
 #
@@ -16,7 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
-# a library of any functions for spell(book) managment
+# a library of any functions for spell(book) management
 
 import GemRB
 import CommonTables
@@ -26,6 +25,7 @@ from ie_stats import *
 from ie_action import ACT_LEFT, ACT_RIGHT
 from ie_spells import *
 from ie_restype import RES_2DA
+from ie_modal import MS_TURNUNDEAD
 
 #################################################################
 # this is in the operator module of the standard python lib
@@ -52,16 +52,19 @@ def GetUsableMemorizedSpells(actor, BookType):
 			if not Spell0["Flags"]:
 				# depleted, so skip
 				continue
-			if Spell0["SpellResRef"] in spellResRefs:
+			resref = Spell0["SpellResRef"].lower()
+			if resref in spellResRefs:
 				# add another one, so we can get the count more cheaply later
-				spellResRefs.append (Spell0["SpellResRef"])
+				spellResRefs.append (resref)
+				continue
+			spellResRefs.append (resref)
+			Spell = GemRB.GetSpell(resref)
+			if not Spell:
 				continue
-			spellResRefs.append (Spell0["SpellResRef"])
-			Spell = GemRB.GetSpell(Spell0["SpellResRef"])
 			Spell['BookType'] = BookType # just another sorting key
 			Spell['SpellIndex'] = GemRB.GetSpelldataIndex (actor, Spell["SpellResRef"], 1<<BookType) # crucial!
 			if Spell['SpellIndex'] == -1:
-				print "Error, memorized spell not found!", Spell["SpellResRef"], 1<<BookType
+				GemRB.Log (LOG_ERROR, "GetUsableMemorizedSpells", "Memorized spell not found! " + Spell["SpellResRef"] + " of type " + str(1<<BookType))
 			Spell['SpellIndex'] += 1000 * 1<<BookType
 			memorizedSpells.append (Spell)
 
@@ -71,10 +74,11 @@ def GetUsableMemorizedSpells(actor, BookType):
 	# count and remove the duplicates
 	memorizedSpells2 = []
 	for spell in memorizedSpells:
-		if spell["SpellResRef"] in spellResRefs:
-			spell['MemoCount'] = spellResRefs.count(spell["SpellResRef"])
-			while spell["SpellResRef"] in spellResRefs:
-				spellResRefs.remove(spell["SpellResRef"])
+		resref = spell["SpellResRef"].lower()
+		if resref in spellResRefs:
+			spell['MemoCount'] = spellResRefs.count(resref)
+			while resref in spellResRefs:
+				spellResRefs.remove(resref)
 			memorizedSpells2.append(spell)
 
 	return memorizedSpells2
@@ -103,7 +107,8 @@ def GetKnownSpellsDescription(actor, BookType):
 	info = ""
 	spells = GetKnownSpells (actor, BookType)
 	# reverse spells order grouped by SpellLevel
-	spells.sort (lambda fst, snd: -1 if fst['SpellLevel'] == snd['SpellLevel'] else 0)
+	spells.reverse ()
+	spells.sort (key = lambda spell: spell['SpellLevel'])
 	for spell in spells:
 		info += GemRB.GetString (spell['SpellName']) + "\n"
 	return info
@@ -147,11 +152,16 @@ def GetMemorizedSpells(actor, BookType, level):
 		Spell = GemRB.GetSpell(Spell0["SpellResRef"])
 		Spell['KnownCount'] = 1
 		Spell['MemoCount'] = Spell0["Flags"]
+		Spell['SpellIndex'] = i
 		memoSpells.append (Spell)
 
 	return memoSpells
 
 # direct access to the spellinfo struct
+def HasSpellinfoSpell(pc, resRef):
+	spellResRefs = GemRB.GetSpelldata (pc)
+	return resRef in spellResRefs
+
 # SpellIndex is the index of the spell in the struct, but we add a thousandfold of the spell type for later use in SpellPressed
 def GetSpellinfoSpells(actor, BookType):
 	memorizedSpells = []
@@ -191,8 +201,14 @@ def SortUsableSpells(memorizedSpells):
 # BookType is a spellbook type bitfield (1-mage, 2-priest, 4-innate and others in iwd2)
 # Offset is a control ID offset here for iwd2 purposes
 def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
+	import GUICommon
 	actor = GemRB.GameGetFirstSelectedActor ()
 
+	# bardsongs weren't saved in iwd1, so learn them now if needed
+	if GameCheck.IsIWD1 () and BookType == (1 << IE_SPELL_TYPE_SONG) and HasSpell (actor, IE_SPELL_TYPE_SONG, 0, "SPIN151") == -1:
+		level = GemRB.GetPlayerStat (actor, IE_LEVEL)
+		GUICommon.AddClassAbilities (actor, "clabbard", level, level)
+
 	# check if we're dealing with a temporary spellbook
 	if GemRB.GetVar("ActionLevel") == UAW_2DASPELLS:
 		allSpells = GetSpellinfoSpells (actor, BookType)
@@ -218,7 +234,7 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 		memorizedSpells = SortUsableSpells(allSpells)
 
 	# start creating the controls
-	import GUICommonWindows
+	import ActionsWindow
 	# TODO: ASCOL, ROWS
 	#AsCol = CommonTables.SpellDisplay.GetValue (layout, "AS_COL")
 	#Rows = CommonTables.SpellDisplay.GetValue (layout, "ROWS")
@@ -229,32 +245,46 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 		Button = Window.GetControl (Offset)
 		Button.SetText ("")
 		if Start:
-			GUICommonWindows.SetActionIconWorkaround (Button, ACT_LEFT, 0)
+			ActionsWindow.SetActionIconWorkaround (Button, ACT_LEFT, 0)
 			Button.SetState (IE_GUI_BUTTON_UNPRESSED)
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-			Button.SetTooltip ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 
 	# disable all spells if fx_disable_spellcasting was run with the same type
 	# but only if there are any spells of that type to disable
 	disabled_spellcasting = GemRB.GetPlayerStat(actor, IE_CASTING, 0)
+	classRowName = GUICommon.GetClassRowName (actor)
+	extraActionButton = not (GameCheck.IsIWD2() or GameCheck.IsPST()) and classRowName == "CLERIC_THIEF"
 	actionLevel = GemRB.GetVar ("ActionLevel")
 
-	#order is: mage, cleric, innate, class, song, (defaults to 1, item)
-	spellSections = [2, 4, 8, 16, 16]
+	# spellType will have IE_SPL_ITEM (...) not IE_SPELL_TYPE_PRIEST (...)!
+	# IE_CASTING order is: item, cleric, mage, innate, class, song
+	# bg2 spellType order: special, mage, priest, psionic, innate, song
+	spellSections = [1, 4, 2, 1, 8, 1]
 
 	# create the spell icon buttons
 	buttonCount = 12 - More * 2 # GUIBT_COUNT in PCStatsStruct
 	for i in range (buttonCount):
 		Button = Window.GetControl (i+Offset+More)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
+		Button.OnRightPress (None)
+
+		# yuck, handle offloaded cleric/thief icon
+		# original did picking, we do turning, since it is used less often
+		if i + Start == 0 and extraActionButton:
+			Button.SetActionIcon (globals(), 6, 1)
+			Button.OnPress (ActionsWindow.ActionTurnPressed)
+			if GemRB.GetModalState (actor) == MS_TURNUNDEAD:
+				Button.SetState (IE_GUI_BUTTON_SELECTED)
+			else:
+				Button.SetState (IE_GUI_BUTTON_ENABLED)
+			Button.SetText ("")
+			continue
 
 		if i+Start >= len(memorizedSpells):
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 			Button.SetText ("")
-			Button.SetTooltip ("")
 			continue
 		Spell = memorizedSpells[i+Start]
 		spellType = Spell['SpellType']
@@ -273,13 +303,17 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 		specialSpell = (specialSpell & SP_IDENTIFY) or ((specialSpell & SP_SURGE) and actionLevel == UAW_ALLMAGE)
 		if specialSpell & SP_SILENCE and Spell['HeaderFlags'] & 0x2000000: # SF_IGNORES_SILENCE
 			specialSpell ^= SP_SILENCE
-		if specialSpell or (disabled_spellcasting&spellType):
+
+		disabled = disabled_spellcasting & spellType
+		if disabled_spellcasting == 16: # class ("magical" spell in ee)
+			disabled = not (Spell["HeaderFlags"] & 0x4000) # SF_HLA
+		if specialSpell or disabled:
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.EnableBorder(1, 0)
 		else:
 			Button.SetState (IE_GUI_BUTTON_UNPRESSED)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommonWindows.SpellPressed)
-			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, GUICommonWindows.SpellShiftPressed)
+			Button.OnPress (ActionsWindow.SpellPressed)
+			Button.OnShiftPress (ActionsWindow.SpellShiftPressed)
 
 		if Spell['SpellResRef']:
 			Button.SetSprites ("guibtbut", 0, 0,1,2,3)
@@ -295,14 +329,13 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 	# scroll right button
 	if More:
 		Button = Window.GetControl (Offset+buttonCount+1)
-		GUICommonWindows.SetActionIconWorkaround (Button, ACT_RIGHT, buttonCount)
+		ActionsWindow.SetActionIconWorkaround (Button, ACT_RIGHT, buttonCount)
 		Button.SetText ("")
 		if len(memorizedSpells) - Start > 10:
 			Button.SetState (IE_GUI_BUTTON_UNPRESSED)
 		else:
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-			Button.SetTooltip ("")
 
 #################################################################
 # routines used during character generation and levelup
@@ -316,7 +349,7 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 def GetIWD2Spells (kit, usability, level, baseClass = -1):
 	spells = []
 	if baseClass == -1:
-		print "Error, didn't pass a base class in iwd2!"
+		print("Error, didn't pass a base class in iwd2!")
 		return spells
 	# to use baseClass as a column index, we first need to convert it,
 	# since only casters are in the table. But only casters have spell stats too
@@ -333,8 +366,8 @@ def GetIWD2Spells (kit, usability, level, baseClass = -1):
 		if badSchools == -1:
 			badSchools = 0
 	else:
-		# only wizards have alignment and school restrictions
-		kit = usability = 0
+		# only wizards care about spell schools
+		kit = 0
 
 	spellsTable = GemRB.LoadTable ("listspll")
 	spellCount = spellsTable.GetRowCount ()
@@ -366,7 +399,6 @@ def GetIWD2Spells (kit, usability, level, baseClass = -1):
 
 def GetMageSpells (Kit, Alignment, Level, baseClass = -1):
 	MageSpells = []
-	SpellType = 99
 	v = CommonTables.Aligns.FindValue (3, Alignment)
 	Usability = Kit | CommonTables.Aligns.GetValue(v, 5)
 	WildMages = True
@@ -375,24 +407,24 @@ def GetMageSpells (Kit, Alignment, Level, baseClass = -1):
 		return GetIWD2Spells (Kit, Usability, Level, baseClass)
 
 	SpellsTable = GemRB.LoadTable ("spells")
-	for i in range(SpellsTable.GetValue ("MAGE", str(Level), GTV_INT)):
+	SpellCount = SpellsTable.GetValue ("MAGE", str(Level), GTV_INT)
+	for i in range(SpellCount):
 		SpellName = "SPWI%d%02d"%(Level,i+1)
 		ms = GemRB.GetSpell (SpellName, 1)
 		if ms == None:
 			continue
 
+		SpellType = 1
 		if Usability & ms['SpellExclusion']:
 			SpellType = 0
-		else:
-			SpellType = 1
-			if Kit & (1 << ms['SpellSchool']+5): # of matching specialist school
-				SpellType = 2
-			# Wild mage spells are of normal schools, so we have to find them
-			# separately. Generalists can learn any spell but the wild ones, so
-			# we check if the mage is wild and if a generalist wouldn't be able
-			# to learn the spell.
-			if WildMages and Kit == 0x8000 and (0x4000 & ms['SpellExclusion']):
-				SpellType = 2
+		elif Kit & (1 << ms['SpellSchool'] + 5): # of matching specialist school
+			SpellType = 2
+		# Wild mage spells are of normal schools, so we have to find them
+		# separately. Generalists can learn any spell but the wild ones, so
+		# we check if the mage is wild and if a generalist wouldn't be able
+		# to learn the spell.
+		elif WildMages and Kit == 0x8000 and (0x4000 & ms['SpellExclusion']):
+			SpellType = 2
 		MageSpells.append ([SpellName, SpellType])
 
 	return MageSpells
@@ -417,8 +449,8 @@ def GetLearnableDomainSpells (pc, Level, baseClassName = -1):
 	BaseClassIndex = CommonTables.Classes.GetRowIndex (baseClassName)
 	# columns correspond to kits in the same order
 	KitIndex = GUICommonWindows.GetKitIndex (pc, BaseClassIndex)
-	if KitIndex == -1:
-		print "GetLearnableDomainSpells: couldn't determine the kit, bailing out!"
+	if KitIndex is None:
+		print("GetLearnableDomainSpells: couldn't determine the kit, bailing out!")
 		return Learnable
 	# calculate the offset from the first cleric kit
 	KitIndex -= CommonTables.Classes.FindValue ("CLASS", BaseClassIndex+1)
@@ -439,13 +471,13 @@ def GetLearnablePriestSpells (Class, Alignment, Level, booktype=0):
 	v = CommonTables.Aligns.FindValue(3, Alignment)
 	#usability is the bitset we look for
 	Usability = CommonTables.Aligns.GetValue(v, 5)
-	SpellListTable = None
+
 	if GameCheck.IsIWD2():
 		row = CommonTables.ClassSkills.FindValue ("SPLTYPE", booktype)
 		rowName = CommonTables.ClassSkills.GetRowName (row)
 		Class = CommonTables.Classes.GetValue (rowName, "ID", GTV_INT)
 		spells = GetIWD2Spells (0, Usability, Level, Class)
-		spells = map(lambda e: e[0], spells) # ignore the second member
+		spells = [e[0] for e in filter(lambda s: s[1], spells)] # skip disallowed spells
 		return spells
 
 	SpellsTable = GemRB.LoadTable ("spells")
@@ -468,9 +500,8 @@ def GetPriestSpellTable(tablename):
 	if GameCheck.IsIWD2():
 		return tablename # no need for this folly
 
-	if not GemRB.HasResource (tablename, RES_2DA):
-		if tablename == "MXSPLDRU":
-			return "MXSPLPRS"
+	if not GemRB.HasResource (tablename, RES_2DA) and tablename == "MXSPLDRU":
+		return "MXSPLPRS"
 	return tablename
 
 def SetupSpellLevels (pc, TableName, Type, Level):
@@ -488,7 +519,7 @@ def SetupSpellLevels (pc, TableName, Type, Level):
 		# specialist mages get an extra spell if they already know that level
 		# FIXME: get a general routine to find specialists
 		school = GemRB.GetVar("MAGESCHOOL")
-		if (Type == IE_SPELL_TYPE_WIZARD and school != 0) or \
+		if (Type == IE_SPELL_TYPE_WIZARD and school is not None) or \
 			(GameCheck.IsIWD2() and Type == IE_IWD2_SPELL_WIZARD and not (kit&0x4000)):
 			if value > 0:
 				value += 1
@@ -533,12 +564,10 @@ def HasSorcererBook (pc, cls=-1):
 		ClassName = GUICommon.GetClassRowName (cls, "class")
 	SorcererBook = CommonTables.ClassSkills.GetValue (ClassName, "BOOKTYPE")
 	if SorcererBook == "*" or ClassName == "":
-		return False
-	return IsSorcererBook (SorcererBook)
-
-def CannotLearnSlotSpell ():
-	pc = GemRB.GameGetSelectedPCSingle ()
+		return 0
+	return SorcererBook if IsSorcererBook (SorcererBook) else 0
 
+def CannotLearnSlotSpell (slot_item, pc):
 	# disqualify sorcerers immediately
 	if HasSorcererBook (pc):
 		return LSR_STAT
@@ -547,12 +576,7 @@ def CannotLearnSlotSpell ():
 	if GameCheck.IsIWD2():
 		booktype = IE_IWD2_SPELL_WIZARD
 
-	if GameCheck.IsPST():
-		import GUIINV
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
-	else:
-		slot_item = GemRB.GetSlotItem (pc, GemRB.GetVar ("ItemButton"))
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
+	spell_ref = GemRB.GetItem (slot_item['ItemResRef'])['Spell']
 	spell = GemRB.GetSpell (spell_ref)
 	level = spell['SpellLevel']
 
@@ -563,17 +587,29 @@ def CannotLearnSlotSpell ():
 	if HasSpell (pc, booktype, level-1, spell_ref) != -1:
 		return LSR_KNOWN
 
-	# level check (needs enough intelligence for this level of spell)
+	# level check on core+ difficulties (needs enough intelligence for this level of spell)
 	dumbness = GemRB.GetPlayerStat (pc, IE_INT)
-	if level > GemRB.GetAbilityBonus (IE_INT, 1, dumbness):
+	hard = GemRB.GetVar ("Difficulty Level") >= 3
+	if hard and level > GemRB.GetAbilityBonus (IE_INT, 1, dumbness):
 		return LSR_LEVEL
 
 	spell_count = GemRB.GetKnownSpellsCount (pc, booktype, level-1)
-	if spell_count >= GemRB.GetAbilityBonus (IE_INT, 2, dumbness):
+	if not GameCheck.IsIWD2 () and spell_count >= GemRB.GetAbilityBonus (IE_INT, 2, dumbness):
 		return LSR_FULL
 
 	return 0
 
+def LearnPriestSpell (pc, spell, bookType, level):
+	if GameCheck.IsIWD2():
+		if bookType == IE_IWD2_SPELL_DOMAIN:
+			GemRB.LearnSpell (pc, spell, 0, bookType, level)
+		else:
+			# perhaps forcing would be fine here too, but it's untested and
+			# iwd2 cleric schools grant certain spells at different levels
+			GemRB.LearnSpell (pc, spell, 0, bookType)
+	else:
+		GemRB.LearnSpell (pc, spell)
+
 def LearnPriestSpells (pc, level, mask, baseClassName = -1):
 	"""Learns all the priest spells through the given spell level.
 
@@ -599,19 +635,11 @@ def LearnPriestSpells (pc, level, mask, baseClassName = -1):
 		for spell in learnable:
 			# if the spell isn't learned, learn it
 			if HasSpell (pc, booktype, i, spell) < 0:
-				if GameCheck.IsIWD2():
-					if booktype == IE_IWD2_SPELL_DOMAIN:
-						GemRB.LearnSpell (pc, spell, 0, 1<<booktype, i)
-					else:
-						# perhaps forcing would be fine here too, but it's untested and
-						# iwd2 cleric schools grant certain spells at different levels
-						GemRB.LearnSpell (pc, spell, 0, 1<<booktype)
-				else:
-					GemRB.LearnSpell (pc, spell)
+				LearnPriestSpell (pc, spell, booktype, i)
 	return
 
 
-def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
+def RemoveKnownSpells (pc, spelltype, level1=1, level2=1, noslots=0, kit=0):
 	"""Removes all known spells of a given type between two spell levels.
 
 	If noslots is true, all memorization counts are set to 0.
@@ -619,9 +647,9 @@ def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
 	this is only used when removing spells in a dualclass."""
 
 	# choose the correct limit based upon class type
-	if type == IE_SPELL_TYPE_WIZARD or GameCheck.IsIWD2():
+	if spelltype == IE_SPELL_TYPE_WIZARD or GameCheck.IsIWD2():
 		limit = 9
-	elif type == IE_SPELL_TYPE_PRIEST:
+	elif spelltype == IE_SPELL_TYPE_PRIEST:
 		limit = 7
 
 		# make sure that we get the original kit, if we have one
@@ -647,7 +675,7 @@ def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
 			# don't know how this would happen, but better to be safe
 			if originalkit == kit:
 				originalkit = 0
-	elif type == IE_SPELL_TYPE_INNATE:
+	elif spelltype == IE_SPELL_TYPE_INNATE:
 		limit = 1
 	else: # can't do anything if an improper spell type is sent
 		return 0
@@ -662,14 +690,14 @@ def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
 	# remove all spells for each level
 	for level in range (level1-1, level2):
 		# we need the count because we remove each spell in reverse order
-		count = GemRB.GetKnownSpellsCount (pc, type, level)
+		count = GemRB.GetKnownSpellsCount (pc, spelltype, level)
 		mod = count-1
 
 		for spell in range (count):
 			# see if we need to check for kit
-			if type == IE_SPELL_TYPE_PRIEST and kit:
+			if spelltype == IE_SPELL_TYPE_PRIEST and kit:
 				# get the spell's ref data
-				ref = GemRB.GetKnownSpell (pc, type, level, mod-spell)
+				ref = GemRB.GetKnownSpell (pc, spelltype, level, mod-spell)
 				ref = GemRB.GetSpell (ref['SpellResRef'], 1)
 
 				# we have to look at the originalkit as well specifically for ranger/cleric dual-classes
@@ -679,11 +707,11 @@ def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
 					continue
 
 			# remove the spell
-			GemRB.RemoveSpell (pc, type, level, mod-spell)
+			GemRB.RemoveSpell (pc, spelltype, level, mod-spell)
 
 		# remove memorization counts if desired
 		if noslots:
-			GemRB.SetMemorizableSpellsCount (pc, 0, type, level)
+			GemRB.SetMemorizableSpellsCount (pc, 0, spelltype, level)
 
 	# return success
 	return 1
@@ -695,7 +723,7 @@ def LearnSpell(pc, spellref, booktype, level, count, flags=0):
 	if SpellIndex < 0:
 		ret = GemRB.LearnSpell (pc, spellref, flags, booktype)
 		if ret != LSR_OK and ret != LSR_KNOWN:
-			raise RuntimeError, "Failed learning spell: %s !" %(spellref)
+			raise RuntimeError ("Failed ({}) learning spell: {}".format(ret, spellref))
 
 		SpellIndex = HasSpell (pc, booktype, level, spellref)
 		count -= 1
@@ -705,18 +733,25 @@ def LearnSpell(pc, spellref, booktype, level, count, flags=0):
 
 	if SpellIndex == -1:
 		# should never happen
-		raise RuntimeError, "LearnSpell: Severe spellbook problems: %s !" %(spellref)
+		raise RuntimeError("LearnSpell: Severe spellbook problems: " + str(spellref))
 
 	for j in range(count):
 		GemRB.MemorizeSpell (pc, booktype, level, SpellIndex, flags&LS_MEMO)
 
 def LearnFromScroll (pc, slot):
 	slot_item = GemRB.GetSlotItem (pc, slot)
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
+	spell_ref = GemRB.GetItem (slot_item['ItemResRef'])['Spell']
 
-	ret = GemRB.LearnSpell (pc, spell_ref, LS_STATS|LS_ADDXP)
+	bookType = -1
+	if GameCheck.IsIWD2 () and GemRB.GetPlayerStat (pc, IE_LEVELMAGE):
+		bookType = IE_IWD2_SPELL_WIZARD
+	ret = GemRB.LearnSpell (pc, spell_ref, LS_STATS | LS_ADDXP, bookType)
 
 	# destroy the scroll, just one in case of a stack
 	GemRB.RemoveItem (pc, slot, 1)
 
+	# bg2 tutorial needs this
+	if slot_item["ItemResRef"] == "ttscrl01":
+		GemRB.SetGlobal ("TutorialLesson", "GLOBAL", 20)
+
 	return ret
