@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2010 The GemRB Project
 #
@@ -20,9 +19,8 @@
 
 import GemRB
 from ie_restype import RES_2DA
-from ie_sounds import CHAN_GUI
+from ie_sounds import CHAN_GUI, SND_SPEECH, GEM_SND_VOL_AMBIENTS
 from GUIDefines import *
-import GUICommon
 import GameCheck
 
 TextScreen = None
@@ -46,7 +44,7 @@ def StartTextScreen ():
 	global TextScreen, TextArea, TableName, Row
 
 	# for easier development
-	if GameCheck.IsGemRBDemo () and GemRB.GetVar ("SkipIntroVideos"):
+	if GameCheck.IsGemRBDemo () and GemRB.GetVar ("SkipIntroVideos") and not GemRB.GetGameVar ("completed"):
 		GemRB.GamePause (0, 3)
 		return
 
@@ -70,6 +68,7 @@ def StartTextScreen ():
 	elif GameCheck.IsGemRBDemo ():
 		#TODO: set MusicName
 		TableName = "chapters"
+		Message = "*"
 
 	if TableName == "":
 		EndTextScreen ()
@@ -79,21 +78,16 @@ def StartTextScreen ():
 		TextTable = GemRB.LoadTable ("textscrn", 1)
 		if TextTable != None:
 			TxtRow = TextTable.GetRowIndex (TableName)
-			if TxtRow >= 0:
+			if TxtRow is not None:
 				ID = TextTable.GetValue (TxtRow, 0)
 				MusicName = TextTable.GetValue (TxtRow, 1)
 				Message = TextTable.GetValue (TxtRow, 2)
 
 	if Message != "*":
-		GemRB.DisplayString (Message, 0xff0000)
-
-	if GameCheck.IsIWD2 () or GameCheck.IsGemRBDemo ():
-		GemRB.LoadWindowPack ("GUICHAP", 800, 600)
-	else:
-		GemRB.LoadWindowPack ("GUICHAP", 640, 480)
+		GemRB.DisplayString (Message, ColorRed)
 
 	Table = GemRB.LoadTable (TableName)
-	if GameCheck.IsBG2():
+	if GameCheck.IsBG2OrEE ():
 		LoadPic = Table.GetValue (-1, -1)
 		if LoadPic.startswith ("*"): # BG2 epilogues
 			ID = 63
@@ -105,20 +99,25 @@ def StartTextScreen ():
 		ID = GemRB.GetGameVar("CHAPTER") & 0x7fffffff
 		Chapter = ID + 1
 
+	# Textscreen should always stop currently playing music before starting
+	# to play any chapter music.
+	GemRB.HardEndPL ()
+
 	if MusicName != "*":
 		GemRB.LoadMusicPL (MusicName + ".mus")
-	else:
-		GemRB.HardEndPL ()
-
-	TextScreen = GemRB.LoadWindow (ID)
-	TextScreen.SetFrame ()
 
+	TextScreen = GemRB.LoadWindow (ID, "GUICHAP")
 	TextArea = TextScreen.GetControl (2)
+	TextArea.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
+	TextArea.SetColor (ColorWhitish, TA_COLOR_INITIALS)
+
+	GameWin = GemRB.GetView("GAMEWIN")
+	GameWin.SetDisabled(True)
 
 	if GameCheck.IsBG1():
 		#these suckers couldn't use a fix row
 		FindTextRow (Table)
-	elif GameCheck.IsBG2():
+	elif GameCheck.IsBG2OrEE ():
 		FindTextRow (Table)
 		if LoadPic != "":
 			if ID == 63:
@@ -127,15 +126,15 @@ def StartTextScreen ():
 				PicButton.SetPicture (LoadPic)
 				PicButton.SetState (IE_GUI_BUTTON_LOCKED)
 			else:
-				TextScreen.SetPicture (LoadPic)
+				TextScreen.SetBackground (LoadPic)
 	else:
 		Row = Chapter
 
 	#caption
 	Value = Table.GetValue (Row, 0)
 	#don't display the fake -1 string (No caption in toscst.2da)
-	if Value!="NONE" and Value>0 and TextScreen.HasControl(0x10000000):
-		Label=TextScreen.GetControl (0x10000000)
+	Label = TextScreen.GetControl (0x10000000)
+	if Label and Value!="NONE" and Value>0:
 		Label.SetText (Value)
 
 	#done
@@ -144,8 +143,8 @@ def StartTextScreen ():
 		Button.SetText (84)
 	else:
 		Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, EndTextScreen)
-	Button.SetFlags (IE_GUI_BUTTON_DEFAULT|IE_GUI_BUTTON_CANCEL,OP_OR)
+	Button.OnPress (EndTextScreen)
+	Button.MakeDefault()
 
 	#replay
 	Button=TextScreen.GetControl (3)
@@ -153,13 +152,12 @@ def StartTextScreen ():
 		Button.SetText (85)
 	else:
 		Button.SetText (16510)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ReplayTextScreen)
+	Button.OnPress (ReplayTextScreen)
 
 	#if this was opened from somewhere other than game control close that window
-	GUICommon.CloseOtherWindow(None)
-	GemRB.HideGUI ()
-	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE) #removing the gamecontrol screen
-	TextScreen.SetVisible (WINDOW_VISIBLE)
+	import GUICommonWindows
+	GUICommonWindows.CloseTopWindow ()
+
 	TextScreen.ShowModal (MODAL_SHADOW_NONE)
 
 	ReplayTextScreen()
@@ -176,19 +174,18 @@ def ToggleAmbients (state):
 		AmbientVolume = GemRB.GetVar ("Volume Ambients")
 		GemRB.SetVar ("Volume Ambients", 0)
 
-	GemRB.UpdateAmbientsVolume ()
+	GemRB.UpdateVolume (GEM_SND_VOL_AMBIENTS)
 
 def EndTextScreen ():
 	global TextScreen, TableName
 
 	if TextScreen:
-		TextScreen.SetVisible (WINDOW_INVISIBLE)
-		TextScreen.Unload ()
+		TextScreen.Close ()
 		GemRB.HardEndPL ()
-		GemRB.PlaySound(None, CHAN_GUI, 0, 0, 4)
+		GemRB.PlaySound(None, CHAN_GUI, 0, 0, SND_SPEECH)
 
-	GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE) # enable the gamecontrol screen
-	GemRB.UnhideGUI ()
+	GameWin = GemRB.GetView("GAMEWIN")
+	GameWin.SetDisabled(False)
 	ToggleAmbients (1)
 	GemRB.GamePause (0, 3)
 
@@ -199,8 +196,7 @@ def EndTextScreen ():
 def ReplayTextScreen ():
 	global TextArea, TableName, Row
 
-	# stop any current speech, flag of 5 = GEM_SND_RELATIVE|GEM_SND_SPEECH
-	GemRB.PlaySound(None, CHAN_GUI, 0, 0, 5)
+	GemRB.PlaySound(None, CHAN_GUI, 0, 0, SND_SPEECH)
 
 	Table = GemRB.LoadTable (TableName)
 	Count = Table.GetColumnCount (Row)
@@ -216,6 +212,6 @@ def ReplayTextScreen ():
 		text += "\n\n" if not GameCheck.IsGemRBDemo () else ""
 		text += GemRB.GetString(Table.GetValue (Row, i), 14)
 
-	TextArea.ChapterText (text)
+	TextArea.SetChapterText (text)
 
 	return
