@@ -50,16 +50,16 @@ def CalcLimits(Abidx):
 	Maximum = 18
 
 	tmp = Abclasrq.GetValue(KitIndex, Abidx)
-	if tmp!=0 and tmp>Minimum:
+	if tmp > Minimum:
 		Minimum = tmp
 
 	Race = Abracerq.GetRowIndex(RaceName)
 	tmp = Abracerq.GetValue(Race, Abidx*2)
-	if tmp!=0 and tmp>Minimum:
+	if tmp > Minimum:
 		Minimum = tmp
 
 	tmp = Abracerq.GetValue(Race, Abidx*2+1)
-	if tmp!=0 and tmp>Maximum:
+	if tmp > Maximum:
 		Maximum = tmp
 
 	Race = Abracead.GetRowIndex(RaceName)
@@ -81,7 +81,6 @@ def RollPress():
 	PointsLeft = 0
 	SumLabel = AbilityWindow.GetControl(0x10000002)
 	SumLabel.SetText("0")
-	SumLabel.SetUseRGB(1)
 
 	if HasStrExtra:
 		e = GemRB.Roll(1,100,0)
@@ -103,7 +102,6 @@ def RollPress():
 			Label.SetText("18/"+str(e) )
 		else:
 			Label.SetText(str(v) )
-		Label.SetUseRGB(1)
 	return
 
 def OnLoad():
@@ -112,14 +110,7 @@ def OnLoad():
 	global AbilityTable, Abclasrq, Abclsmod, Abracerq, Abracead
 	global KitIndex, Minimum, Maximum, MyChar
 
-	if GUICommon.CloseOtherWindow (OnLoad):
-		if(AbilityWindow):
-			AbilityWindow.Unload()
-			AbilityWindow = None
-		return
-
 	Abracead = GemRB.LoadTable("ABRACEAD")
-	#Abclsmod = GemRB.LoadTable("ABCLSMOD")
 	Abclasrq = GemRB.LoadTable("ABCLASRQ")
 	Abracerq = GemRB.LoadTable("ABRACERQ")
 
@@ -137,9 +128,8 @@ def OnLoad():
 
 	KitIndex = Abclasrq.GetRowIndex(KitName)
 
-	GemRB.LoadWindowPack("GUICG", 640, 480)
 	AbilityTable = GemRB.LoadTable("ability")
-	AbilityWindow = GemRB.LoadWindow(4)
+	AbilityWindow = GemRB.LoadWindow(4, "GUICG")
 
 	RerollButton = AbilityWindow.GetControl(2)
 	RerollButton.SetText(11982)
@@ -150,37 +140,38 @@ def OnLoad():
 
 	BackButton = AbilityWindow.GetControl(36)
 	BackButton.SetText(15416)
-	BackButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	BackButton.MakeEscape()
 	DoneButton = AbilityWindow.GetControl(0)
 	DoneButton.SetText(11973)
-	DoneButton.SetFlags(IE_GUI_BUTTON_DEFAULT,OP_OR)
+	DoneButton.MakeDefault()
 	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
 
 	RollPress()
 	StorePress()
 	for i in range(6):
 		Button = AbilityWindow.GetControl(i+30)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, JustPress)
+		Button.OnPress (JustPress)
 		Button.SetVarAssoc("Ability", i)
 
 		Button = AbilityWindow.GetControl(i*2+16)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, LeftPress)
+		Button.OnPress (LeftPress)
 		Button.SetVarAssoc("Ability", i )
+		Button.SetActionInterval (200)
 
 		Button = AbilityWindow.GetControl(i*2+17)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, RightPress)
+		Button.OnPress (RightPress)
 		Button.SetVarAssoc("Ability", i )
+		Button.SetActionInterval (200)
 
 	TextAreaControl = AbilityWindow.GetControl(29)
 	TextAreaControl.SetText(17247)
 
-	StoreButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, StorePress)
-	RecallButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, RecallPress)
-	RerollButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, RollPress)
-	DoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, NextPress)
-	BackButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, BackPress)
-	AbilityWindow.ShowModal(MODAL_SHADOW_NONE)
-	GemRB.SetRepeatClickFlags(GEM_RK_DISABLE, OP_NAND)
+	StoreButton.OnPress (StorePress)
+	RecallButton.OnPress (RecallPress)
+	RerollButton.OnPress (RollPress)
+	DoneButton.OnPress (NextPress)
+	BackButton.OnPress (lambda: CharGenCommon.back(AbilityWindow))
+	AbilityWindow.ShowModal(MODAL_SHADOW_GRAY)
 	return
 
 def RightPress():
@@ -209,7 +200,6 @@ def RightPress():
 
 def JustPress():
 	Abidx = GemRB.GetVar("Ability")
-	Ability = GemRB.GetVar("Ability "+str(Abidx) )
 	CalcLimits(Abidx)
 	GemRB.SetToken("MINIMUM",str(Minimum) )
 	GemRB.SetToken("MAXIMUM",str(Maximum) )
@@ -270,28 +260,17 @@ def RecallPress():
 	PointsLeft = GemRB.GetVar("Ability -1")
 	return
 
-def BackPress():
-	GemRB.SetRepeatClickFlags(GEM_RK_DISABLE, OP_OR)
-	CharGenCommon.back()
-
 def NextPress():
-	GemRB.SetRepeatClickFlags(GEM_RK_DISABLE, OP_OR)
-	
+	AbilityWindow.Close()
 	AbilityTable = GemRB.LoadTable ("ability")
 	AbilityCount = AbilityTable.GetRowCount ()
-	
-	# print our diagnostic as we loop (so as not to duplicate)
+
 	for i in range (AbilityCount):
 		StatID = AbilityTable.GetValue (i, 3)
-		StatName = AbilityTable.GetRowName (i)
 		StatValue = GemRB.GetVar ("Ability "+str(i))
 		GemRB.SetPlayerStat (MyChar, StatID, StatValue)
-		print "\t",StatName,":\t", StatValue
 
 	GemRB.SetPlayerStat (MyChar, IE_STREXTRA, GemRB.GetVar ("StrExtra"))
-	print "\tSTREXTRA:\t",GemRB.GetVar ("StrExtra")
-
-        GemRB.SetRepeatClickFlags(GEM_RK_DISABLE, OP_OR)
 
 	CharGenCommon.next()
 
