@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -23,7 +22,7 @@ from ie_stats import IE_ALIGNMENT, IE_REPUTATION
 
 def OnLoad():
 	MyChar = GemRB.GetVar ("Slot")
-	GemRB.SetVar ("Alignment", -1) #alignment
+	GemRB.SetVar ("Alignment", None) #alignment
 	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT, 0)
 	GemRB.SetPlayerStat (MyChar, IE_REPUTATION, 0)
 
