@@ -23,6 +23,8 @@ import CommonTables
 from ie_stats import *
 from GUIDefines import *
 
+import CharGenCommon
+
 AlignmentWindow = 0
 TextAreaControl = 0
 DoneButton = 0
@@ -42,8 +44,8 @@ def OnLoad():
 
 	AlignmentOk = GemRB.LoadTable("ALIGNMNT")
 
-	GemRB.LoadWindowPack("GUICG", 640, 480)
-	AlignmentWindow = GemRB.LoadWindow(3)
+	AlignmentWindow = GemRB.LoadWindow(3, "GUICG")
+	CharGenCommon.PositionCharGenWin(AlignmentWindow)
 
 	for i in range(9):
 		Button = AlignmentWindow.GetControl(i+2)
@@ -53,63 +55,46 @@ def OnLoad():
 
 		if AlignmentOk.GetValue(KitName, CommonTables.Aligns.GetValue (i, 4)) != 0:
 			Button.SetState(IE_GUI_BUTTON_ENABLED)
-			Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, AlignmentPress)
+			Button.OnPress (AlignmentPress)
 			Button.SetVarAssoc("Alignment", i)
 
 	BackButton = AlignmentWindow.GetControl(13)
 	BackButton.SetText(15416)
-	BackButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	BackButton.MakeEscape()
 	DoneButton = AlignmentWindow.GetControl(0)
 	DoneButton.SetText(11973)
-	DoneButton.SetFlags(IE_GUI_BUTTON_DEFAULT,OP_OR)
+	DoneButton.MakeDefault()
 
 	TextAreaControl = AlignmentWindow.GetControl(11)
 	TextAreaControl.SetText(9602)
 
-	DoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, NextPress)
-	BackButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, BackPress)
-	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
-	AlignmentWindow.SetVisible(WINDOW_VISIBLE)
+	DoneButton.OnPress (NextPress)
+	BackButton.OnPress (BackPress)
+	DoneButton.SetDisabled(True)
+	AlignmentWindow.Focus()
 	return
 
 def AlignmentPress():
 	Alignment = GemRB.GetVar("Alignment")
 	TextAreaControl.SetText (CommonTables.Aligns.GetValue (Alignment, 1))
-	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
+	DoneButton.SetDisabled(False)
 	GemRB.SetVar ("Alignment", CommonTables.Aligns.GetValue (Alignment, 3))
 	return
 
 def BackPress():
 	if AlignmentWindow:
-		AlignmentWindow.Unload()
+		AlignmentWindow.Close ()
 	GemRB.SetVar("Alignment",-1)  #scrapping the alignment value
 	GemRB.SetNextScript("CharGen4")
 	return
 
 def NextPress():
 	if AlignmentWindow:
-		AlignmentWindow.Unload()
+		AlignmentWindow.Close ()
 	# save previous stats:
 	#       alignment
-	#       reputation
-	#       alignment abilities
 	Alignment = GemRB.GetVar ("Alignment")
 	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT, Alignment)
 
-	# use the alignment to apply starting reputation
-	RepTable = GemRB.LoadTable ("repstart")
-	AlignmentAbbrev = CommonTables.Aligns.FindValue (3, Alignment)
-	Rep = RepTable.GetValue (AlignmentAbbrev, 0) * 10
-	GemRB.SetPlayerStat (MyChar, IE_REPUTATION, Rep)
-
-	# set the party rep if this in the main char
-	if MyChar == 1:
-		GemRB.GameSetReputation (Rep)
-
-	# diagnostic output
-	print "CharGen5 output:"
-	print "\tAlignment: ",Alignment
-	print "\tReputation: ",Rep
-
 	GemRB.SetNextScript("CharGen5") #appearance
 	return
