@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -21,6 +20,7 @@ import GemRB
 from GUIDefines import *
 from ie_stats import *
 from ie_spells import LS_MEMO
+import GameCheck
 import GUICommon
 import Spellbook
 import CommonTables
@@ -38,6 +38,9 @@ Classes = []		# << classes (ids)
 Level = []		# << levels for each class
 EnhanceGUI = 0		# << toggle for scrollbar and 25th hla slot
 
+# setup our scroll index
+GemRB.SetVar("HLATopIndex", 0)
+
 def OpenHLAWindow (actor, numclasses, classes, levels):
 	"""Opens the HLA selection window."""
 
@@ -48,7 +51,7 @@ def OpenHLAWindow (actor, numclasses, classes, levels):
 	if (GemRB.GetVar("GUIEnhancements")&GE_SCROLLBARS):
 		EnhanceGUI = 1
 
-	# save our variables 
+	# save our variables
 	pc = actor
 	NumClasses = numclasses
 	Classes = classes
@@ -56,7 +59,7 @@ def OpenHLAWindow (actor, numclasses, classes, levels):
 	HLACount = GemRB.GetVar ("HLACount")
 
 	# we use the same window as sorcerer spell selection
-	HLAWindow = GemRB.LoadWindow (8)
+	HLAWindow = GemRB.LoadWindow (8, "GUIREC")
 
 	# get all our HLAs (stored in HLAAbilities)
 	GetHLAs ()
@@ -66,34 +69,38 @@ def OpenHLAWindow (actor, numclasses, classes, levels):
 	TitleLabel.SetText (63818)
 
 	# create the done button
-	HLADoneButton = HLAWindow.GetControl (28)
-	HLADoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, HLADonePress)
-	HLADoneButton.SetText(11973)
-	HLADoneButton.SetFlags(IE_GUI_BUTTON_DEFAULT, OP_OR)
-	if HLACount:
-		HLADoneButton.SetState(IE_GUI_BUTTON_DISABLED)
+	if GameCheck.IsBG2EE ():
+		HLADoneButton = HLAWindow.GetControl (42)
 	else:
-		HLADoneButton.SetState(IE_GUI_BUTTON_ENABLED)
+		HLADoneButton = HLAWindow.GetControl (28)
+	HLADoneButton.OnPress (HLADonePress)
+	HLADoneButton.SetText(11973)
+	HLADoneButton.MakeDefault()
+	if HLACount:
+		HLADoneButton.SetDisabled(True)
+	else:
+		HLADoneButton.SetDisabled(False)
 
 	# setup our text area
-	HLATextArea = HLAWindow.GetControl(26)
+	if GameCheck.IsBG2EE ():
+		HLATextArea = HLAWindow.GetControl (41)
+	else:
+		HLATextArea = HLAWindow.GetControl (26)
 
-	print "Number of HLAs:",len (HLAAbilities)
+	print("Number of HLAs:",len (HLAAbilities))
 
 	# create a scrollbar if need-be
 	if ( len (HLAAbilities) >= 25 ) and EnhanceGUI:
 		# setup extra 25th HLA slot:
 		HLAWindow.CreateButton (24, 231, 345, 42, 42)
 		if ( len (HLAAbilities) > 25):
-			# setup our scroll index
-			GemRB.SetVar("HLATopIndex", 0)
 			# setup scrollbar
-			HLAWindow.CreateScrollBar (1000, 290,142, 16,252, "GUISCRCW")
-			ScrollBar = HLAWindow.GetControl (1000)
-			ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, HLAShowAbilities)
+			ScrollBar = HLAWindow.CreateScrollBar (1000, {'x' : 290, 'y' : 142, 'w' : 16, 'h' : 252}, "GUISCRCW")
+			ScrollBar.OnChange (HLAShowAbilities)
 			#with enhanced GUI we have 5 rows of 5 abilities (the last one is 'the extra slot')
-			ScrollBar.SetVarAssoc ("HLATopIndex", GUICommon.ceildiv ( ( len (HLAAbilities)-25 ) , 5 ) + 1 )
-			ScrollBar.SetDefaultScrollBar ()
+			count = GUICommon.ceildiv (len(HLAAbilities) - 25, 5) + 1
+			ScrollBar.SetVarAssoc ("HLATopIndex", count, 0, count)
+			HLAWindow.SetEventProxy(ScrollBar)
 
 	# draw our HLAs and show the window
 	HLAShowAbilities ()
@@ -131,19 +138,15 @@ def HLADonePress ():
 			Spellbook.LearnSpell (pc, HLARef[3:], HLAType, HLALevel, 1, LS_MEMO)
 
 		#save the number of this HLA memorized
-		#TODO: check param2 (0 seems to work ok)
 		GemRB.ApplyEffect(pc, "HLA", HLAAbilities[i][2], 0, HLARef[3:])
 
 	# close the window
 	if HLAWindow:
-		HLAWindow.Unload ()
+		HLAWindow.Close ()
 
 	# so redraw skills knows we're done
 	GemRB.SetVar ("HLACount", 0)
 
-	import LevelUp
-	LevelUp.LevelUpWindow.Invalidate ()
-
 	return
 
 def HLAShowAbilities ():
@@ -160,11 +163,11 @@ def HLAShowAbilities ():
 			break
 		SpellButton = HLAWindow.GetControl (i)
 		if i + j >= len (HLAAbilities):
-			SpellButton.SetState (IE_GUI_BUTTON_DISABLED)
+			SpellButton.SetDisabled(True)
 			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 			continue
 		else:
-			SpellButton.SetState(IE_GUI_BUTTON_ENABLED)
+			SpellButton.SetDisabled(False)
 			SpellButton.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
 
 		# fill in the button with the spell data
@@ -175,7 +178,7 @@ def HLAShowAbilities ():
 		SpellButton.SetTooltip(Spell['SpellName'])
 		SpellButton.SetSpellIcon(HLARef, 1)
 		SpellButton.SetVarAssoc("ButtonPressed", i)
-		SpellButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, HLASelectPress)
+		SpellButton.OnPress (HLASelectPress)
 		SpellButton.SetSprites("GUIBTBUT", 0,0,1,2,3)
 		SpellButton.SetFlags(IE_GUI_BUTTON_PICTURE, OP_OR)
 
@@ -183,12 +186,13 @@ def HLAShowAbilities ():
 		if HLAAbilities[i+j][1] == 0:
 			SpellButton.SetState(IE_GUI_BUTTON_LOCKED)
 			# shade red
-			SpellButton.SetBorder (0, 0,0, 0,0, 200,0,0,100, 1,1)
+			color = {'r' : 200, 'g' : 0, 'b' : 0, 'a' : 100}
+			SpellButton.SetBorder (0, color, 1,1)
 		else:
 			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
 			# unset any borders on this button or an un-learnable from last level
 			# will still shade red even though it is clickable
-			SpellButton.SetBorder (0, 0,0, 0,0, 0,0,0,0, 0,0)
+			SpellButton.SetBorder (0, None, 0,0)
 
 	# show which spells are selected
 	HLAShowSelectedAbilities ()
@@ -227,7 +231,7 @@ def HLASelectPress ():
 			HLACount += 1
 			HLANewAbilities[i] = 0
 			HLAAbilities[i][2] -= 1 # internal counter
-			HLADoneButton.SetState (IE_GUI_BUTTON_DISABLED)
+			HLADoneButton.SetDisabled (True)
 		else: # selecting
 			# we don't have any picks left
 			if HLACount == 0:
@@ -239,7 +243,7 @@ def HLASelectPress ():
 			HLANewAbilities[i] = 1
 			HLAAbilities[i][2] += 1 # increment internal counter
 			if HLACount == 0:
-				HLADoneButton.SetState (IE_GUI_BUTTON_ENABLED)
+				HLADoneButton.SetDisabled (False)
 
 		# recheck internal exclusions and prereqs
 		HLARecheckPrereqs (i)
@@ -278,16 +282,16 @@ def HLAMarkButton (i, select):
 	j = ( GemRB.GetVar("HLATopIndex") + 1 ) * 5 - 5
 
 	if select:
-		type = IE_GUI_BUTTON_SELECTED
+		state = IE_GUI_BUTTON_SELECTED
 	else:
 		if HLAAbilities[i][1]:
-			type = IE_GUI_BUTTON_ENABLED
+			state = IE_GUI_BUTTON_ENABLED
 		else: # can't learn
-			type = IE_GUI_BUTTON_LOCKED
+			state = IE_GUI_BUTTON_LOCKED
 
 	# we have to use the index on the actual grid
 	SpellButton = HLAWindow.GetControl(i-j)
-	SpellButton.SetState(type)
+	SpellButton.SetState(state)
 	return
 
 def GetHLAs ():
@@ -325,17 +329,17 @@ def GetHLAs ():
 
 		# actually load the table
 		HLAClassTable = GemRB.LoadTable (HLAClassTable)
-		print "HLA Class/Kit:",ClassName
+		print("HLA Class/Kit:", ClassName)
 
 		# save all our HLAs from this class
 		for j in range (HLAClassTable.GetRowCount ()):
 			HLARef = HLAClassTable.GetValue (j, 0, GTV_STR)
-			print "\tHLA",j,":",HLARef
+			print("\tHLA " + str(j) +  ": " + HLARef)
 
 			# make sure we have an ability here
 			if HLARef == "*":
-				print "\t\tEnd of HLAs"
-				break	
+				print("\t\tEnd of HLAs")
+				break
 
 			# [ref to hla, memorizable?, num memorized, pre-req ref, excluded ref]
 			SaveArray = [\
@@ -345,51 +349,51 @@ def GetHLAs ():
 				HLAClassTable.GetValue (j, 6, GTV_STR),\
 				HLAClassTable.GetValue (j, 7, GTV_STR)]
 
-			# make sure we fall within the min and max paramaters
+			# make sure we fall within the min and max parameters
 			if HLAClassTable.GetValue (j, 3) > CurrentLevel or HLAClassTable.GetValue (j, 4) < CurrentLevel:
-				print "\t\tNot within parameters"
+				print("\t\tNot within parameters")
 				HLAAbilities.append(SaveArray)
 				continue
-		
+
 			# see if we're alignment restricted (we never get them)
 			HLAAlign = HLAClassTable.GetValue (j, 8, GTV_STR)
 			if HLAAlign == "ALL_EVIL" and GemRB.GetPlayerStat (pc, IE_ALIGNMENT) < 6:
 				# don't even save this one because we can never get it
-				print "\t\tNeeds ALL_EVIL"
+				print("\t\tNeeds ALL_EVIL")
 				continue
 			elif HLAAlign == "ALL_GOOD" and GemRB.GetPlayerStat (pc, IE_ALIGNMENT) > 2:
 				# ditto
-				print "\t\tNeeds ALL_GOOD"
+				print("\t\tNeeds ALL_GOOD")
 				continue
 
 			# make sure we haven't already surpassed the number of time memorizable
 			HLANumAllowed = HLAClassTable.GetValue (j, 5)
-			print "\t\tHLA count:",SaveArray[2]
+			print("\t\tHLA count: " + str(SaveArray[2]))
 			if SaveArray[2] >= HLANumAllowed:
-				print "\t\tOnly allowed to learn",HLANumAllowed,"times"
+				print("\t\tOnly allowed to learn " + str(HLANumAllowed) + " times")
 				HLAAbilities.append(SaveArray)
 				continue
 
 			# make sure we haven't learned an HLA that excludes this one
 			HLAMemorized = GemRB.CountEffects (pc, "HLA", -1, -1, SaveArray[4][3:])
-			print "\t\tHLAExcluded count:",HLAMemorized
+			print("\t\tHLAExcluded count: " + str(HLAMemorized))
 			if (SaveArray[4] != "*") and (HLAMemorized > 0):
-				print "\t\tExcluded by:",SaveArray[4]
+				print("\t\tExcluded by: " + str(SaveArray[4]))
 				HLAAbilities.append(SaveArray)
 				continue
 
 			# we meet the prereqs so we can learn the HLA
 			HLAMemorized = GemRB.CountEffects (pc, "HLA", -1, -1, SaveArray[3][3:])
-			print "\t\tHLAPre count:",HLAMemorized
+			print("\t\tHLAPre count: " + str(HLAMemorized))
 			if (SaveArray[3] == "*") or (HLAMemorized > 0):
-				print "\t\tWe can learn it!"
+				print("\t\tWe can learn it!")
 				MaxHLACount += 1
 				SaveArray[1] = 1
 				HLAAbilities.append (SaveArray)
 				continue
 
 			# we didn't meet prereqs :(
-			print "\t\tNeed pre-req:",SaveArray[3]
+			print("\t\tNeed pre-req: " + str(SaveArray[3]))
 			HLAAbilities.append (SaveArray)
 
 	# create an array to store our abilities as they are selected
@@ -403,9 +407,9 @@ def GetHLAs ():
 	return
 
 def HLARecheckPrereqs (index):
-	"""Rechecks the HLA prequisites on the index on the fly."""
+	"""Rechecks the HLA prerequisites on the index on the fly."""
 
-	# the numer of times memorized
+	# the number of times memorized
 	Ref = HLAAbilities[index][0]
 	Memorized = HLAAbilities[index][2]
 
@@ -436,7 +440,7 @@ def HLARecheckPrereqs (index):
 						if (HLAAbilities[j][0] == HLAAbilities[i][4]) and (HLAAbilities[j][2] <= 0): # can learn
 							HLAAbilities[i][1] = 1
 							break
-				else: # no exlusions
+				else: # no exclusions
 					HLAAbilities[i][1] = 1
 			else: # prereqs not met
 				HLAAbilities[i][1] = 0
