@@ -47,3 +47,14 @@ CHAN_AMB_NONL = "AMBIENTN"
 CHAN_WALK_CHAR = "WALKINGC"
 CHAN_WALK_MONSTER = "WALKINGM"
 CHAN_ARMOR = "ARMOR"
+
+# sound handling flags, from Audio.h
+SND_SPATIAL = 1
+SND_LOOPING = 2
+SND_SPEECH = 4
+SND_QUEUE = 8
+SND_EFX = 16
+
+# from Audio.h
+GEM_SND_VOL_MUSIC = 1
+GEM_SND_VOL_AMBIENTS = 2
