@@ -30,7 +30,7 @@ import IDLUCommon
 BioWindow = 0
 BioData = 0
 BioStrRef = 0
-EditControl = 0
+EditControl = None
 PortraitName = ""
 
 def OnLoad():
@@ -74,6 +74,7 @@ def ClearPress():
 
 	GemRB.SetToken("BIO", "")
 	EditControl.SetText (GemRB.GetToken("BIO") )
+	EditControl.Focus ()
 	return
 
 def RevertPress():
@@ -83,42 +84,44 @@ def RevertPress():
 	ClassName = GUICommon.GetClassRowName (MyChar)
 	BioStrRef = BioTable.GetValue (ClassName, "BIO")
 	GemRB.SetToken ("BIO", GemRB.GetString(BioStrRef) )
-	if type (EditControl) != type (7350): # just some int
+	if EditControl: # just some int
 		EditControl.SetText (GemRB.GetToken("BIO") )
 	return
 
 def BioCancelPress():
 	GemRB.SetToken("BIO",BioData)
 	if BioWindow:
-		BioWindow.Unload ()
+		BioWindow.Close ()
 	return
 
 def BioDonePress():
 	GemRB.SetToken ("BIO", EditControl.QueryText())
 	if BioWindow:
-		BioWindow.Unload ()
+		BioWindow.Close ()
 	return
 
 def BioPress():
 	global BioWindow, EditControl, BioData
 
 	BioData = GemRB.GetToken("BIO")
-	BioWindow = Window = GemRB.LoadWindow (51)
+	BioWindow = Window = GemRB.LoadWindow (51, "GUICG")
 	Button = Window.GetControl (5)
 	Button.SetText (2240)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RevertPress)
+	Button.OnPress (RevertPress)
 
 	Button = Window.GetControl (6)
 	Button.SetText (18622)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ClearPress)
+	Button.OnPress (ClearPress)
 
 	Button = Window.GetControl (1)
 	Button.SetText (11962)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, BioDonePress)
+	Button.OnPress (BioDonePress)
 
 	Button = Window.GetControl (2)
 	Button.SetText (36788)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, BioCancelPress)
+	Button.OnPress (BioCancelPress)
+
+	Window.DeleteControl (3)
 
 	EditControl = Window.GetControl (4)
 	BioData = GemRB.GetToken("BIO")
@@ -127,7 +130,7 @@ def BioPress():
 	else:
 		EditControl.SetText (BioData )
 	Window.ShowModal (MODAL_SHADOW_GRAY)
-	EditControl.SetStatus (IE_GUI_CONTROL_FOCUSED)
+	EditControl.Focus()
 	return
 
 def SaveBio():
@@ -184,13 +187,12 @@ def NextPress():
 	GemRB.SetPlayerStat (MyChar, IE_HITPOINTS, GemRB.HasFeat (MyChar, FEAT_TOUGHNESS) * 3, 0)
 	LUCommon.SetupHP (MyChar)
 
-	t=GemRB.GetVar ("Alignment")
-	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT, CommonTables.Aligns.GetValue (t, 3))
-	TmpTable=GemRB.LoadTable ("repstart")
-	#t=CommonTables.Aligns.FindValue (3,t)
-	t=TmpTable.GetValue (t,0)
+	# must be set, see #2091
+	TmpTable = GemRB.LoadTable ("repstart")
+	t = TmpTable.GetValue (GemRB.GetVar ("Alignment") - 1, 0) * 10
 	GemRB.SetPlayerStat (MyChar, IE_REPUTATION, t)
-	TmpTable=GemRB.LoadTable ("strtgold")
+
+	TmpTable = GemRB.LoadTable ("strtgold")
 	a = TmpTable.GetValue (Class, 1) #number of dice
 	b = TmpTable.GetValue (Class, 0) #size
 	c = TmpTable.GetValue (Class, 2) #adjustment
@@ -203,7 +205,7 @@ def NextPress():
 		e=0
 	t = GemRB.Roll(a,b,c)*d+e
 	GemRB.SetPlayerStat (MyChar, IE_GOLD, t)
-	GemRB.SetPlayerStat (MyChar, IE_HATEDRACE, GemRB.GetVar ("HatedRace") )
+	GemRB.SetPlayerStat (MyChar, IE_HATEDRACE, GemRB.GetVar ("HatedRace") or 0)
 
 	GUICommon.SetColorStat (MyChar, IE_HAIR_COLOR, GemRB.GetVar ("Color1") )
 	GUICommon.SetColorStat (MyChar, IE_SKIN_COLOR, GemRB.GetVar ("Color2") )
@@ -215,7 +217,8 @@ def NextPress():
 	GemRB.SetPlayerStat (MyChar, IE_EA, 2 )
 	GemRB.SetPlayerName (MyChar, GemRB.GetToken ("CHARNAME"), 0)
 
-	# feats are set already in the Feats step
+	# feats are set already in the Feats step, but not all their consequences, since users could backpedal
+	IDLUCommon.LearnFeatInnates (MyChar, MyChar, True)
 
 	#does all the rest
 	LargePortrait = GemRB.GetToken ("LargePortrait")
