@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003 The GemRB Project
 #
@@ -36,28 +35,25 @@ def OnLoad ():
 	global LoadWindow, TextAreaControl, Games, ScrollBar
 
 	GemRB.SetToken ("SaveDir", "mpsave") # iwd2 is always using 'mpsave'
-	GemRB.LoadWindowPack ("GUILOAD", 800 ,600)
-	LoadWindow = GemRB.LoadWindow (0)
-	LoadWindow.SetFrame ()
+	LoadWindow = GemRB.LoadWindow (0, "GUILOAD")
 
 	CancelButton=LoadWindow.GetControl (22)
 	CancelButton.SetText (13727)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CancelPress)
-	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	CancelButton.OnPress (LoadWindow.Close)
+	CancelButton.MakeEscape()
 
-	GemRB.SetVar ("LoadIdx",0)
 	for i in range (5):
 		Button = LoadWindow.GetControl (55+i)
 		Button.SetText (15590)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LoadGamePress)
+		Button.OnPress (LoadGamePress)
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
-		Button.SetVarAssoc ("LoadIdx",i)
+		Button.SetValue (i)
 
 		Button = LoadWindow.GetControl (60+i)
 		Button.SetText (13957)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteGamePress)
+		Button.OnPress (DeleteGamePress)
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
-		Button.SetVarAssoc ("LoadIdx",i)
+		Button.SetValue (i)
 
 		#area previews
 		Button = LoadWindow.GetControl (1+i)
@@ -72,115 +68,105 @@ def OnLoad ():
 			Button.SetSize (21, 21)
 
 	ScrollBar=LoadWindow.GetControl (23)
-	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, ScrollBarPress)
+	ScrollBar.OnChange (ScrollBarPress)
 	Games=GemRB.GetSaveGames()
 	TopIndex = max (0, len(Games) - 5)
 
-	ScrollBar.SetVarAssoc ("TopIndex", len(Games))
+	ScrollBar.SetVarAssoc ("TopIndex", TopIndex, 0, TopIndex)
 	ScrollBarPress ()
-	LoadWindow.SetVisible (WINDOW_VISIBLE)
+
+	LoadWindow.SetEventProxy(ScrollBar)
+	LoadWindow.Focus()
 	return
 
 def ScrollBarPress ():
 	#draw load game portraits
-	Pos = GemRB.GetVar ("TopIndex")
+	Pos = GemRB.GetVar ("TopIndex") or 0
 	for i in range (5):
 		ActPos = Pos + i
 
 		Button1 = LoadWindow.GetControl (55+i)
 		Button2 = LoadWindow.GetControl (60+i)
+		ScreenShotButton = LoadWindow.GetControl (1 + i)
 		if ActPos<len(Games):
 			Button1.SetState (IE_GUI_BUTTON_ENABLED)
 			Button2.SetState (IE_GUI_BUTTON_ENABLED)
+			ScreenShotButton.SetPicture (Games[ActPos].GetPreview())
+			Slotname = Games[ActPos].GetName()
+			GameDate = Games[ActPos].GetGameDate()
+			SaveDate = Games[ActPos].GetDate()
 		else:
 			Button1.SetState (IE_GUI_BUTTON_DISABLED)
 			Button2.SetState (IE_GUI_BUTTON_DISABLED)
-
-		if ActPos<len(Games):
-			Slotname = Games[ActPos].GetName()
-		else:
+			ScreenShotButton.SetPicture (None)
 			Slotname = ""
+			GameDate = ""
+			SaveDate = ""
+
 		Label = LoadWindow.GetControl (0x10000005+i)
 		Label.SetText (Slotname)
 
-		if ActPos<len(Games):
-			Slotname = Games[ActPos].GetGameDate()
-		else:
-			Slotname = ""
 		Label = LoadWindow.GetControl (0x1000000a+i)
-		Label.SetText (Slotname)
+		Label.SetText (GameDate)
 
-		if ActPos<len(Games):
-			Slotname = Games[ActPos].GetDate()
-		else:
-			Slotname = ""
 		Label = LoadWindow.GetControl (0x1000000f+i)
-		Label.SetText (Slotname)
+		Label.SetText (SaveDate)
 
-		Button=LoadWindow.GetControl (1+i)
-		if ActPos<len(Games):
-			Button.SetSprite2D(Games[ActPos].GetPreview())
-		else:
-			Button.SetPicture ("")
 		for j in range (min(6, MAX_PARTY_SIZE)):
 			Button=LoadWindow.GetControl (25 + i*min(6, MAX_PARTY_SIZE) + j)
 			if ActPos<len(Games):
-				Button.SetSprite2D(Games[ActPos].GetPortrait(j))
+				Button.SetPicture (Games[ActPos].GetPortrait(j))
 			else:
-				Button.SetPicture ("")
+				Button.SetPicture (None)
 	return
 
-def LoadGamePress ():
+def LoadGamePress (btn):
 	if LoadWindow:
-		LoadWindow.Unload ()
-	Pos = GemRB.GetVar ("TopIndex")+GemRB.GetVar ("LoadIdx")
+		LoadWindow.Close ()
+	Pos = GemRB.GetVar ("TopIndex") + btn.Value
 	LoadScreen.StartLoadScreen()
 	GemRB.LoadGame(Games[Pos]) #loads and enters savegame
 	GemRB.EnterGame ()
 	return
 
-def DeleteGameConfirm():
+def DeleteGameConfirm (btn):
 	global Games
 
 	TopIndex = GemRB.GetVar ("TopIndex")
-	Pos = TopIndex +GemRB.GetVar ("LoadIdx")
+	Pos = TopIndex + btn.Value
 	GemRB.DeleteSaveGame(Games[Pos])
-	if TopIndex>0:
-		GemRB.SetVar ("TopIndex",TopIndex-1)
 	del Games[Pos]
-	ScrollBar.SetVarAssoc ("TopIndex", len(Games))
+	if TopIndex > 0:
+		TopIndex = TopIndex - 1
+	ScrollBar.SetVarAssoc ("TopIndex", TopIndex, 0, max (0, len(Games) - 5))
 	ScrollBarPress ()
 	if ConfirmWindow:
-		ConfirmWindow.Unload ()
-	LoadWindow.SetVisible (WINDOW_VISIBLE)
+		ConfirmWindow.Close ()
+	LoadWindow.Focus()
 	return
 
 def DeleteGameCancel ():
 	if ConfirmWindow:
-		ConfirmWindow.Unload ()
-	LoadWindow.SetVisible (WINDOW_VISIBLE)
+		ConfirmWindow.Close ()
+	LoadWindow.Focus()
 	return
 
-def DeleteGamePress ():
+def DeleteGamePress (btn):
 	global ConfirmWindow
 
-	LoadWindow.SetVisible (WINDOW_INVISIBLE)
-	ConfirmWindow=GemRB.LoadWindow (1)
+	ConfirmWindow=GemRB.LoadWindow (1, "GUILOAD")
+	ConfirmWindow.SetFlags (WF_ALPHA_CHANNEL, OP_OR)
+
 	Text=ConfirmWindow.GetControl (0)
 	Text.SetText (15305)
 	DeleteButton=ConfirmWindow.GetControl (1)
 	DeleteButton.SetText (13957)
-	DeleteButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteGameConfirm)
+	DeleteButton.OnPress (DeleteGameConfirm)
+	DeleteButton.SetValue (btn.Value)
 	CancelButton=ConfirmWindow.GetControl (2)
 	CancelButton.SetText (13727)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DeleteGameCancel)
-	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	CancelButton.OnPress (DeleteGameCancel)
+	CancelButton.MakeEscape()
 
-	ConfirmWindow.SetVisible (WINDOW_VISIBLE)
-	return
-
-def CancelPress ():
-	if LoadWindow:
-		LoadWindow.Unload ()
-	GemRB.SetNextScript ("Start")
+	ConfirmWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
