@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -30,24 +29,20 @@ import GUIRECCommon
 import IDLUCommon
 import LUCommon
 from GUIDefines import *
+from GUICommonWindows import CreateTopWinLoader, ToggleWindow, OpenWindowOnce
 from ie_stats import *
 from ie_restype import *
 from ie_feats import FEAT_ARMORED_ARCANA
 
-SelectWindow = 0
 Topic = None
 HelpTable = None
 DescTable = None
 RecordsWindow = None
 RecordsTextArea = None
 InformationWindow = None
-PortraitWindow = None
-OptionsWindow = None
-OldPortraitWindow = None
-OldOptionsWindow = None
+
 BonusSpellTable = None
 HateRaceTable = None
-PauseState = None
 
 if not BonusSpellTable:
 	BonusSpellTable = GemRB.LoadTable ("mxsplbon")
@@ -64,57 +59,11 @@ def Exportable(pc):
 	if GemRB.GetPlayerStat (pc, IE_STATE_ID)&STATE_DEAD: return False
 	return True
 
-def OpenRecordsWindow ():
-	global RecordsWindow, OptionsWindow, PortraitWindow
-	global OldPortraitWindow, OldOptionsWindow, SelectWindow
-	global BonusSpellTable, HateRaceTable, PauseState
+def InitRecordsWindow (Window):
+	global RecordsWindow
+	global BonusSpellTable, HateRaceTable
 
-	if GUICommon.CloseOtherWindow (OpenRecordsWindow):
-
-		GUIRECCommon.CloseSubSubCustomizeWindow ()
-		GUIRECCommon.CloseSubCustomizeWindow ()
-		GUIRECCommon.CloseCustomizeWindow ()
-		GUIRECCommon.ExportCancelPress()
-		GUIRECCommon.CloseBiographyWindow ()
-		CloseHelpWindow ()
-
-		if RecordsWindow:
-			RecordsWindow.Unload ()
-		if OptionsWindow:
-			OptionsWindow.Unload ()
-		if PortraitWindow:
-			PortraitWindow.Unload ()
-
-		RecordsWindow = None
-		GemRB.SetVar ("OtherWindow", -1)
-		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE)
-		GemRB.UnhideGUI ()
-		GUICommonWindows.PortraitWindow = OldPortraitWindow
-		OldPortraitWindow = None
-		GUICommonWindows.OptionsWindow = OldOptionsWindow
-		OldOptionsWindow = None
-		GUICommonWindows.SetSelectionChangeHandler (None)
-		GemRB.GamePause (PauseState, 3)
-		return
-
-	PauseState = GemRB.GamePause (3, 1)
-	GemRB.GamePause (1, 3)
-
-	GemRB.HideGUI ()
-	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE)
-
-	GemRB.LoadWindowPack ("GUIREC", 800, 600)
-	RecordsWindow = Window = GemRB.LoadWindow (2)
-	GemRB.SetVar ("OtherWindow", RecordsWindow.ID)
-	#saving the original portrait window
-	OldPortraitWindow = GUICommonWindows.PortraitWindow
-	PortraitWindow = GUICommonWindows.OpenPortraitWindow ()
-	OldOptionsWindow = GUICommonWindows.OptionsWindow
-	OptionsWindow = GemRB.LoadWindow (0)
-	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 0, OpenRecordsWindow)
-	Window.SetFrame ()
-
-	Window.SetKeyPressEvent (GUICommonWindows.SwitchPCByKey)
+	RecordsWindow = Window
 
 	#portrait icon
 	Button = Window.GetControl (2)
@@ -124,77 +73,77 @@ def OpenRecordsWindow ():
 	#information (help files)
 	Button = Window.GetControl (1)
 	Button.SetText (11946)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenHelpWindow)
+	Button.OnPress (OpenHelpWindow)
 
 	#biography
 	Button = Window.GetControl (59)
 	Button.SetText (18003)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenBiographyWindow)
+	Button.OnPress (GUIRECCommon.OpenBiographyWindow)
 
 	#export
 	Button = Window.GetControl (36)
 	Button.SetText (13956)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenExportWindow)
+	Button.OnPress (GUIRECCommon.OpenExportWindow)
 
 	#customize
 	Button = Window.GetControl (50)
 	Button.SetText (10645)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenCustomizeWindow)
+	Button.OnPress (GUIRECCommon.OpenCustomizeWindow)
 
 	#general
-	GemRB.SetVar ("SelectWindow", 1)
-
 	Button = Window.GetControl (60)
 	Button.SetTooltip (40316)
 	Button.SetVarAssoc ("SelectWindow", 1)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateRecordsWindow)
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#weapons and armour
 	Button = Window.GetControl (61)
 	Button.SetTooltip (40317)
 	Button.SetVarAssoc ("SelectWindow", 2)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateRecordsWindow)
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#skills and feats
 	Button = Window.GetControl (62)
 	Button.SetTooltip (40318)
 	Button.SetVarAssoc ("SelectWindow", 3)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateRecordsWindow)
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#miscellaneous
 	Button = Window.GetControl (63)
 	Button.SetTooltip (33500)
 	Button.SetVarAssoc ("SelectWindow", 4)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateRecordsWindow)
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
+
+	GemRB.SetVar ("SelectWindow", 1)
 
 	#level up
 	Button = Window.GetControl (37)
 	Button.SetText (7175)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLevelUpWindow)
+	Button.OnPress (OpenLevelUpWindow)
 
-	GUICommonWindows.SetSelectionChangeHandler (UpdateRecordsWindow)
-
-	UpdateRecordsWindow ()
 	return
 
 def ColorDiff (Window, Label, diff):
+	Color = {'r' : 0, 'g' : 0, 'b' : 0}
 	if diff>0:
-		Label.SetTextColor (0, 255, 0)
+		Color['g'] = 255
 	elif diff<0:
-		Label.SetTextColor (255, 0, 0)
+		Color['r'] = 255
 	else:
-		Label.SetTextColor (255, 255, 255)
+		Color = {'r' : 255, 'g' : 255, 'b' : 255}
+
+	Label.SetColor (Color)
 	return
 
 def ColorDiff2 (Window, Label, diff):
 	if diff:
-		Label.SetTextColor (255, 255, 0)
+		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 	else:
-		Label.SetTextColor (255, 255, 255)
+		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
 	return
 
 def GetBonusSpells (pc):
@@ -260,13 +209,13 @@ def DisplayFavouredEnemy (pc, RangerLevel, second, RecordsTextArea):
 		RaceID = GemRB.GetPlayerStat(pc, IE_HATEDRACE2+second)
 	if RaceID:
 		FavouredIndex = HateRaceTable.FindValue (1, RaceID)
-		if FavouredIndex == -1:
+		if FavouredIndex is None:
 			return
 		FavouredName = HateRaceTable.GetValue (FavouredIndex, 0)
 		if second == -1:
-			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel+4)/5), 0))
+			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5), 0))
 		else:
-			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel+4)/5-second-1), 0))
+			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5 - second - 1), 0))
 
 def GetFavoredClass (pc, code):
 	if GemRB.GetPlayerStat (pc, IE_SEX)==1:
@@ -279,6 +228,8 @@ def GetFavoredClass (pc, code):
 # returns the effective character level modifier
 def GetECL (pc):
 	RaceIndex = IDLUCommon.GetRace (pc)
+	if RaceIndex == None:
+		return 0
 	RaceRowName = CommonTables.Races.GetRowName (RaceIndex)
 	return CommonTables.Races.GetValue (RaceRowName, "ECL")
 
@@ -390,8 +341,8 @@ def DisplayGeneral (pc, targetTextArea):
 		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(32052) + "[/color]\n")
 		StateTable = GemRB.LoadTable ("statdesc")
 		for c in effects:
-			tmp = StateTable.GetValue (str(ord(c)-66), "DESCRIPTION", GTV_REF)
-			RecordsTextArea.Append ("[cap]"+c+"%[/cap][p]" + tmp + "[/p]")
+			tmp = StateTable.GetValue (str(c - 66), "DESCRIPTION", GTV_REF)
+			RecordsTextArea.Append ("[cap][int=" + str(c) + "]%[/cap][p]" + tmp + "[/p]")
 
 	# TODO: Active Feats (eg. Power attack 4)
 
@@ -540,9 +491,9 @@ def PlusMinusStat(value):
 def CascadeToHit(total, ac, apr, slot):
 	cascade = PlusMinusStat(total)
 	babDec = 5
-	if ac["Wisdom"]:
-		if slot == 10: # fist slot - nothing is equipped
-			babDec = 3
+	if ac["Wisdom"] and slot == 10: # fist slot - nothing is equipped
+		babDec = 3
+
 	for i in range(1, apr):
 		if total-i*babDec > 0: # skips negative ones, meaning a lower number of attacks can be displayed
 			cascade = cascade  + "/" + PlusMinusStat(total-i*babDec)
@@ -600,7 +551,7 @@ def WeaponOfHand(pc, combatdet, dualwielding, left=0):
 	slot = combatdet["Slot"]
 	slot_item = GemRB.GetSlotItem (pc, slot, 1)
 	if not slot_item:
-		print "ARGHH, no slot item at slot %d, bailing out!" %(combatdet["Slot"])
+		print("ARGHH, no slot item at slot %d, bailing out!" %(combatdet["Slot"]))
 		return
 	item = GemRB.GetItem (slot_item["ItemResRef"])
 
@@ -801,26 +752,26 @@ def DisplayWeapons (pc):
 		failure = GemRB.GetSpellFailure (pc)
 		total = failure["Total"]
 		arcana = 5*GemRB.HasFeat(pc, FEAT_ARMORED_ARCANA)
-		other = 5*(failure["Armor"] + failure["Shield"]) - arcana
+		armor = 5 * (failure["Armor"] + failure["Shield"])
+		other = armor - arcana
 		# account for armored arcana only if it doesn't cause a negative value (eating into the misc bonus)
-		verbose = 0
 		if other < 0:
 			other = total
 		else:
 			other = total - other
-			verbose = 1
+
 		if total < 0:
 			total = 0
 		RecordsTextArea.Append (DelimitedText (41390, str (total)+"%", 0))
 		# Armor Penalty (same as for skills and everything else)
-		if verbose and failure["Armor"]:
+		if failure["Armor"]:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (39816, PlusMinusStat(5*failure["Armor"])+"%", 0))
 		# Shield Penalty
-		if verbose and failure["Shield"]:
+		if failure["Shield"]:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (39822, PlusMinusStat(5*failure["Shield"])+"%", 0))
-		if arcana:
+		if armor and arcana:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (36352, PlusMinusStat(-arcana)+"%", 0))
 		# Other, just a guess to show the remainder
@@ -896,8 +847,6 @@ def DelimitedText(strref, text, newlines=1, delimiter=": "):
 
 #character information
 def DisplayMisc (pc):
-	Window = RecordsWindow
-
 	TotalPartyExp = 0
 	TotalCount = 0
 	for i in range (1, GemRB.GetPartySize() + 1):
@@ -931,25 +880,23 @@ def DisplayMisc (pc):
 
 	# Total Experience Value in Party
 	if TotalPartyExp:
-		val = stat['KillsTotalXP']*100/TotalPartyExp
+		val = stat['KillsTotalXP'] * 100 // TotalPartyExp
 	else:
 		val = 0
 	RecordsTextArea.Append (DelimitedText (11951, str(val) + "%", 0))
 
 	# Percentage of Total Kills in Party
 	if TotalPartyExp:
-		val = stat['KillsTotalCount']*100/TotalCount
+		val = stat['KillsTotalCount'] * 100 // TotalCount
 	else:
 		val = 0
 	RecordsTextArea.Append (DelimitedText (11952, str(val) + "%", 0))
 
 	return
 
-def UpdateRecordsWindow ():
+def UpdateRecordsWindow (Window):
 	global RecordsTextArea
 
-	Window = RecordsWindow
-
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	#name
@@ -958,7 +905,7 @@ def UpdateRecordsWindow ():
 
 	#portrait
 	Button = Window.GetControl (2)
-	Button.SetPicture (GemRB.GetPlayerPortrait (pc,0))
+	Button.SetPicture (GemRB.GetPlayerPortrait (pc, 0)["Sprite"])
 
 	# armorclass
 	Label = Window.GetControl (0x10000028)
@@ -1054,6 +1001,7 @@ def UpdateRecordsWindow ():
 
 	RecordsTextArea = Window.GetControl (45)
 	RecordsTextArea.SetText ("")
+	RecordsTextArea.SetColor (ColorWhitish, TA_COLOR_INITIALS)
 
 	DisplayCommon (pc)
 
@@ -1069,15 +1017,17 @@ def UpdateRecordsWindow ():
 
 	#if actor is uncontrollable, make this grayed
 	GUICommon.AdjustWindowVisibility (Window, pc, 0)
-	PortraitWindow.SetVisible (WINDOW_VISIBLE)
-	OptionsWindow.SetVisible (WINDOW_VISIBLE)
+
 	return
 
+ToggleRecordsWindow = CreateTopWinLoader(2, "GUIREC", ToggleWindow, InitRecordsWindow, UpdateRecordsWindow, True)
+OpenRecordsWindow = CreateTopWinLoader(2, "GUIREC", OpenWindowOnce, InitRecordsWindow, UpdateRecordsWindow, True)
+
 def CloseHelpWindow ():
 	global DescTable, InformationWindow
 
 	if InformationWindow:
-		InformationWindow.Unload ()
+		InformationWindow.Close ()
 		InformationWindow = None
 	if DescTable:
 		DescTable = None
@@ -1087,7 +1037,7 @@ def CloseHelpWindow ():
 def OpenHelpWindow ():
 	global HelpTable, InformationWindow
 
-	InformationWindow = Window = GemRB.LoadWindow (57)
+	InformationWindow = Window = GemRB.LoadWindow (57, "GUIREC")
 
 	HelpTable = GemRB.LoadTable ("topics")
 	GemRB.SetVar("Topic", 0)
@@ -1102,7 +1052,7 @@ def OpenHelpWindow ():
 		if title:
 			Label.SetText (title)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateHelpWindow)
+			Button.OnPress (UpdateHelpWindow)
 		else:
 			Label.SetText ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -1110,7 +1060,7 @@ def OpenHelpWindow ():
 	#done
 	Button = Window.GetControl (1)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseHelpWindow)
+	Button.OnPress (CloseHelpWindow)
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	UpdateHelpWindow ()
 	return
@@ -1129,28 +1079,28 @@ def UpdateHelpWindow ():
 		#Button = Window.GetControl (i+27)
 		Label = Window.GetControl (i+0x10000004)
 		if Topic==i:
-			Label.SetTextColor (255,255,0)
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 		else:
-			Label.SetTextColor (255,255,255)
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
 
 	resource = HelpTable.GetValue (Topic, 1)
 	if DescTable:
 		DescTable = None
 
-	DescTable = GemRB.LoadTable (resource)
-
-	ScrollBar = Window.GetControl (4)
+	if resource:
+		DescTable = GemRB.LoadTable (resource)
 
 	startrow = HelpTable.GetValue (Topic, 4)
 	if startrow<0:
-		i=-startrow-10
+		i = -startrow - 10
 	else:
-		i = DescTable.GetRowCount ()-10-startrow
-
-	if i<1: i=1
+		if DescTable:
+			i = DescTable.GetRowCount () - 10 - startrow
+		if i < 0: i = 0
 
+	ScrollBar = Window.GetControl (4)
 	ScrollBar.SetVarAssoc ("TopIndex", i)
-	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, RefreshHelpWindow)
+	ScrollBar.OnChange (RefreshHelpWindow)
 
 	RefreshHelpWindow ()
 	return
@@ -1167,25 +1117,26 @@ def RefreshHelpWindow ():
 	if startrow<0: startrow = 0
 
 	for i in range(11):
-		title = DescTable.GetValue (i+startrow+TopIndex, titlecol)
-
 		Button = Window.GetControl (i+71)
 		Label = Window.GetControl (i+0x10000030)
 
 		if i+TopIndex==Selected:
-			Label.SetTextColor (255,255,0)
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 		else:
-			Label.SetTextColor (255,255,255)
-		if title>0:
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
+		title = ""
+		if DescTable:
+			title = DescTable.GetValue (i+startrow+TopIndex, titlecol)
+		if title != "*":
 			Label.SetText (title)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
 			Button.SetVarAssoc ("Selected", i+TopIndex)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RefreshHelpWindow)
+			Button.OnPress (RefreshHelpWindow)
 		else:
 			Label.SetText ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 
-	if Selected<0:
+	if Selected<0 or DescTable == None:
 		desc=""
 	else:
 		desc = DescTable.GetValue (Selected+startrow, desccol)
@@ -1201,17 +1152,17 @@ def CloseLUWindow ():
 	global LUWindow, LUKitWindow
 
 	if LUKitWindow:
-		LUKitWindow.Unload ()
+		LUKitWindow.Close ()
 		LUKitWindow = None
 
 	if LUWindow:
-		LUWindow.Unload ()
+		LUWindow.Close ()
 		LUWindow = None
 
 def OpenLevelUpWindow ():
 	global LUWindow, LevelDiff
 
-	LUWindow = Window = GemRB.LoadWindow (54)
+	LUWindow = Window = GemRB.LoadWindow (54, "GUIREC")
 
 	# Figure out the level difference
 	# the original ignored all but the fighter row in xplevel.2da
@@ -1222,14 +1173,13 @@ def OpenLevelUpWindow ():
 	nextLevel = LUCommon.GetNextLevelFromExp (xp, 5)
 	levelSum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
 	LevelDiff = nextLevel - levelSum - GetECL(pc)
-	print 1111111, nextLevel, levelSum, GetECL(pc), LevelDiff
 
 	# next
 	Button = Window.GetControl (0)
 	Button.SetText (36789)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLUKitWindow)
+	Button.OnPress (OpenLUKitWindow)
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
-	Button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	Button.MakeDefault ()
 
 	# static Class selection
 	#Label = Window.GetControl (0x10000000)
@@ -1237,7 +1187,7 @@ def OpenLevelUpWindow ():
 
 	# 2-12 are the class name buttons
 	# 15-25 are the class level labels
-	GemRB.SetVar ("LUClass", -1)
+	GemRB.SetVar ("LUClass", None)
 	for i in range(2,13):
 		Button = Window.GetControl (i)
 		Label = Window.GetControl (0x10000000 + i+13)
@@ -1258,7 +1208,7 @@ def OpenLevelUpWindow ():
 		else:
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc ("LUClass", i-2)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUClassPress)
+		Button.OnPress (LUClassPress)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 
 	# description
@@ -1272,8 +1222,7 @@ def OpenLevelUpWindow ():
 	# cancel
 	Button = Window.GetControl (27)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseLUWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
+	Button.OnPress (CloseLUWindow)
 
 	Window.ShowModal (MODAL_SHADOW_NONE)
 
@@ -1287,7 +1236,7 @@ def LUClassPress ():
 
 	# display class info
 	TextArea = Window.GetControl (13)
-	i = GemRB.GetVar ("LUClass")
+	i = GemRB.GetVar ("LUClass") or 0
 	ClassDesc = CommonTables.Classes.GetRowName (i)
 	ClassDesc = CommonTables.Classes.GetValue (ClassDesc, "DESC_REF", GTV_REF)
 	TextArea.SetText (ClassDesc)
@@ -1338,10 +1287,10 @@ def HandleSpecFlagExclusion(pc, lucls, lukit):
 			if lukit&kit:
 				return
 			else:
-				raise RuntimeError, "Picked second or non-existing kit (%d) of same class (%d)!?" %(lukit, lucls)
+				raise RuntimeError("Picked second or non-existing kit (%d) of same class (%d)!?" %(lukit, lucls))
 	else: # picked a new class
 		kitBits = [((kit >> x) & 1)<<x  for x in range(31, -1, -1)]
-		kitBits = filter(lambda v: v > 0, kitBits)
+		kitBits = [v for v in kitBits if v > 0]
 		# check all the current kits - kit bits
 		for k in kitBits:
 			if k not in orderFavs:
@@ -1375,16 +1324,16 @@ def OpenLUKitWindow ():
 	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
 	hasKits = CommonTables.Classes.FindValue ("CLASS", LUClassID)
 	kitIndex = GUICommonWindows.GetKitIndex (pc, LUClass)
-	if hasKits == -1 or kitIndex > 0:
+	if hasKits is None or kitIndex > 0:
 		kitName = CommonTables.Classes.GetRowName (kitIndex)
 		kitID = CommonTables.Classes.GetValue (kitName, "ID", GTV_INT)
-		if hasKits == -1:
+		if hasKits is None:
 			kitID = 0
 		HandleSpecFlagExclusion(pc, LUClassID, kitID)
 		LUNextPress ()
 		return
 
-	LUKitWindow = Window = GemRB.LoadWindow (6)
+	LUKitWindow = Window = GemRB.LoadWindow (6, "GUIREC")
 
 	# title
 	Label = Window.GetControl (0x10000000)
@@ -1393,9 +1342,9 @@ def OpenLUKitWindow ():
 	# next
 	Button = Window.GetControl (0)
 	Button.SetText (36789)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUNextPress)
+	Button.OnPress (LUNextPress)
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
-	Button.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
+	Button.MakeDefault ()
 
 	# 1 does not exist, 2-10 are kit buttons
 	# 11 scrollbar, 12 back
@@ -1408,7 +1357,7 @@ def OpenLUKitWindow ():
     # skip to the first kit of this class
 	kitOffset = hasKits
 
-	GemRB.SetVar ("LUKit", -1)
+	GemRB.SetVar ("LUKit", None)
 	for i in range(9):
 		Button = Window.GetControl (i+2)
 
@@ -1423,7 +1372,7 @@ def OpenLUKitWindow ():
 		Button.SetText (kitTitle)
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc ("LUKit", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUKitPress)
+		Button.OnPress (LUKitPress)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 
 	Window.ShowModal (MODAL_SHADOW_NONE)
@@ -1435,8 +1384,8 @@ def LUKitPress ():
 
 	# display kit info
 	TextArea = Window.GetControl (13)
-	i = GemRB.GetVar ("LUKit")
-	LUClass = GemRB.GetVar ("LUClass")
+	i = GemRB.GetVar ("LUKit") or 0
+	LUClass = GemRB.GetVar ("LUClass") or 0
 	LUClassName = CommonTables.Classes.GetRowName (LUClass)
 	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
 	kitOffset = CommonTables.Classes.FindValue ("CLASS", LUClassID)
@@ -1447,11 +1396,7 @@ def LUKitPress ():
 	# set it to the kit value, so we don't need these gimnastics later
 	kitID = CommonTables.Classes.GetValue (kitName, "ID", GTV_INT)
 	GemRB.SetVar ("LUKit", kitID)
-	pc = GemRB.GameGetSelectedPCSingle ()
-	HandleSpecFlagExclusion(pc, LUClassID, kitID)
-
-	oldKits = GemRB.GetPlayerStat (pc, IE_KIT, 1)
-	GemRB.SetPlayerStat (pc, IE_KIT, oldKits|kitID)
+	GemRB.SetVar ("LUKitJustSet", LUClassID)
 
 # continue with level up via chargen methods
 def LUNextPress ():
@@ -1462,6 +1407,15 @@ def LUNextPress ():
 
 	# grant an ability point or three (each 4 levels)
 	pc = GemRB.GameGetSelectedPCSingle ()
+
+	if (GemRB.GetVar ("LUKitJustSet") or 0) != 0:
+		oldKits = GemRB.GetPlayerStat (pc, IE_KIT, 1)
+		kitID = GemRB.GetVar ("LUKit")
+		GemRB.SetPlayerStat (pc, IE_KIT, oldKits | kitID)
+
+		HandleSpecFlagExclusion(pc, GemRB.GetVar ("LUKitJustSet"), kitID)
+		GemRB.SetVar ("LUKitJustSet", 0)
+
 	levelSum = GemRB.GetPlayerStat (pc, IE_CLASSLEVELSUM)
 	rankDiff = (levelSum+LevelDiff)//4 - levelSum//4
 	if rankDiff > 0:
@@ -1475,17 +1429,17 @@ def LUNextPress ():
 def FinishLevelUp():
 	# kit
 	pc = GemRB.GameGetSelectedPCSingle ()
-	LUKit = GemRB.GetVar ("LUKit")
+	LUKit = GemRB.GetVar ("LUKit") or 0
 
 	# saving throws
-	LUClass = GemRB.GetVar ("LUClass") # index, not ID
+	LUClass = GemRB.GetVar ("LUClass") or 0 # index, not ID
 	LUClassName = CommonTables.Classes.GetRowName (LUClass)
 	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
 	IDLUCommon.SetupSavingThrows (pc, LUClassID)
 
 	# hit points
 	Levels = [ GemRB.GetPlayerStat (pc, l) for l in IDLUCommon.Levels ]
-	LevelDiff = GemRB.GetVar ("LevelDiff")
+	LevelDiff = GemRB.GetVar ("LevelDiff") or 0
 	LevelDiffs = [ 0 ] * len(Levels)
 	LevelDiffs[LUClass] = LevelDiff
 	# SetupHP expects the target level already
@@ -1507,9 +1461,9 @@ def FinishLevelUp():
 	BABTable = CommonTables.Classes.GetValue (LUClassName, "TOHIT")
 	BABTable = GemRB.LoadTable (BABTable)
 	currentBAB = GemRB.GetPlayerStat (pc, IE_TOHIT, 1)
-	oldBAB = BABTable.GetValue (str(oldLevel), "BASE_ATTACK")
+	oldBAB = BABTable.GetValue (str(oldLevel), "BASE_ATTACK", GTV_INT)
 	newLevel = oldLevel + LevelDiff
-	newBAB = BABTable.GetValue (str(newLevel), "BASE_ATTACK")
+	newBAB = BABTable.GetValue (str(newLevel), "BASE_ATTACK", GTV_INT)
 	GemRB.SetPlayerStat (pc, IE_TOHIT, currentBAB + newBAB - oldBAB)
 
 	# class level
@@ -1517,11 +1471,11 @@ def FinishLevelUp():
 
 	# now we're finally done
 	GemRB.SetVar ("LevelDiff", 0)
-	GemRB.SetVar ("LUClass", -1)
+	GemRB.SetVar ("LUClass", None)
 	GemRB.SetVar ("LUKit", 0)
-	# DisplayGeneral (pc) is not enough for a refresh refresh
-	OpenRecordsWindow ()
-	OpenRecordsWindow ()
+
+	UpdateRecordsWindow (RecordsWindow)
+
 
 ###################################################
 # End of file GUIREC.py
