@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003 The GemRB Project
 #
@@ -22,6 +21,7 @@
 ###################################################
 
 import GemRB
+import ActionsWindow
 import GUICommon
 import CommonTables
 import Spellbook
@@ -45,7 +45,7 @@ CID_WEAPONS = 2
 CID_SPELLS = 3
 CID_ITEMS = 4
 CID_ABILITIES = 5
-# FIXME: there are buttons 6-10 we hide completely. What are they?
+# buttons 6-10 are completely redundant
 CID_HANDLE1 = 11
 CID_HANDLE2 = 12
 CID_PREV = 13
@@ -66,12 +66,12 @@ MENU_MODE_DIALOG = 6
 float_menu_mode = MENU_MODE_SINGLE
 float_menu_index = 0
 float_menu_selected = None
-type = None
+spelltype = None
 
 def UseSpell ():
 	pc = GemRB.GameGetFirstSelectedPC ()
 	slot = float_menu_selected+float_menu_index
-	print "spell", type, slot
+	print("spell", spelltype, slot)
 	GemRB.SpellCast (pc, 1<<type, slot)
 	return
 
@@ -85,9 +85,8 @@ def UseWeapon ():
 	GemRB.SetEquippedQuickSlot (pc, float_menu_selected)
 	return
 
-def DoSingleAction ():
-	i = GemRB.GetVar ('ItemButton')
-	print i
+def DoSingleAction (btn):
+	i = btn.Value
 	OpenFloatMenuWindow ()
 	if i == 0:
 		GUIMA.OpenMapWindow ()
@@ -101,7 +100,7 @@ def DoSingleAction ():
 		pc = GemRB.GameGetFirstSelectedPC ()
 		ClassName = GUICommon.GetClassRowName (pc)
 		if CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL") == "*":
-			GUIMG.OpenMageWindow ()
+			GUIMG.OpenSpellWindow ()
 		else:
 			GUIPR.OpenPriestWindow ()
 
@@ -109,17 +108,10 @@ def OpenFloatMenuWindow (x=0, y=0):
 	global FloatMenuWindow
 	global float_menu_mode, float_menu_index, float_menu_selected
 
-	GemRB.HideGUI ()
-
-	if FloatMenuWindow:
-		if FloatMenuWindow:
-			FloatMenuWindow.Unload ()
-		FloatMenuWindow = None
-
+	handler = GUICommonWindows.SelectionChangeHandler
+	def OnClose():
 		GemRB.GamePause (False, 0)
-		GemRB.SetVar ("FloatWindow", -1)
-		GUICommonWindows.SetSelectionChangeMultiHandler (None)
-		GemRB.UnhideGUI ()
+		GUICommonWindows.SetSelectionChangeHandler (handler)
 
 		if float_menu_selected==None:
 			return
@@ -134,81 +126,76 @@ def OpenFloatMenuWindow (x=0, y=0):
 			UseSpell()
 		return
 
-	ActionLevel = 0
 	if not GemRB.GameGetFirstSelectedPC ():
-		GemRB.UnhideGUI ()
 		return
 
 	GemRB.GamePause (True, 0)
 	GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
 
-	GemRB.LoadWindowPack ("GUIWORLD")
-	FloatMenuWindow = Window = GemRB.LoadWindow (3)
-	GemRB.SetVar ("FloatWindow", Window.ID)
+	FloatMenuWindow = Window = GemRB.LoadWindow (3, "GUIWORLD")
+	Window.OnUnFocus(lambda: GemRB.SetTimer(Window.Close, 0, 0))
 
-	Window.SetPos (x, y, WINDOW_CENTER | WINDOW_BOUNDED)
+	Window.SetFlags (WF_DRAGGABLE|WF_ALPHA_CHANNEL, OP_OR)
+	Window.SetPos (x, y)
+	Window.SetTooltip (8199)
+	Window.SetAction (OnClose, ACTION_WINDOW_CLOSED)
 
 	# portrait button
 	Button = Window.GetControl (CID_PORTRAIT)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectNextPC)
-	Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenFloatMenuWindow)
+	Button.OnPress (FloatMenuSelectNextPC)
+	Button.OnRightPress (Window.Close)
 
 	# Initiate Dialogue
 	Button = Window.GetControl (CID_DIALOG)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectDialog)
+	Button.OnPress (FloatMenuSelectDialog)
 	Button.SetTooltip (8191)
 
 	# Attack/Select Weapon
 	Button = Window.GetControl (CID_WEAPONS)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectWeapons)
+	Button.OnPress (FloatMenuSelectWeapons)
 	Button.SetTooltip (8192)
 
 	# Cast spell
 	Button = Window.GetControl (CID_SPELLS)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectSpells)
+	Button.OnPress (FloatMenuSelectSpells)
 	Button.SetTooltip (8193)
 
 	# Use Item
 	Button = Window.GetControl (CID_ITEMS)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectItems)
+	Button.OnPress (FloatMenuSelectItems)
 	Button.SetTooltip (8194)
 
 	# Use Special Ability
 	Button = Window.GetControl (CID_ABILITIES)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuSelectAbilities)
+	Button.OnPress (FloatMenuSelectAbilities)
 	Button.SetTooltip (8195)
 
 	# Menu Anchors/Handles
-	Button = Window.GetControl (CID_HANDLE1)
-	Button.SetTooltip (8199)
-	Button.SetFlags (IE_GUI_BUTTON_DRAGGABLE, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, FloatMenuDrag)
-
-	Button = Window.GetControl (CID_HANDLE2)
-	Button.SetTooltip (8199)
-	Button.SetFlags (IE_GUI_BUTTON_DRAGGABLE, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, FloatMenuDrag)
-
+	Handle = Window.GetControl (CID_HANDLE1)
+	Handle.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
+	Handle = Window.GetControl (CID_HANDLE2)
+	Handle.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
+	
 	# Rotate Items left (to begin)
 	Button = Window.GetControl (CID_PREV)
 	Button.SetTooltip (8197)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuPreviousItem)
+	Button.OnPress (FloatMenuPreviousItem)
 
 	# Rotate Items right (to end)
 	Button = Window.GetControl (CID_NEXT)
 	Button.SetTooltip (8198)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, FloatMenuNextItem)
+	Button.OnPress (FloatMenuNextItem)
 
 	# 6 - 10 - items/spells/other
-	for i in range (6, 11):
-		Button = Window.GetControl (i)
-		Button.SetPos (65535, 65535)
+	for i in range (CID_ABILITIES + 1, CID_HANDLE1):
+		if Window.GetControl (i):
+			Window.DeleteControl (i)
 
 	# 15 - 19 - empty item slot
 	for i in range (SLOT_COUNT):
 		Button = Window.GetControl (CID_SLOTS + i)
 		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
-		Button.SetVarAssoc ('ItemButton', i)
+		Button.SetVarAssoc ("ItemButton", i)
 		Button.SetFont ('NUMBER')
 		Button.SetPushOffset (0, 0)
 
@@ -249,10 +236,14 @@ def OpenFloatMenuWindow (x=0, y=0):
 
 	float_menu_index = 0
 
-	GUICommonWindows.SetSelectionChangeMultiHandler (FloatMenuSelectAnotherPC)
+	def SelChange():
+		FloatMenuSelectAnotherPC()
+		if handler:
+			handler()
+
+	GUICommonWindows.SetSelectionChangeHandler (SelChange)
 	UpdateFloatMenuWindow ()
 
-	GemRB.UnhideGUI ()
 	return
 
 def UpdateFloatMenuWindow ():
@@ -270,24 +261,17 @@ def UpdateFloatMenuWindow ():
 	Button = Window.GetControl (CID_NEXT)
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
 
+	updaterFunc = None
 	if float_menu_mode == MENU_MODE_SINGLE:
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuSingleAction (i)
-
+		updaterFunc = UpdateFloatMenuSingleAction
 	elif float_menu_mode == MENU_MODE_GROUP:
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuGroupAction (i)
-
+		updaterFunc = UpdateFloatMenuGroupAction
 	elif float_menu_mode == MENU_MODE_WEAPONS:
 		# weapons
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuItem (pc, i, 1)
-
+		updaterFunc = lambda i: UpdateFloatMenuItem (pc, i, 1)
 	elif float_menu_mode == MENU_MODE_ITEMS:
 		# items
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuItem (pc, i, 0)
-
+		updaterFunc = lambda i: UpdateFloatMenuItem (pc, i, 0)
 	elif float_menu_mode == MENU_MODE_SPELLS:
 		# spells
 		RefreshSpellList(pc, False)
@@ -295,9 +279,7 @@ def UpdateFloatMenuWindow ():
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button = Window.GetControl (CID_NEXT)
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuSpell (pc, i)
-
+		updaterFunc = lambda i: UpdateFloatMenuSpell (pc, i)
 	elif float_menu_mode == MENU_MODE_ABILITIES:
 		# abilities
 		RefreshSpellList(pc, True)
@@ -307,12 +289,14 @@ def UpdateFloatMenuWindow ():
 		if float_menu_index+3<len(spell_list):
 			Button = Window.GetControl (CID_NEXT)
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
-		for i in range (SLOT_COUNT):
-			UpdateFloatMenuSpell (pc, i)
-
+		updaterFunc = lambda i: UpdateFloatMenuSpell (pc, i)
 	elif float_menu_mode == MENU_MODE_DIALOG:
-		for i in range (SLOT_COUNT):
-			ClearSlot (i)
+		updaterFunc = ClearSlot
+
+	if not updaterFunc:
+		return
+	for i in range (SLOT_COUNT):
+		updaterFunc (i)
 
 	return
 
@@ -332,20 +316,15 @@ def UpdateFloatMenuSingleAction (i):
 	Button = Window.GetControl (CID_SLOTS + i)
 
 	Button.SetSprites (butts[i][0], 0, 0, 1, 2, 3)
-	Button.SetPicture('')
+	Button.SetPicture(None)
 	Button.SetTooltip (butts[i][1])
 	Button.SetText ('')
 	Button.SetState (IE_GUI_BUTTON_ENABLED)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoSingleAction)
+	Button.OnPress (DoSingleAction)
 
 
 def UpdateFloatMenuGroupAction (i):
 	Window = FloatMenuWindow
-	#butts = [ ('AMGUARD', 31657, ''),
-	#	  ('AMTLK', 41653, ''),
-	#	  ('AMATTCK', 41654, ''),
-	#	  ('AMALLSTP', 41655, ''),
-	#	  ('AMGENS', '', '') ]
 
 	butts = [ ACT_DEFEND, ACT_TALK, ACT_ATTACK, ACT_STOP, ACT_SEARCH ]
 	# Guard
@@ -354,9 +333,6 @@ def UpdateFloatMenuGroupAction (i):
 	# Abort Current Action
 	Button = Window.GetControl (CID_SLOTS + i)
 
-	#Button.SetSprites (butts[i][0], 0, 0, 1, 2, 3)
-	#Button.SetTooltip (butts[i][1])
-	#Button.SetText ('')
 	Button.SetActionIcon (globals(), butts[i], i+1 )
 	Button.SetState (IE_GUI_BUTTON_ENABLED)
 
@@ -364,21 +340,21 @@ def RefreshSpellList(pc, innate):
 	global spell_list, type
 
 	if innate:
-		type = IE_SPELL_TYPE_INNATE
+		spelltype = IE_SPELL_TYPE_INNATE
 	else:
 		ClassName = GUICommon.GetClassRowName (pc)
 		if CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL") == "*":
-			type = IE_SPELL_TYPE_WIZARD
+			spelltype = IE_SPELL_TYPE_WIZARD
 		else:
-			type = IE_SPELL_TYPE_PRIEST
+			spelltype = IE_SPELL_TYPE_PRIEST
 
-	type = 1<<type
+	spelltype = 1 << spelltype
 	spell_list = []
 	for i in range(16):
-		if type & (1<<i):
+		if spelltype & (1<<i):
 			spell_list += Spellbook.GetUsableMemorizedSpells (pc, i)
 
-	GemRB.SetVar ("Type", type)
+	GemRB.SetVar ("Type", spelltype)
 	GemRB.SetVar ("QSpell", -1)
 	return
 
@@ -421,25 +397,25 @@ def UpdateFloatMenuItem (pc, i, weapons):
 		else:
 			Button.SetTooltip (item['ItemNameIdentified'])
 		#Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectItem)
+		Button.OnPress (SelectItem)
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
-		#Button.SetVarAssoc ('ItemButton', i)
 		return
 
 	ClearSlot (i)
 
 
-def SelectItem ():
+def SelectItem (btn):
 	global float_menu_selected
 
-	Window = FloatMenuWindow
-	Button = GemRB.GetVar ('ItemButton')
 	#simulating radiobutton+checkbox hybrid
-	if float_menu_selected == Button:
+	if float_menu_selected == btn.Value:
 		float_menu_selected = None
 	else:
-		float_menu_selected = Button
-	UpdateFloatMenuWindow()
+		float_menu_selected = btn.Value
+	if GUICommon.UsingTouchInput ():
+		FloatMenuWindow.Close ()
+	else:
+		UpdateFloatMenuWindow()
 	return
 
 def UpdateFloatMenuSpell (pc, i):
@@ -450,9 +426,14 @@ def UpdateFloatMenuSpell (pc, i):
 	if i == float_menu_selected:
 		Button.SetBAM ('AMHILITE', 0, 0)
 	else:
-		Button.SetPicture('')
+		Button.SetPicture(None)
 		#Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
 
+	def CloseOnPress (callback):
+		if GUICommon.UsingTouchInput ():
+			FloatMenuWindow.Close ()
+		callback ()
+
 	if i + float_menu_index < len (spell_list):
 		SpellResRef = spell_list[i + float_menu_index]["SpellResRef"]
 		Button.SetSpellIcon (SpellResRef)
@@ -460,15 +441,15 @@ def UpdateFloatMenuSpell (pc, i):
 
 		spell = GemRB.GetSpell (SpellResRef)
 		Button.SetTooltip (spell['SpellName'])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommonWindows.SpellPressed)
+		Button.OnPress (lambda: CloseOnPress (ActionsWindow.SpellPressed))
 		Button.SetVarAssoc ("Spell", spell_list[i + float_menu_index]['SpellIndex'])
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	elif i < 3 and GemRB.GetPlayerStat (pc, IE_CLASS) == 9:
 		# handle thieving
 		thieving = [ ACT_SEARCH, ACT_THIEVING, ACT_STEALTH ]
-		acts = [ GUICommonWindows.ActionSearchPressed, GUICommonWindows.ActionThievingPressed, GUICommonWindows.ActionStealthPressed ]
+		acts = [ ActionsWindow.ActionSearchPressed, ActionsWindow.ActionThievingPressed, ActionsWindow.ActionStealthPressed ]
 		Button.SetActionIcon (globals(), thieving[i])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, acts[i])
+		Button.OnPress (lambda: CloseOnPress (acts[i]))
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		ClearSlot (i)
@@ -476,9 +457,8 @@ def UpdateFloatMenuSpell (pc, i):
 def ClearSlot (i):
 	Button = FloatMenuWindow.GetControl (CID_SLOTS + i)
 	Button.SetSprites ('AMGENS', 0, 0, 1, 0, 0)
-	Button.SetPicture ('')
+	Button.SetPicture (None)
 	Button.SetText ('')
-	Button.SetTooltip ('')
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
 
 def FloatMenuSelectPreviousPC ():
@@ -515,14 +495,14 @@ def FloatMenuSelectAnotherPC ():
 	return
 
 def FloatMenuSelectDialog ():
-	global float_menu_mode, float_menu_selected
+	global float_menu_mode, float_menu_index, float_menu_selected
 	float_menu_mode = MENU_MODE_DIALOG
 	float_menu_index = 0
 	float_menu_selected = None
 	GemRB.GameControlSetTargetMode (TARGET_MODE_TALK)
 	# close it immediately for touch users, since it's hard for them rightclick to close
 	if GUICommon.UsingTouchInput ():
-		OpenFloatMenuWindow ()
+		FloatMenuWindow.Close ()
 	else:
 		UpdateFloatMenuWindow ()
 	return
@@ -533,10 +513,7 @@ def FloatMenuSelectWeapons ():
 	float_menu_index = 0
 	float_menu_selected = None
 	GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK)
-	if GUICommon.UsingTouchInput ():
-		OpenFloatMenuWindow ()
-	else:
-		UpdateFloatMenuWindow ()
+	UpdateFloatMenuWindow ()
 	return
 
 def FloatMenuSelectItems ():
@@ -544,10 +521,7 @@ def FloatMenuSelectItems ():
 	float_menu_mode = MENU_MODE_ITEMS
 	float_menu_index = 0
 	float_menu_selected = None
-	if GUICommon.UsingTouchInput ():
-		OpenFloatMenuWindow ()
-	else:
-		UpdateFloatMenuWindow ()
+	UpdateFloatMenuWindow ()
 	return
 
 def FloatMenuSelectSpells ():
@@ -562,10 +536,7 @@ def FloatMenuSelectSpells ():
 	float_menu_mode = MENU_MODE_SPELLS
 	float_menu_index = 0
 	float_menu_selected = None
-	if GUICommon.UsingTouchInput ():
-		OpenFloatMenuWindow ()
-	else:
-		UpdateFloatMenuWindow ()
+	UpdateFloatMenuWindow ()
 	return
 
 def FloatMenuSelectAbilities ():
@@ -573,10 +544,7 @@ def FloatMenuSelectAbilities ():
 	float_menu_mode = MENU_MODE_ABILITIES
 	float_menu_index = 0
 	float_menu_selected = None
-	if GUICommon.UsingTouchInput ():
-		OpenFloatMenuWindow ()
-	else:
-		UpdateFloatMenuWindow ()
+	UpdateFloatMenuWindow ()
 	return
 
 def FloatMenuPreviousItem ():
@@ -603,9 +571,4 @@ def FloatMenuNextItem ():
 	UpdateFloatMenuWindow ()
 	return
 
-def FloatMenuDrag ():
-	dx, dy = GemRB.GetVar ("DragX"), GemRB.GetVar ("DragY")
-	FloatMenuWindow.SetPos (dx, dy, WINDOW_RELATIVE | WINDOW_BOUNDED)
-	return
-
 #############################
