@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003 The GemRB Project
 #
@@ -32,12 +31,6 @@ import GUICommon
 import GUICommonWindows
 from GUIDefines import *
 
-###################################################
-JournalWindow = None
-LogWindow = None
-BeastsWindow = None
-QuestsWindow = None
-
 # list of all assigned (0) or completed (1) quests
 global quests
 quests = [ [], [] ]
@@ -58,125 +51,99 @@ global BeastImage
 BeastImage = None
 StartTime = 0
 
-PauseState = None
-
 ###################################################
-def OpenJournalWindow ():
-	global JournalWindow, PortraitWindow, ActionsWindow
-	global StartTime, PauseState
+def InitJournalWindow (JournalWindow):
+	global StartTime
 
 	Table = GemRB.LoadTable("YEARS")
 	StartTime = Table.GetValue("STARTTIME", "VALUE")
-	
-	if GUICommon.CloseOtherWindow (OpenJournalWindow):
-		if LogWindow: OpenLogWindow()
-		if BeastsWindow: OpenBeastsWindow()
-		if QuestsWindow: OpenQuestsWindow()
-		
-		GemRB.HideGUI ()
-		
-		if JournalWindow:
-			JournalWindow.Unload()
-		#making the portraitwindow visible again
-		GUICommonWindows.EnableAnimatedWindows ()
-		GemRB.SetVar ("OtherWindow", -1)
-		PortraitWindow = None
-		ActionsWindow = None
-		JournalWindow = None
-
-		GUICommon.GameWindow.SetVisible (WINDOW_VISIBLE)
-		GemRB.UnhideGUI ()
-		GemRB.GamePause (PauseState, 3)
-		return	
-
-	PauseState = GemRB.GamePause (3, 1)
-	GemRB.GamePause (1, 3)
-		
-	GemRB.HideGUI ()
-	GemRB.LoadWindowPack ("GUIJRNL")
-	JournalWindow = GemRB.LoadWindow (0)
-	GemRB.SetVar ("OtherWindow", JournalWindow.ID)
-	GUICommonWindows.DisableAnimatedWindows ()
 
 	# Quests
 	Button = JournalWindow.GetControl (0)
 	Button.SetText (20430)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenQuestsWindow)
+	Button.OnPress (OpenQuestsWindow)
 
 	# Beasts
 	Button = JournalWindow.GetControl (1)
 	Button.SetText (20634)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBeastsWindow)
+	Button.OnPress (OpenBeastsWindow)
 
 	# Journal
 	Button = JournalWindow.GetControl (2)
 	Button.SetText (20635)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLogWindow)
+	Button.OnPress (OpenLogWindow)
 
 	# Done
 	Button = JournalWindow.GetControl (3)
 	Button.SetText (20636)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenJournalWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-	Button.SetStatus (IE_GUI_CONTROL_FOCUSED)
+	Button.OnPress (JournalWindow.Close)
+	Button.MakeEscape()
 
-	#JournalWindow.SetVisible (WINDOW_VISIBLE)
-	GemRB.UnhideGUI()
-	
+	return
 
+ToggleJournalWindow = GUICommonWindows.CreateTopWinLoader(0, "GUIJRNL", GUICommonWindows.ToggleWindow, InitJournalWindow)
+OpenJournalWindow = GUICommonWindows.CreateTopWinLoader(0, "GUIJRNL", GUICommonWindows.OpenWindowOnce, InitJournalWindow)
+
+def CloseAll(win):
+	win.Close ()
+	GUICommonWindows.CloseTopWindow ()
 
 ###################################################
 def OpenQuestsWindow ():
-	global JournalWindow, QuestsWindow, QuestsList, QuestDesc
+	global QuestsList, QuestDesc
 	
-	GemRB.HideGUI()
+	QuestsWindow = GemRB.LoadWindow (1, "GUIJRNL")
 	
-	if QuestsWindow:
-		if QuestsWindow:
-			QuestsWindow.Unload()
-		QuestsWindow = None
+	def OnJournalAssignedPress ():
+		global selected_quest_class
+
+		# Assigned Quests
+		Label = QuestsWindow.GetControl (0x10000005)
+		Label.SetText (38585)
+
+		selected_quest_class = 0
+		PopulateQuestsList ()
 		
-		GemRB.SetVar ("OtherWindow", JournalWindow.ID)
-		
-		GemRB.UnhideGUI()
-		return
-	
-	QuestsWindow = Window = GemRB.LoadWindow (1)
-	GemRB.SetVar ("OtherWindow", Window.ID)
-	
+	def OnJournalCompletedPress ():
+		global selected_quest_class
+
+		# Completed Quests
+		Label = QuestsWindow.GetControl (0x10000005)
+		Label.SetText (39527)
+
+		selected_quest_class = 1
+		PopulateQuestsList ()
+
 	# Assigned
-	Button = Window.GetControl (8)
+	Button = QuestsWindow.GetControl (8)
 	Button.SetText (39433)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnJournalAssignedPress)
+	Button.OnPress (OnJournalAssignedPress)
 
 	# Completed
-	Button = Window.GetControl (9)
+	Button = QuestsWindow.GetControl (9)
 	Button.SetText (39434)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnJournalCompletedPress)
+	Button.OnPress (OnJournalCompletedPress)
 
 	# Back
-	Button = Window.GetControl (5)
+	Button = QuestsWindow.GetControl (5)
 	Button.SetText (46677)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenQuestsWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-	Button.SetStatus (IE_GUI_CONTROL_FOCUSED)
+	Button.OnPress (QuestsWindow.Close)
+	Button.MakeEscape()
+	Button.Focus()
 
 	# Done
-	Button = Window.GetControl (0)
+	Button = QuestsWindow.GetControl (0)
 	Button.SetText (20636)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenJournalWindow)
+	Button.OnPress (lambda: CloseAll(QuestsWindow))
 
-	QuestsList = List = Window.GetControl (1)
-	List.SetVarAssoc ('SelectedQuest', -1)
-	List.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, OnJournalQuestSelect)
+	QuestsList = List = QuestsWindow.GetControl (1)
+	List.SetVarAssoc ('SelectedQuest', None)
+	List.OnSelect (OnJournalQuestSelect)
 
-	QuestDesc = Window.GetControl (3)
+	QuestDesc = QuestsWindow.GetControl (3)
 
 	EvaluateAllQuests ()
 	PopulateQuestsList ()
-
-	#QuestsWindow.SetVisible (WINDOW_VISIBLE)
-	GemRB.UnhideGUI()
 	
 
 def OnJournalQuestSelect ():
@@ -184,29 +151,8 @@ def OnJournalQuestSelect ():
 	q = quests[selected_quest_class][row]
 	QuestDesc.SetText (int (q[1])) 
 	
-def OnJournalAssignedPress ():
-	global selected_quest_class
-
-	# Assigned Quests
-	Label = QuestsWindow.GetControl (0x10000005)
-	Label.SetText (38585)
-
-	selected_quest_class = 0
-	PopulateQuestsList ()
-	
-def OnJournalCompletedPress ():
-	global selected_quest_class
-
-	# Completed Quests
-	Label = QuestsWindow.GetControl (0x10000005)
-	Label.SetText (39527)
-
-	selected_quest_class = 1
-	PopulateQuestsList ()
-
-
 def PopulateQuestsList ():
-	GemRB.SetVar ('SelectedQuest', -1)
+	GemRB.SetVar ('SelectedQuest', None)
 	QuestDesc.Clear ()
 	
 	lookup = lambda quest: int(GemRB.GetINIQuestsKey (str (quest[0]), 'title', '0'))
@@ -216,7 +162,6 @@ def PopulateQuestsList ():
 def EvaluateCondition (var, value, condition):
 	cur_value = int (GemRB.GetGameVar (var))
 
-	#print cur_value, condition, value
 	if condition == 'EQ':
 		return cur_value == int (value)
 	if condition == 'NE':
@@ -226,7 +171,7 @@ def EvaluateCondition (var, value, condition):
 	elif condition == 'LT':
 		return cur_value < int (value)
 	else:
-		print 'Unknown condition in quests.ini:', condition
+		print('Unknown condition in quests.ini:', condition)
 		return None
 
 def EvaluateQuest (index):
@@ -249,11 +194,9 @@ def EvaluateQuest (index):
 
 			completed = completed and EvaluateCondition (var, value, condition)
 
-			#print 'CC:', var, int (GemRB.GetGameVar (var)), condition, value, ': ', completed
 			if not completed: break
 
 		if completed:
-			#print "COMPLETED", suff
 			desc = GemRB.GetINIQuestsKey (tag, 'descCompleted' + suff, '0')
 			return (1, desc)
 
@@ -267,11 +210,9 @@ def EvaluateQuest (index):
 
 		assigned = assigned and EvaluateCondition (var, value, condition)
 
-		#print 'AC:', var, condition, value, ': ', assigned
 		if not assigned: break
 
 	if assigned:
-		#print "ASSIGNED"
 		desc = GemRB.GetINIQuestsKey (tag, 'descAssigned', '0')
 		return (0, desc)
 
@@ -292,59 +233,44 @@ def EvaluateAllQuests ():
 ###################################################
 
 def OpenBeastsWindow ():
-	global JournalWindow, BeastsWindow, BeastsList, BeastImage, BeastDesc
+	global BeastsList, BeastImage, BeastDesc
 	
-	GemRB.HideGUI()
-	
-	if BeastsWindow:
-		if BeastsWindow:
-			BeastsWindow.Unload()
-		BeastsWindow = None
-		
-		GemRB.SetVar ("OtherWindow", JournalWindow.ID)
-		
-		GemRB.UnhideGUI()
-		return
-	
-	BeastsWindow = Window = GemRB.LoadWindow (2)
-	GemRB.SetVar ("OtherWindow", BeastsWindow.ID)
+	BeastsWindow = GemRB.LoadWindow (2, "GUIJRNL")
 
 	# PC
 	Button = BeastsWindow.GetControl (5)
 	Button.SetText (20637)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnJournalPCPress)
+	Button.OnPress (OnJournalPCPress)
 
 	# NPC
 	Button = BeastsWindow.GetControl (6)
 	Button.SetText (20638)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnJournalNPCPress)
+	Button.OnPress (OnJournalNPCPress)
 
 	# Back
 	Button = BeastsWindow.GetControl (7)
 	Button.SetText (46677)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBeastsWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-	Button.SetStatus (IE_GUI_CONTROL_FOCUSED)
+	Button.OnPress (BeastsWindow.Close)
+	Button.MakeEscape()
+	Button.Focus()
 
 	# Done
 	Button = BeastsWindow.GetControl (4)
 	Button.SetText (20636)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenJournalWindow)
+	Button.OnPress (lambda: CloseAll(BeastsWindow))
 
-	BeastsList = List = Window.GetControl (0)
-	List.SetVarAssoc ('SelectedBeast', -1)
-	List.SetEvent(IE_GUI_TEXTAREA_ON_SELECT, OnJournalBeastSelect)
+	BeastsList = List = BeastsWindow.GetControl (0)
+	List.SetVarAssoc ('SelectedBeast', None)
+	List.OnSelect (OnJournalBeastSelect)
 
-	Window.CreateButton (8, 19, 19, 281, 441)
-	BeastImage = Window.GetControl (8)
+	BeastImage = BeastsWindow.CreateButton (8, 19, 19, 281, 441)
 	BeastImage.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 
-	BeastDesc = Window.GetControl (2)
+	BeastDesc = BeastsWindow.GetControl (2)
 	
 	EvaluateAllBeasts ()
 	PopulateBeastsList ()
 	
-	GemRB.UnhideGUI()
 	return
 
 def OnJournalBeastSelect ():
@@ -371,7 +297,7 @@ def OnJournalNPCPress ():
 
 
 def PopulateBeastsList ():
-	GemRB.SetVar ('SelectedBeast', -1)
+	GemRB.SetVar ('SelectedBeast', None)
 	BeastDesc.Clear ()
 	BeastImage.SetPicture ('default')
 
@@ -399,41 +325,27 @@ def EvaluateAllBeasts ():
 ###################################################
 
 def OpenLogWindow ():
-	global JournalWindow, LogWindow
-	
-	GemRB.HideGUI()
-	
-	if LogWindow:
-		if LogWindow:
-			LogWindow.Unload()
-		LogWindow = None
-		
-		GemRB.SetVar ("OtherWindow", JournalWindow.ID)
-		
-		GemRB.UnhideGUI()
-		return
-	
-	LogWindow = Window = GemRB.LoadWindow (3)
-	GemRB.SetVar ("OtherWindow", Window.ID)
+	LogWindow = GemRB.LoadWindow (3, "GUIJRNL")
 
 	# Back
-	Button = Window.GetControl (1)
+	Button = LogWindow.GetControl (1)
 	Button.SetText (46677)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLogWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-	Button.SetStatus (IE_GUI_CONTROL_FOCUSED)
+	Button.OnPress (LogWindow.Close)
+	Button.MakeEscape()
+	Button.Focus()
 
 	# Done
-	Button = Window.GetControl (0)
+	Button = LogWindow.GetControl (0)
 	Button.SetText (20636)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenJournalWindow)
+	Button.OnPress (lambda: CloseAll(LogWindow))
 
 	# text area
-	Text = Window.GetControl (2)
+	Text = LogWindow.GetControl (2)
 
 	# limit the log to the last entries (original did something similar)
 	js = GemRB.GetJournalSize (0)
 	frame = 250
+	journalText = ""
 	for i in range (js-frame, js):
 		je = GemRB.GetJournalEntry (0, i)
 
@@ -450,12 +362,11 @@ def OpenLogWindow ():
 		date = str (1 + dt)
 		#time = str (gt - dt*86400)
 		
-		Text.Append ("[color=FFFF00]" + GemRB.GetString(19310)+" "+date+":[/color]")
-		Text.Append (" " + GemRB.GetString (je['Text']) + "\n\n")
+		journalText += "[color=FFFF00]" + GemRB.GetString(19310)+" "+date+":[/color]"
+		journalText += " " + GemRB.GetString (je['Text']) + "\n\n"
 			
-	Window.SetVisible (WINDOW_VISIBLE)
+	Text.SetText (journalText)
+	LogWindow.Focus()
 	
-	GemRB.UnhideGUI()
-
 ###################################################
 # End of file GUIJRNL.py
